@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_asf.c,v 1.87 2002/12/08 21:43:50 miguelfreitas Exp $
+ * $Id: demux_asf.c,v 1.103 2003/01/29 18:53:54 miguelfreitas Exp $
  *
  * demultiplexer for asf streams
  *
@@ -42,10 +42,13 @@
 #include "xine_internal.h"
 #include "demux.h"
 #include "xineutils.h"
+#include "asfheader.h"
+#include "xmlparser.h"
 
 /*
 #define LOG
 */
+
 #define CODEC_TYPE_AUDIO       0
 #define CODEC_TYPE_VIDEO       1
 #define CODEC_TYPE_CONTROL     2
@@ -53,63 +56,10 @@
 
 #define DEFRAG_BUFSIZE    65536
 
-/*
- * define asf GUIDs (list from avifile)
- */
-#define GUID_ERROR                              0
-
-    /* base ASF objects */
-#define GUID_ASF_HEADER                         1
-#define GUID_ASF_DATA                           2
-#define GUID_ASF_SIMPLE_INDEX                   3
-
-    /* header ASF objects */
-#define GUID_ASF_FILE_PROPERTIES                4
-#define GUID_ASF_STREAM_PROPERTIES              5
-#define GUID_ASF_STREAM_BITRATE_PROPERTIES      6
-#define GUID_ASF_CONTENT_DESCRIPTION            7
-#define GUID_ASF_EXTENDED_CONTENT_ENCRYPTION    8
-#define GUID_ASF_SCRIPT_COMMAND                 9
-#define GUID_ASF_MARKER                        10
-#define GUID_ASF_HEADER_EXTENSION              11
-#define GUID_ASF_BITRATE_MUTUAL_EXCLUSION      12
-#define GUID_ASF_CODEC_LIST                    13
-#define GUID_ASF_EXTENDED_CONTENT_DESCRIPTION  14
-#define GUID_ASF_ERROR_CORRECTION              15
-#define GUID_ASF_PADDING                       16
-    
-    /* stream properties object stream type */
-#define GUID_ASF_AUDIO_MEDIA                   17
-#define GUID_ASF_VIDEO_MEDIA                   18
-#define GUID_ASF_COMMAND_MEDIA                 19
-
-    /* stream properties object error correction type */
-#define GUID_ASF_NO_ERROR_CORRECTION           20
-#define GUID_ASF_AUDIO_SPREAD                  21
-
-    /* mutual exclusion object exlusion type */
-#define GUID_ASF_MUTEX_BITRATE                 22
-#define GUID_ASF_MUTEX_UKNOWN                  23
-
-    /* header extension */
-#define GUID_ASF_RESERVED_1                    24
-    
-    /* script command */
-#define GUID_ASF_RESERVED_SCRIPT_COMMNAND      25
-
-    /* marker object */
-#define GUID_ASF_RESERVED_MARKER               26
-
-    /* various */
-/*
-#define GUID_ASF_HEAD2                         27
-*/
-#define GUID_ASF_AUDIO_CONCEAL_NONE            27
-#define GUID_ASF_CODEC_COMMENT1_HEADER         28
-#define GUID_ASF_2_0_HEADER                    29
-
-#define GUID_END                               30
+#define WRAP_THRESHOLD       5*90000
 
+#define PTS_AUDIO 0
+#define PTS_VIDEO 1
 
 typedef struct {
   int               num;
@@ -140,9 +90,10 @@ typedef struct demux_asf_s {
   int               keyframe_found;
 
   int               seqno;
-  int               packet_size;
-  int               packet_flags;
-
+  uint32_t          packet_size;
+  uint8_t           packet_flags;
+  uint32_t          data_size;
+  
   asf_stream_t      streams[MAX_NUM_STREAMS];
   uint32_t          bitrates[MAX_NUM_STREAMS];
   int               num_streams;
@@ -170,21 +121,22 @@ typedef struct demux_asf_s {
   /* packet filling */
   int               packet_size_left;
 
-  /* frame rate calculations */
+  /* frame rate calculations, discontinuity detection */
 
-  int64_t           last_video_pts;
+  int64_t           last_pts[2];
   int32_t           frame_duration;
+  int               send_newpts;
+  int64_t           last_frame_pts;
 
   /* only for reading */
-  int               packet_padsize;
+  uint32_t          packet_padsize;
   int               nb_frames;
-  int               segtype;
+  uint8_t           frame_flag;
+  uint8_t           segtype;
   int               frame;
 
   int               status;
 
-  int               send_discontinuity;
-
   /* byte reordering from audio streams */
   int               reorder_h;
   int               reorder_w;
@@ -196,6 +148,7 @@ typedef struct demux_asf_s {
   /* first packet position */
   int64_t           first_packet_pos;
 
+  int               reference_mode;
 } demux_asf_t ;
 
 typedef struct {
@@ -209,132 +162,6 @@ typedef struct {
 } demux_asf_class_t;
 
 
-typedef struct {
-  uint32_t v1;
-  uint16_t v2;
-  uint16_t v3;
-  uint8_t  v4[8];
-} GUID;
-
-static const struct
-{
-    const char* name;
-    const GUID  guid;
-} guids[] =
-{
-    { "error",
-    { 0x0,} },
-
-
-    /* base ASF objects */
-    { "header",
-    { 0x75b22630, 0x668e, 0x11cf, { 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c }} },
-
-    { "data",
-    { 0x75b22636, 0x668e, 0x11cf, { 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c }} },
-
-    { "simple index",
-    { 0x33000890, 0xe5b1, 0x11cf, { 0x89, 0xf4, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xcb }} },
-
-
-    /* header ASF objects */
-    { "file properties",
-    { 0x8cabdca1, 0xa947, 0x11cf, { 0x8e, 0xe4, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 }} },
-
-    { "stream header",
-    { 0xb7dc0791, 0xa9b7, 0x11cf, { 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 }} },
-
-    { "stream bitrate properties", /* (http://get.to/sdp) */
-    { 0x7bf875ce, 0x468d, 0x11d1, { 0x8d, 0x82, 0x00, 0x60, 0x97, 0xc9, 0xa2, 0xb2 }} },
-
-    { "content description",
-    { 0x75b22633, 0x668e, 0x11cf, { 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c }} },
-
-    { "extended content encryption",
-    { 0x298ae614, 0x2622, 0x4c17, { 0xb9, 0x35, 0xda, 0xe0, 0x7e, 0xe9, 0x28, 0x9c }} },
-
-    { "script command",
-    { 0x1efb1a30, 0x0b62, 0x11d0, { 0xa3, 0x9b, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 }} },
-
-    { "marker",
-    { 0xf487cd01, 0xa951, 0x11cf, { 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 }} },
-
-    { "header extension",
-    { 0x5fbf03b5, 0xa92e, 0x11cf, { 0x8e, 0xe3, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 }} },
-
-    { "bitrate mutual exclusion",
-    { 0xd6e229dc, 0x35da, 0x11d1, { 0x90, 0x34, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xbe }} },
-
-    { "codec list",
-    { 0x86d15240, 0x311d, 0x11d0, { 0xa3, 0xa4, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 }} },
-
-    { "extended content description",
-    { 0xd2d0a440, 0xe307, 0x11d2, { 0x97, 0xf0, 0x00, 0xa0, 0xc9, 0x5e, 0xa8, 0x50 }} },
-
-    { "error correction",
-    { 0x75b22635, 0x668e, 0x11cf, { 0xa6, 0xd9, 0x00, 0xaa, 0x00, 0x62, 0xce, 0x6c }} },
-
-    { "padding",
-    { 0x1806d474, 0xcadf, 0x4509, { 0xa4, 0xba, 0x9a, 0xab, 0xcb, 0x96, 0xaa, 0xe8 }} },
-
-
-    /* stream properties object stream type */
-    { "audio media",
-    { 0xf8699e40, 0x5b4d, 0x11cf, { 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b }} },
-
-    { "video media",
-    { 0xbc19efc0, 0x5b4d, 0x11cf, { 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b }} },
-
-    { "command media",
-    { 0x59dacfc0, 0x59e6, 0x11d0, { 0xa3, 0xac, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 }} },
-
-
-    /* stream properties object error correction */
-    { "no error correction",
-    { 0x20fb5700, 0x5b55, 0x11cf, { 0xa8, 0xfd, 0x00, 0x80, 0x5f, 0x5c, 0x44, 0x2b }} },
-
-    { "audio spread",
-    { 0xbfc3cd50, 0x618f, 0x11cf, { 0x8b, 0xb2, 0x00, 0xaa, 0x00, 0xb4, 0xe2, 0x20 }} },
-
-
-    /* mutual exclusion object exlusion type */
-    { "mutex bitrate",
-    { 0xd6e22a01, 0x35da, 0x11d1, { 0x90, 0x34, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xbe }} },
-
-    { "mutex unknown", 
-    { 0xd6e22a02, 0x35da, 0x11d1, { 0x90, 0x34, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xbe }} },
-
-
-    /* header extension */
-    { "reserved_1",
-    { 0xabd3d211, 0xa9ba, 0x11cf, { 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 }} },
-
-
-    /* script command */
-    { "reserved script command",
-    { 0x4B1ACBE3, 0x100B, 0x11D0, { 0xA3, 0x9B, 0x00, 0xA0, 0xC9, 0x03, 0x48, 0xF6 }} },
-
-    /* marker object */
-    { "reserved marker",
-    { 0x4CFEDB20, 0x75F6, 0x11CF, { 0x9C, 0x0F, 0x00, 0xA0, 0xC9, 0x03, 0x49, 0xCB }} },
-
-    /* various */
-    /* Already defined (reserved_1)
-    { "head2",
-    { 0xabd3d211, 0xa9ba, 0x11cf, { 0x8e, 0xe6, 0x00, 0xc0, 0x0c, 0x20, 0x53, 0x65 }} },
-    */
-    { "audio conceal none",
-    { 0x49f1a440, 0x4ece, 0x11d0, { 0xa3, 0xac, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 }} },
-
-    { "codec comment1 header",
-    { 0x86d15241, 0x311d, 0x11d0, { 0xa3, 0xa4, 0x00, 0xa0, 0xc9, 0x03, 0x48, 0xf6 }} },
-
-    { "asf 2.0 header",
-    { 0xd6e229d1, 0x35da, 0x11d1, { 0x90, 0x34, 0x00, 0xa0, 0xc9, 0x03, 0x49, 0xbe }} },
-
-};
-
-
 static uint8_t get_byte (demux_asf_t *this) {
 
   uint8_t buf;
@@ -467,10 +294,6 @@ static void asf_send_audio_header (demux_asf_t *this, int stream) {
 #ifdef LOG
   printf ("demux_asf: wavex header is %d bytes long\n", this->wavex_size);
 #endif
-  if ( !this->streams[stream].buf_type ) {
-    printf ("demux_asf: unknown audio type 0x%x\n", wavex->wFormatTag);
-    this->streams[stream].buf_type = BUF_CONTROL_NOP;
-  } 
 
   buf->size = this->wavex_size;
   buf->type = this->streams[stream].buf_type;
@@ -493,14 +316,6 @@ static void asf_send_video_header (demux_asf_t *this, int stream) {
 
   this->stream->stream_info[XINE_STREAM_INFO_VIDEO_FOURCC] = bih->biCompression;
 
-  if( !this->streams[stream].buf_type ) {
-    printf ("demux_asf: unknown video format %.4s\n",
-	    (char*)&bih->biCompression);
-
-    this->status = DEMUX_FINISHED;
-    return;
-  }
-
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
   buf->decoder_flags   = BUF_FLAG_HEADER;
   buf->decoder_info[1] = 3000; /* FIXME ? */
@@ -546,9 +361,9 @@ static int asf_read_header (demux_asf_t *this) {
 
           end_time =  get_le64 (this); 
       
-          this->length = get_le64(this) / 10000000; 
+          this->length = get_le64(this) / 10000; 
           if (this->length)
-            this->rate = this->input->get_length (this->input) / this->length;
+            this->rate = this->input->get_length (this->input) / (this->length / 1000);
           else
             this->rate = 0;
 
@@ -629,6 +444,10 @@ static int asf_read_header (demux_asf_t *this) {
 
             this->streams[this->num_streams].buf_type = 
               formattag_to_buf_audio ( wavex->wFormatTag );
+            if ( !this->streams[this->num_streams].buf_type ) {
+              printf ("demux_asf: unknown audio type 0x%x\n", wavex->wFormatTag);
+              this->streams[this->num_streams].buf_type = BUF_AUDIO_UNKNOWN;
+            } 
 
             this->streams[this->num_streams].fifo        = this->audio_fifo;
             this->streams[this->num_streams].stream_id   = stream_id;
@@ -662,6 +481,12 @@ static int asf_read_header (demux_asf_t *this) {
 
               this->streams[this->num_streams].buf_type = 
                 fourcc_to_buf_video(bih->biCompression);
+              if( !this->streams[this->num_streams].buf_type ) {
+                printf ("demux_asf: unknown video format %.4s\n",
+                        (char*)&bih->biCompression);
+
+                this->streams[this->num_streams].buf_type = BUF_VIDEO_UNKNOWN;
+              }
 
               this->streams[this->num_streams].fifo         = this->video_fifo;
               this->streams[this->num_streams].stream_id    = stream_id;
@@ -672,7 +497,8 @@ static int asf_read_header (demux_asf_t *this) {
               printf ("demux_asf: invalid bih_size received (%d), v_stream ignored.\n", i );
 
 #ifdef LOG
-            printf ("demux_asf: found a video stream id=%d \n", stream_id);
+            printf ("demux_asf: found a video stream id=%d, buf_type=%08x \n", 
+		    stream_id, this->streams[this->num_streams].buf_type);
 #endif
             this->num_video_streams++;
           }
@@ -777,97 +603,92 @@ static uint32_t asf_get_packet(demux_asf_t *this) {
 
   int64_t   timestamp;
   int       duration;
-  uint32_t  data_size;
-  uint8_t   ecc_flags = 0;
+  uint8_t   ecd_flags;
   uint8_t   buf[16];
-  uint32_t  p_hdr_size = 0;
-#ifdef LOG
-  int       i;
-#endif
+  uint32_t  p_hdr_size;
+  int       invalid_packet;
   
-  ecc_flags = get_byte(this); p_hdr_size += 1;
-  p_hdr_size += this->input->read (this->input, buf, ecc_flags & 15);
+  do {
+    ecd_flags = get_byte(this); p_hdr_size = 1;
+    invalid_packet = 0;
+  
+    /* skip ecd */
+    if (ecd_flags & 0x80)
+      p_hdr_size += this->input->read (this->input, buf, ecd_flags & 0x0F);
+    
+    /* skip invalid packet */
+    if (ecd_flags & 0x70) {
 #ifdef LOG
-  printf("ecc_flags: %d ", ecc_flags);
-  for (i = 0; i < (ecc_flags & 15); i++) 
-    printf(", %d", buf[i]);
-  printf("\n");
+      printf("demux_asf: skip invalid packet: %d\n", ecd_flags);
 #endif
+      this->input->seek (this->input, this->packet_size - p_hdr_size, SEEK_CUR);
+      invalid_packet = 1;
+    }  
 
+    if( this->status != DEMUX_OK )
+      return 0;
+
+  } while (invalid_packet);
+  
   if( this->status != DEMUX_OK )
     return 0;
   
   this->packet_flags = get_byte(this);  p_hdr_size += 1;
   this->segtype = get_byte(this);  p_hdr_size += 1;
 
-  /* Read packet size (plen): */
+  /* packet size */
   switch((this->packet_flags >> 5) & 3) {
     case 1:
-      data_size = get_byte(this); p_hdr_size += 1;
-      break;
+      this->data_size = get_byte(this); p_hdr_size += 1; break;
     case 2:
-      data_size = get_le16(this); p_hdr_size += 2;
-      break;
+      this->data_size = get_le16(this); p_hdr_size += 2; break;
     case 3:
-      data_size = get_le32(this); p_hdr_size += 4;
-      break;
+      this->data_size = get_le32(this); p_hdr_size += 4; break;
     default:
-      data_size = 0;
+      this->data_size = 0;
   }
-
-  /* Read sequence: */
+  
+  /* sequence */
   switch ((this->packet_flags >> 1) & 3) {
     case 1:
-      get_byte(this); p_hdr_size += 1;
-      break;
+      get_byte(this); p_hdr_size += 1; break;
     case 2:
-      get_le16(this); p_hdr_size += 2;
-      break;
+      get_le16(this); p_hdr_size += 2; break;
     case 3:
-      get_le32(this); p_hdr_size += 4;
-      break;
+      get_le32(this); p_hdr_size += 4; break;
   }
 
-  /* Read padding size */
+  /* padding size */
   switch ((this->packet_flags >> 3) & 3){
     case 1:
-      this->packet_padsize = get_byte(this); p_hdr_size += 1;
-      break;
+      this->packet_padsize = get_byte(this); p_hdr_size += 1; break;
     case 2:
-      this->packet_padsize = get_le16(this); p_hdr_size += 2;
-      break;
+      this->packet_padsize = get_le16(this); p_hdr_size += 2; break;
     case 3:
-      this->packet_padsize = get_le32(this); p_hdr_size += 4;
-      break;
+      this->packet_padsize = get_le32(this); p_hdr_size += 4; break;
     default:
       this->packet_padsize = 0;
   }
-    
+  
   timestamp = get_le32(this); p_hdr_size += 4;
   duration  = get_le16(this); p_hdr_size += 2;
-  
-  if (this->packet_flags & 0x01) {
-    this->nb_frames = get_byte(this) & 0x3F; p_hdr_size += 1;
-  } else {
-    this->nb_frames = 1;
-  }
-    
-  this->frame = 0;
-    
+
   if ((this->packet_flags >> 5) & 3) {
-    /* absolute packet size */
+    /* absolute data size */
 #ifdef LOG
-    printf ("demux_asf: absolute packet size\n");
+    printf ("demux_asf: absolute data size\n");
 #endif
-    this->packet_padsize = this->packet_size - data_size;
+    this->packet_padsize = this->packet_size - this->data_size; /* not used */
   } else {
-    /* relative packet size */
+    /* relative data size */
 #ifdef LOG
-    printf ("demux_asf: relative packet size\n");
+    printf ("demux_asf: relative data size\n");
 #endif
+    this->data_size = this->packet_size - this->packet_padsize;
   }
-  this->packet_size_left = this->packet_size - p_hdr_size;
   
+  /* this->packet_size_left = this->packet_size - p_hdr_size; */
+  this->packet_size_left = this->data_size - p_hdr_size;
 #ifdef LOG
   printf ("demux_asf: new packet, size = %d, size_left = %d, flags = 0x%02x, padsize = %d, this->packet_size = %d\n",
     data_size, this->packet_size_left, this->packet_flags, this->packet_padsize, this->packet_size);
@@ -885,17 +706,58 @@ static void hexdump (unsigned char *data, int len, xine_t *xine) {
 
 }
 
-static void asf_send_discontinuity (demux_asf_t *this, int64_t pts) {
+/* redefine abs as macro to handle 64-bit diffs.
+   i guess llabs may not be available everywhere */
+#define abs(x) ( ((x)<0) ? -(x) : (x) )
+
+static void check_newpts (demux_asf_t *this, int64_t pts, int video, int frame_end) {
+  int64_t diff;
+
+  diff = pts - this->last_pts[video];
+
+  if (pts && (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD)) ) {
 
 #ifdef LOG
-  printf ("demux_asf: send_discontinuity, pts %lld\n", pts);
+    printf ("demux_asf: sending newpts %lld (video = %d diff = %lld)\n", pts, video, diff);
 #endif
 
-  if (this->buf_flag_seek) {
-    xine_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
-    this->buf_flag_seek = 0;
-  } else {
-    xine_demux_control_newpts(this->stream, pts, 0);
+    if (this->buf_flag_seek) {
+      xine_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
+      this->buf_flag_seek = 0;
+    } else {
+      xine_demux_control_newpts(this->stream, pts, 0);
+    }
+
+    this->send_newpts = 0;
+    this->last_pts[1-video] = 0;
+  } 
+
+  if (pts) 
+    this->last_pts[video] = pts;
+
+  /*
+   * frame rate estimation
+   */
+
+  if (pts && video && frame_end) {
+
+    if (this->last_frame_pts) {
+
+      diff = pts - this->last_frame_pts;
+      
+      if ( (diff>0) && (diff < WRAP_THRESHOLD) ) {
+#ifdef LOG
+	printf ("demux_asf: last_frame_pts = %8lld, diff=%8lld\n",
+		this->last_frame_pts, diff);
+#endif
+	this->frame_duration = (15*this->frame_duration + diff) / 16;
+#ifdef LOG
+	printf ("demux_asf: frame_duration is %d\n", this->frame_duration);
+#endif
+      }
+    }
+    
+    this->last_frame_pts = pts;
   }
 }
 
@@ -906,7 +768,8 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
 				      int frag_len, int payload_size) {
 
   buf_element_t *buf;
-  int bufsize;
+  int            bufsize;
+  int            package_done;
 
   if (stream->frag_offset == 0) {
     /* new packet */
@@ -939,34 +802,21 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
     buf = stream->fifo->buffer_pool_alloc (stream->fifo);
     this->input->read (this->input, buf->content, bufsize);
 
-    if (stream->fifo == this->video_fifo) {
-      buf->input_pos  = this->input->get_current_pos (this->input);
-      if (this->rate)
-	buf->input_time = buf->input_pos / this->rate;
-      else
-	buf->input_time = 0;
-    } else {
-      buf->input_pos  = 0 ;
-      buf->input_time = 0 ;
-    }
+    buf->extra_info->input_pos  = this->input->get_current_pos (this->input);
+    if (this->rate)
+      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
+                                          * 1000 / this->rate);
+    else
+      buf->extra_info->input_time = 0;
 
 #ifdef LOG
     printf ("demux_asf: input pos is %lld, input time is %d\n",
-	    buf->input_pos,
-	    buf->input_time);
+	    buf->extra_info->input_pos,
+	    buf->extra_info->input_time);
 #endif
 
     buf->pts        = timestamp * 90;
 
-    if (buf->pts && this->send_discontinuity) {
-      if (this->keyframe_found) {
-        this->send_discontinuity = 0;
-        asf_send_discontinuity (this, buf->pts);
-      } else {
-        buf->pts = 0;
-      }
-    }
-
     buf->type       = stream->buf_type;
     buf->size       = bufsize;
     timestamp       = 0;
@@ -974,22 +824,25 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
     stream->frag_offset += bufsize;
     frag_len -= bufsize;
 
+    package_done = (stream->frag_offset == payload_size);
+
+    if ((buf->type & BUF_MAJOR_MASK) == BUF_VIDEO_BASE) 
+      check_newpts (this, buf->pts, PTS_VIDEO, package_done);
+    else
+      check_newpts (this, buf->pts, PTS_AUDIO, package_done);
+
     /* test if whole packet read */
-    if (stream->frag_offset == payload_size) {
+    if (package_done) {
 
       if ( (buf->type & BUF_MAJOR_MASK) == BUF_VIDEO_BASE) {
-	if (buf->pts && this->last_video_pts) {
-	  this->frame_duration = (3*this->frame_duration + (buf->pts - this->last_video_pts)) / 4;
-        }
-#ifdef LOG
-	printf ("demux_asf: frame_duration is %d\n", this->frame_duration);
-#endif
-	this->last_video_pts = buf->pts;
 
 	buf->decoder_flags   = BUF_FLAG_FRAME_END | BUF_FLAG_FRAMERATE;
 	buf->decoder_info[0] = this->frame_duration;
-      } else
+
+      } else {
+
 	buf->decoder_flags   = BUF_FLAG_FRAME_END;
+      }
 
       stream->frag_offset = 0;
 
@@ -998,6 +851,10 @@ static void asf_send_buffer_nodefrag (demux_asf_t *this, asf_stream_t *stream,
     if( !this->keyframe_found )
       buf->decoder_flags   |= BUF_FLAG_PREVIEW;
     
+#ifdef LOG
+    printf ("demux_asf: buffer type %08x %8d bytes, %8lld pts\n",
+	    buf->type, buf->size, buf->pts);
+#endif
     stream->fifo->put (stream->fifo, buf);
   }
 }
@@ -1043,30 +900,32 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
 	    buf = stream->fifo->buffer_pool_alloc (stream->fifo);
 	    xine_fast_memcpy (buf->content, p, bufsize);
 
-	    if (stream->fifo == this->video_fifo) {
-	      buf->input_pos  = this->input->get_current_pos (this->input);
-	      if (this->rate)
-		buf->input_time = buf->input_pos / this->rate;
-	      else
-		buf->input_time = 0;
-	    } else {
-	      buf->input_pos  = 0 ;
-	      buf->input_time = 0 ;
-	    }
+	    buf->extra_info->input_pos  = this->input->get_current_pos (this->input);
+	    if (this->rate)
+	      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
+                                                  * 1000 / this->rate);
+	    else
+	      buf->extra_info->input_time = 0;
           
 	    buf->pts = stream->timestamp * 90 + stream->ts_per_kbyte * 
 	      (p-stream->buffer) / 1024; 
 
-	    if (buf->pts && this->send_discontinuity) {
-	      buf->pts = 0;
-	    }
-	    
 	    buf->type       = stream->buf_type;
 	    buf->size       = bufsize;
 
+#ifdef LOG
+	    printf ("demux_asf: buffer type %08x %8d bytes, %8lld pts\n",
+		    buf->type, buf->size, buf->pts);
+#endif
+
 	    stream->frag_offset -= bufsize;
 	    p+=bufsize;
 
+	    if ((buf->type & BUF_MAJOR_MASK) == BUF_VIDEO_BASE) 
+	      check_newpts (this, buf->pts, PTS_VIDEO, !stream->frag_offset);
+	    else
+	      check_newpts (this, buf->pts, PTS_AUDIO, !stream->frag_offset);
+
 	    /* test if whole packet read */
 	    if ( !stream->frag_offset )
 	      buf->decoder_flags   |= BUF_FLAG_FRAME_END;
@@ -1110,14 +969,12 @@ static void asf_send_buffer_defrag (demux_asf_t *this, asf_stream_t *stream,
 
 static void asf_read_packet(demux_asf_t *this) {
 
-  uint8_t        raw_id,  stream_id;
+  uint8_t        raw_id, stream_id;
   uint32_t       seq, frag_offset, payload_size, frag_len, rlen;
   int            i;
   int64_t        timestamp;
   asf_stream_t  *stream;
-  
   uint32_t       s_hdr_size = 0;
-  
   uint64_t       current_pos;
   uint32_t       mod;
   uint32_t       psl;
@@ -1129,7 +986,7 @@ static void asf_read_packet(demux_asf_t *this) {
     this->packet_size_left = mod ? this->packet_size - mod : 0;
         
 #ifdef LOG
-    printf ("demux_asf: reading new packet, packet size left %d, %d\n", psl, this->packet_size_left);
+    printf ("demux_asf: reading new packet, packet size left psl=%d, pad=%d, %d\n", psl, this->packet_padsize, this->packet_size_left);
 #endif
 
     if (this->packet_size_left)
@@ -1141,16 +998,28 @@ static void asf_read_packet(demux_asf_t *this) {
       return ;
     }
     
-    if ((this->packet_padsize < 0) || (this->packet_padsize > this->packet_size)) {
+    if (this->packet_padsize > this->packet_size) {
       /* skip packet */
-      printf ("demux_asf: invalid padsize\n");
+      printf ("demux_asf: invalid padsize: %d\n", this->packet_padsize);
       this->frame = this->nb_frames - 1;
       return;
     }
+    
+    /* Multiple frames */
+    this->frame = 0;
+    if (this->packet_flags & 0x01) {
+      this->frame_flag = get_byte(this); s_hdr_size += 1;
+      this->nb_frames = this->frame_flag & 0x3F;
+#ifdef LOG
+      printf ("demux_asf: multiple frames %d\n", this->nb_frames);
+#endif
+    } else {
+      this->frame_flag = 0;
+      this->nb_frames = 1;
+    }
   }
 
   /* read segment header, find stream */
-
   raw_id     = get_byte(this); s_hdr_size += 1;
   stream_id  = raw_id & 0x7f;
 
@@ -1158,7 +1027,7 @@ static void asf_read_packet(demux_asf_t *this) {
 #ifdef LOG
   printf ("demux_asf: got raw_id =%d keyframe_found=%d\n", raw_id, this->keyframe_found);
 #endif
-  
+
   if ( (raw_id & 0x80) || this->keyframe_found || (this->num_video_streams==0)) {
     for (i = 0; i < this->num_streams; i++){
       if (this->streams[i].stream_id == stream_id &&
@@ -1178,40 +1047,30 @@ static void asf_read_packet(demux_asf_t *this) {
       return;
     }
 #endif
-
   }
   
   switch ((this->segtype >> 4) & 3){
   case 1:
-    seq = get_byte(this); s_hdr_size += 1;
-    break;
+    seq = get_byte(this); s_hdr_size += 1; break;
   case 2:
-    seq = get_le16(this); s_hdr_size += 2;
-    break;
+    seq = get_le16(this); s_hdr_size += 2; break;
   case 3:
-    seq = get_le32(this); s_hdr_size += 4;
-    break;
+    seq = get_le32(this); s_hdr_size += 4; break;
   default:
+    printf ("demux_asf: seq=0\n");
     seq = 0;
-    break;
   }
   
   switch ((this->segtype >> 2) & 3) {
     case 1:
-      frag_offset = get_byte(this); s_hdr_size += 1;
-      break;
+      frag_offset = get_byte(this); s_hdr_size += 1; break;
     case 2:
-      frag_offset = get_le16(this); s_hdr_size += 2;
-      break;
+      frag_offset = get_le16(this); s_hdr_size += 2; break;
     case 3:
-      frag_offset = get_le32(this); s_hdr_size += 4;
-      break;
+      frag_offset = get_le32(this); s_hdr_size += 4; break;
     default:
-      printf ("demux_asf: unknown segtype %x\n", this->segtype);
-      /* skip packet */
-      this->frame = this->nb_frames - 1;
-      return;
-      break;
+      printf ("demux_asf: frag_offset=0\n");
+      frag_offset = 0;
   }
 
   /* only set keyframe_found if we have it's beginning */
@@ -1220,19 +1079,15 @@ static void asf_read_packet(demux_asf_t *this) {
 
   switch (this->segtype & 3) {
     case 1:
-      rlen = get_byte(this); s_hdr_size += 1;
-      break;
+      rlen = get_byte(this); s_hdr_size += 1; break;
     case 2:
-      rlen = get_le16(this); s_hdr_size += 2;
-      break;
+      rlen = get_le16(this); s_hdr_size += 2; break;
     case 3:
-      rlen = get_le32(this); s_hdr_size += 4;
-      break;
+      rlen = get_le32(this); s_hdr_size += 4; break;
     default:
       rlen = 0;
-      break;
   }
-
+  
   if (rlen > this->packet_size_left) {
     /* skip packet */
     printf ("demux_asf: invalid rlen %d\n", rlen); 
@@ -1248,23 +1103,22 @@ static void asf_read_packet(demux_asf_t *this) {
   if (rlen == 1) {
     int data_length, data_sent=0;
 
+    /* multiple part segment */
     timestamp = frag_offset;
     get_byte (this); s_hdr_size += 1;
 
     if (this->packet_flags & 0x01) {
-      switch ((this->nb_frames >> 6) & 3) {
+      /* multiple frames */
+      switch ((this->frame_flag >> 6) & 3) {
         case 1:
-          data_length = get_byte(this); s_hdr_size += 1;
-          break;
+          data_length = get_byte(this); s_hdr_size += 1; break;
         case 2:
-          data_length = get_le16(this); s_hdr_size += 2;
-          break;
+          data_length = get_le16(this); s_hdr_size += 2; break;
         case 3:
-          data_length = get_le32(this); s_hdr_size += 4;
-          break;
+          data_length = get_le32(this); s_hdr_size += 4; break;
         default:
 #ifdef LOG
-          printf ("demux_asf: this->nb_frames is null\n"); 
+          printf ("demux_asf: invalid frame_flag %d\n", this->frame_flag); 
 #endif
           data_length = get_le16(this); s_hdr_size += 2;
       }
@@ -1274,10 +1128,9 @@ static void asf_read_packet(demux_asf_t *this) {
 #endif
 
     } else {
-
-      data_length = this->packet_size_left - s_hdr_size - this->packet_padsize; 
+      data_length = this->packet_size_left - s_hdr_size;
 #ifdef LOG
-        printf ("demux_asf: reading grouping single segment, size = %d\n", data_length); 
+      printf ("demux_asf: reading grouping single segment, size = %d\n", data_length); 
 #endif
     }
 
@@ -1323,6 +1176,7 @@ static void asf_read_packet(demux_asf_t *this) {
 
   } else {
 
+    /* single part segment */
     if (rlen >= 8) {
       payload_size  = get_le32(this); s_hdr_size += 4;
       timestamp     = get_le32(this); s_hdr_size += 4;
@@ -1337,16 +1191,26 @@ static void asf_read_packet(demux_asf_t *this) {
     }
     
     if (this->packet_flags & 0x01) {
-      if( (this->nb_frames & 0xc0) == 0x40 ) {
-        frag_len    = get_byte(this); s_hdr_size += 1;
-      } else {
-        frag_len      = get_le16(this); s_hdr_size += 2;
-      }        
+      switch ((this->frame_flag >> 6) & 3) {
+        case 1:
+          frag_len = get_byte(this); s_hdr_size += 1; break;
+        case 2:
+          frag_len = get_le16(this); s_hdr_size += 2; break;
+        case 3:
+          frag_len = get_le32(this); s_hdr_size += 4; break;
+        default:
+#ifdef LOG
+          printf ("demux_asf: invalid frame_flag %d\n", this->frame_flag); 
+#endif
+          frag_len = get_le16(this); s_hdr_size += 2;
+      }
+      
 #ifdef LOG
       printf ("demux_asf: reading part segment, size = %d\n", frag_len);
 #endif
     } else {
-      frag_len = this->packet_size_left - s_hdr_size - this->packet_padsize; 
+      frag_len = this->packet_size_left - s_hdr_size;
+
 #ifdef LOG
       printf ("demux_asf: reading single segment, size = %d\n", frag_len); 
 #endif
@@ -1354,7 +1218,7 @@ static void asf_read_packet(demux_asf_t *this) {
 
     if (frag_len > this->packet_size_left) {
       /* skip packet */
-      printf ("demux_asf: invalid rlen %d\n", rlen);
+      printf ("demux_asf: invalid frag_len %d\n", frag_len);
       this->frame = this->nb_frames - 1;
       return;
     }
@@ -1386,7 +1250,119 @@ static void asf_read_packet(demux_asf_t *this) {
     this->packet_size_left -= frag_len;
   }
 }
+
+/*
+ * parse .asx playlist files
+ */ 
+static int demux_asf_parse_references( demux_asf_t *this) {
+
+  char           *buf = NULL;
+  int             buf_size = 0;
+  int             buf_used = 0;
+  int             len;
+  xine_mrl_reference_data_t *data;
+  xine_event_t    uevent;
+  xml_node_t     *xml_tree, *asx_entry, *asx_ref;
+  xml_property_t *asx_prop;
+  int             result;
+
+
+  /* read file to memory. 
+   * warning: dumb code, but hopefuly ok since reference file is small */
+  do {
+    buf_size += 1024;
+    buf = realloc(buf, buf_size+1);
+    
+    len = this->input->read(this->input, &buf[buf_used], buf_size-buf_used);
+
+    if( len > 0 )
+      buf_used += len;
+      
+    /* 50k of reference file? no way. something must be wrong */
+    if( buf_used > 50*1024 )
+      break;
+  } while( len > 0 );
   
+  if(buf_used)
+    buf[buf_used] = '\0';
+
+  xml_parser_init(buf, buf_used, XML_PARSER_CASE_INSENSITIVE);
+  if((result = xml_parser_build_tree(&xml_tree)) != XML_PARSER_OK)
+    goto __failure;
+      
+  if(!strcasecmp(xml_tree->name, "ASX")) {
+
+    asx_prop = xml_tree->props;
+
+    while((asx_prop) && (strcasecmp(asx_prop->name, "VERSION")))
+      asx_prop = asx_prop->next;
+
+    if(asx_prop) {
+      int  version_major, version_minor = 0;
+
+      if((((sscanf(asx_prop->value, "%d.%d", &version_major, &version_minor)) == 2) ||
+          ((sscanf(asx_prop->value, "%d", &version_major)) == 1)) && 
+         ((version_major == 3) && (version_minor == 0))) {
+    
+        asx_entry = xml_tree->child;
+        while(asx_entry) {
+          if((!strcasecmp(asx_entry->name, "ENTRY")) ||
+             (!strcasecmp(asx_entry->name, "ENTRYREF"))) {
+            char *href   = NULL;
+
+            asx_ref = asx_entry->child;
+            while(asx_ref) {
+
+              if(!strcasecmp(asx_ref->name, "REF")) {
+
+                for(asx_prop = asx_ref->props; asx_prop; asx_prop = asx_prop->next) {
+
+                  if(!strcasecmp(asx_prop->name, "HREF")) {
+
+                    if(!href)
+                      href = asx_prop->value;
+                  }
+                  if(href)
+                    break;
+                }
+              }
+              asx_ref = asx_ref->next;
+            }
+
+            if(href && strlen(href)) {
+              uevent.type = XINE_EVENT_MRL_REFERENCE;
+              uevent.stream = this->stream;
+              uevent.data_length = strlen(href)+sizeof(xine_mrl_reference_data_t);
+              data = malloc(uevent.data_length);
+              uevent.data = data;
+              strcpy(data->mrl, href);
+              data->alternative = 0;
+              xine_event_send(this->stream, &uevent);
+            }
+            href = NULL;
+          }
+          asx_entry = asx_entry->next;
+        }
+      }
+      else
+        printf("demux_asf: Wrong ASX version: %s\n", asx_prop->value);
+
+    }
+    else
+      printf("demux_asf: Unable to find VERSION tag from ASX.\n");
+  }
+  else
+    printf("demux_asf: Unsupported XML type: '%s'.\n", xml_tree->name);
+      
+  xml_parser_free_tree(xml_tree);
+__failure:
+  free(buf);
+  
+  this->status = DEMUX_FINISHED;
+  return this->status;
+}
+
+
 /* 
  * xine specific functions start here
  */
@@ -1395,6 +1371,9 @@ static int demux_asf_send_chunk (demux_plugin_t *this_gen) {
 
   demux_asf_t *this = (demux_asf_t *) this_gen;
 
+  if(this->reference_mode)
+    return demux_asf_parse_references(this);
+
   asf_read_packet (this);
 
   return this->status;
@@ -1424,15 +1403,19 @@ static int demux_asf_get_status (demux_plugin_t *this_gen) {
 static void demux_asf_send_headers (demux_plugin_t *this_gen) {
 
   demux_asf_t *this = (demux_asf_t *) this_gen;
-  int      i;
-  int      stream_id;
-  uint32_t buf_type, max_vrate, max_arate, sum_rate;
-  uint32_t bitrate = 0;
+  int          i;
+  int          stream_id;
+  uint32_t     buf_type, max_vrate, max_arate;
+  uint32_t     bitrate = 0;
   
-  this->video_fifo  = this->stream->video_fifo;
-  this->audio_fifo  = this->stream->audio_fifo;
+  this->video_fifo     = this->stream->video_fifo;
+  this->audio_fifo     = this->stream->audio_fifo;
 
-  this->status = DEMUX_OK;
+  this->last_pts[0]    = 0;
+  this->last_pts[1]    = 0;
+  this->last_frame_pts = 0;
+
+  this->status         = DEMUX_OK;
 
   /* will get overridden later */
   this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 0;
@@ -1457,6 +1440,11 @@ static void demux_asf_send_headers (demux_plugin_t *this_gen) {
   if (this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE)
     this->input->seek (this->input, 0, SEEK_SET);
 
+  if (this->reference_mode) {
+    xine_demux_control_start(this->stream);
+    return;
+  }
+
   if (!asf_read_header (this)) {
 
     printf ("demux_asf: asf_read_header failed.\n");
@@ -1486,14 +1474,11 @@ static void demux_asf_send_headers (demux_plugin_t *this_gen) {
      */
     max_vrate = 0;
     max_arate = 0;
-    sum_rate = 0;
     for (i = 0; i < this->num_streams; i++) {
       buf_type   = (this->streams[i].buf_type & BUF_MAJOR_MASK);
       stream_id  = this->streams[i].stream_id;
       bitrate    = this->bitrates[stream_id];
       
-      sum_rate += bitrate;
-
       printf("demux_asf: stream: %d, bitrate %d bps\n", stream_id, bitrate);
       if ((buf_type == BUF_VIDEO_BASE) &&
 	  (bitrate > max_vrate || this->video_stream_id == 0)) {
@@ -1523,6 +1508,10 @@ static void demux_asf_send_headers (demux_plugin_t *this_gen) {
     
     asf_send_audio_header(this, this->audio_stream);
     asf_send_video_header(this, this->video_stream);
+#ifdef LOG
+    printf ("demux_asf: send header done\n", this->wavex_size);
+#endif
+  
   }
 
   this->frame = 0;
@@ -1542,8 +1531,7 @@ static int demux_asf_seek (demux_plugin_t *this_gen,
   /*
    * seek to start position
    */
-  this->send_discontinuity       = 1;
-  this->last_video_pts           = 0;
+  this->send_newpts              = 1;
   this->frame                    = 0;
   this->nb_frames                = 1;
   this->packet_size_left         = 0;
@@ -1575,6 +1563,7 @@ static int demux_asf_seek (demux_plugin_t *this_gen,
     this->buf_flag_seek = 0;
   } else {
     this->buf_flag_seek = 1;
+    xine_demux_flush_engine(this->stream);
   }
 
   return this->status;
@@ -1601,7 +1590,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
 				    input_plugin_t *input) {
   
   demux_asf_t *this;
-  uint8_t      buf[8192];
+  uint8_t      buf[4096];
   int          len;
 
   switch (stream->content_detection_method) {
@@ -1616,7 +1605,7 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
       if (input->get_capabilities (input) & INPUT_CAP_SEEKABLE) {
 
 	input->seek (input, 0, SEEK_SET);
-	if (input->read (input, buf, 8192) != 8192)
+	if ( (len=input->read (input, buf, 1024)) <= 0)
 	  return NULL;
 
 #ifdef LOG
@@ -1626,8 +1615,12 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
       } else
 	return NULL;
     }      
-    if (memcmp(buf, &guids[GUID_ASF_HEADER].guid, sizeof(GUID)))
-      return NULL;
+
+    if (memcmp(buf, &guids[GUID_ASF_HEADER].guid, sizeof(GUID))) {
+      buf[len] = '\0';
+      if( !strstr(buf,"asx") && !strstr(buf,"ASX") )
+        return NULL;
+    }
       
 #ifdef LOG
     printf ("demux_asf: file starts with an asf header\n");
@@ -1671,6 +1664,22 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this         = xine_xmalloc (sizeof (demux_asf_t));
   this->stream = stream;
   this->input  = input;
+  
+  /*
+   * check for reference stream
+   */ 
+  this->reference_mode = 0;
+  len = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW);
+  if ( (len == INPUT_OPTIONAL_UNSUPPORTED) &&
+       (input->get_capabilities (input) & INPUT_CAP_SEEKABLE) ) {
+    input->seek (input, 0, SEEK_SET);
+    len=input->read (input, buf, 1024);
+  }
+  if(len > 0) {
+    buf[len] = '\0';
+    if( strstr(buf,"asx") || strstr(buf,"ASX") )
+      this->reference_mode = 1;
+  }
 
   this->demux_plugin.send_headers      = demux_asf_send_headers;
   this->demux_plugin.send_chunk        = demux_asf_send_chunk;
@@ -1698,13 +1707,15 @@ static char *get_identifier (demux_class_t *this_gen) {
 }
 
 static char *get_extensions (demux_class_t *this_gen) {
-  return "asf wmv wma";
+  return "asf asx wmv wma";
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return "video/x-ms-asf: asf: ASF animation;"
-         "video/x-ms-wmv: wmv: WMV animation;"
-         "application/vnd.ms-asf: asf: ASF animation;";
+  return "video/x-ms-asf: asf: ASF video;"
+         "video/x-ms-wmv: wmv: WMV video;"
+         "application/vnd.ms-asf: asf: ASF video;"
+         "application/x-mplayer2: asf,asx,asp: mplayer2;"
+         "video/x-ms-asf-plugin: asf,asx,asp: mms animation;";
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -1739,6 +1750,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "asf", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_DEMUX, 20, "asf", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
