@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_avi.c,v 1.136 2002/12/08 21:43:50 miguelfreitas Exp $
+ * $Id: demux_avi.c,v 1.148 2003/01/23 16:12:00 miguelfreitas Exp $
  *
  * demultiplexer for avi streams
  *
@@ -183,7 +183,6 @@ typedef struct demux_avi_s {
   int                  status;
 
   int                  no_audio;
-  int                  have_spu;
 
   uint32_t             video_step;
   uint32_t             AVI_errno;
@@ -322,10 +321,14 @@ static int64_t get_audio_pts (demux_avi_t *this, int track, long posc,
       this->avi->audio[track]->dwRate_audio * 90000.0;
   } else {
     /* constant bitrate */
-
-    return (postot+posb)/
-      this->avi->audio[track]->dwSampleSize * (double) this->avi->audio[track]->dwScale_audio /
-      this->avi->audio[track]->dwRate_audio * 90000.0;
+    if( this->avi->audio[track]->wavex && this->avi->audio[track]->wavex->nBlockAlign )
+      return (postot+posb)/
+        this->avi->audio[track]->wavex->nBlockAlign * (double) this->avi->audio[track]->dwScale_audio /
+        this->avi->audio[track]->dwRate_audio * 90000.0;
+    else
+      return (postot+posb)/
+        this->avi->audio[track]->dwSampleSize * (double) this->avi->audio[track]->dwScale_audio /
+        this->avi->audio[track]->dwRate_audio * 90000.0;
   }
 }
 
@@ -811,6 +814,16 @@ static avi_t *AVI_init(demux_avi_t *this)  {
 
     for(i=0;i<AVI->n_idx;i++)
       if( strncasecmp(AVI->idx[i],AVI->video_tag,3)==0 ) break;
+    
+    /* try again for ##ix */  
+    if (i>=AVI->n_idx) {
+      AVI->video_tag[2] = 'i';
+      AVI->video_tag[3] = 'x';
+    }
+    
+    for(i=0;i<AVI->n_idx;i++)
+      if( strncasecmp(AVI->idx[i],AVI->video_tag,3)==0 ) break;
+    
     if (i>=AVI->n_idx) {
       ERR_EXIT(AVI_ERR_NO_VIDS);
     }
@@ -1075,8 +1088,8 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
         return 0;
       }
 
-      buf->input_pos  = 0;
-      buf->input_time = 0;
+      buf->extra_info->input_time = audio_pts / 90;
+      buf->extra_info->input_pos  = this->input->get_current_pos(this->input);
 
       buf->type = audio->audio_type | i;
 
@@ -1099,8 +1112,13 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
     buf->size       = AVI_read_video (this, this->avi, buf->mem, 2048, &buf->decoder_flags);
     buf->type       = this->avi->video_type;
 
-    buf->input_time = video_pts / 90000;
-    buf->input_pos  = this->input->get_current_pos(this->input);
+    buf->extra_info->input_time = video_pts / 90;
+    buf->extra_info->input_pos  = this->input->get_current_pos(this->input);
+    /* use video_frames-2 instead of video_frames-1 to fix problems with weird
+       non-interleaved streams */
+    buf->extra_info->input_length = 
+      this->avi->video_idx.vindex[this->avi->video_idx.video_frames - 2].pos;
+    buf->extra_info->frame_number = this->avi->video_posf; 
     buf->decoder_flags |= decoder_flags;
 
     if (buf->size<0) {
@@ -1114,23 +1132,6 @@ static int demux_avi_next (demux_avi_t *this, int decoder_flags) {
     */
 
     this->video_fifo->put (this->video_fifo, buf);
-
-    /*
-     * send packages to inform & drive text spu decoder
-     */
-
-    if (this->have_spu && (buf->decoder_flags & BUF_FLAG_FRAME_END)) {
-      buf_element_t *buf;
-      buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-
-      buf->decoder_flags = BUF_FLAG_FRAME_END;
-      buf->type          = BUF_SPU_TEXT;
-      buf->pts           = video_pts;
-
-      buf->decoder_info[1] = this->avi->video_posf;
-
-      this->video_fifo->put (this->video_fifo, buf);
-    }
   }
 
   if( buf ) {
@@ -1191,7 +1192,7 @@ static void demux_avi_send_headers (demux_plugin_t *this_gen) {
       printf ("demux_avi: unknown audio type 0x%x\n",
 	      this->avi->audio[i]->wavex->wFormatTag);
       this->no_audio  = 1;
-      this->avi->audio[i]->audio_type     = BUF_CONTROL_NOP;
+      this->avi->audio[i]->audio_type     = BUF_AUDIO_UNKNOWN;
     } else
       printf ("demux_avi: audio type %s (wFormatTag 0x%x)\n",
 	      buf_audio_name(this->avi->audio[i]->audio_type),
@@ -1228,71 +1229,43 @@ static void demux_avi_send_headers (demux_plugin_t *this_gen) {
 
       printf ("demux_avi: unknown video codec '%.4s'\n",
 	      (char*)&this->avi->bih.biCompression);
-      buf->free_buffer (buf);
-    
-      this->status = DEMUX_FINISHED;
+      this->avi->video_type = BUF_VIDEO_UNKNOWN;
+       
+    }
 
-    } else {
+    buf->type = this->avi->video_type;
+    printf ("demux_avi: video codec is '%s'\n",
+	    buf_video_name(buf->type));
 
-      unsigned char  *sub;
+    this->video_fifo->put (this->video_fifo, buf);
 
+    /* send off the palette, if there is one */
+    if (this->avi->palette_count) {
+      buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+      buf->decoder_flags = BUF_FLAG_SPECIAL;
+      buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
+      buf->decoder_info[2] = this->avi->palette_count;
+      buf->decoder_info_ptr[2] = &this->avi->palette;
+      buf->size = 0;
       buf->type = this->avi->video_type;
-      printf ("demux_avi: video codec is '%s'\n",
-	      buf_video_name(buf->type));
-
       this->video_fifo->put (this->video_fifo, buf);
+    }
 
-      /* send off the palette, if there is one */
-      if (this->avi->palette_count) {
-        buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-        buf->decoder_flags = BUF_FLAG_SPECIAL;
-        buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
-        buf->decoder_info[2] = this->avi->palette_count;
-        buf->decoder_info[3] = (unsigned int)&this->avi->palette;
-        buf->size = 0;
-        buf->type = this->avi->video_type;
-        this->video_fifo->put (this->video_fifo, buf);
+    if (this->audio_fifo) {
+      for (i=0; i<this->avi->n_audio; i++) {
+        avi_audio_t *a = this->avi->audio[i];
+
+        buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+        buf->decoder_flags = BUF_FLAG_HEADER;
+        memcpy (buf->content, a->wavex, a->wavex_len);
+        buf->size = a->wavex_len;
+        buf->type = a->audio_type | i;
+        buf->decoder_info[0] = 0; /* first package, containing wavex */
+        buf->decoder_info[1] = a->wavex->nSamplesPerSec; /* Audio Rate */
+        buf->decoder_info[2] = a->wavex->wBitsPerSample; /* Audio bits */
+        buf->decoder_info[3] = a->wavex->nChannels; /* Audio bits */
+        this->audio_fifo->put (this->audio_fifo, buf);
       }
-
-      if (this->audio_fifo) {
-        for (i=0; i<this->avi->n_audio; i++) {
-          avi_audio_t *a = this->avi->audio[i];
-
-          buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-          buf->decoder_flags = BUF_FLAG_HEADER;
-          memcpy (buf->content, a->wavex, a->wavex_len);
-          buf->size = a->wavex_len;
-          buf->type = a->audio_type | i;
-          buf->decoder_info[0] = 0; /* first package, containing wavex */
-          buf->decoder_info[1] = a->wavex->nSamplesPerSec; /* Audio Rate */
-          buf->decoder_info[2] = a->wavex->wBitsPerSample; /* Audio bits */
-          buf->decoder_info[3] = a->wavex->nChannels; /* Audio bits */
-          this->audio_fifo->put (this->audio_fifo, buf);
-        }
-      }
-
-      /*
-       * send external spu file pointer, if present
-       */
-
-      if (this->input->get_optional_data (this->input, &sub, INPUT_OPTIONAL_DATA_TEXTSPU0)) {
-        buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-        buf->content = sub;
-
-        buf->type = BUF_SPU_TEXT;
-
-        buf->decoder_flags   = BUF_FLAG_HEADER;
-        buf->decoder_info[1] = this->avi->width;
-        buf->decoder_info[2] = this->avi->height;
-
-        this->video_fifo->put (this->video_fifo, buf);
-
-        this->have_spu = 1;
-
-        printf ("demux_avi: text subtitle file available\n");
-
-      } else
-        this->have_spu = 0;
     }
 
     /*
@@ -1463,7 +1436,7 @@ static int demux_avi_get_stream_length (demux_plugin_t *this_gen) {
   demux_avi_t *this = (demux_avi_t *) this_gen;
 
   if (this->avi) {
-    return get_video_pts(this, this->avi->video_idx.video_frames) / 90000 ;
+    return (int)(get_video_pts(this, this->avi->video_idx.video_frames) / 90);
   }
 
   return 0;
@@ -1590,8 +1563,8 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return "video/msvideo: avi: AVI animation;"
-         "video/x-msvideo: avi: AVI animation;";
+  return "video/msvideo: avi: AVI video;"
+         "video/x-msvideo: avi: AVI video;";
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -1625,6 +1598,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "avi", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_DEMUX, 20, "avi", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
