@@ -22,7 +22,7 @@
  * avoid while programming a FLI decoder, visit:
  *   http://www.pcisys.net/~melanson/codecs/
  *
- * $Id: demux_fli.c,v 1.29 2002/12/08 21:43:50 miguelfreitas Exp $
+ * $Id: demux_fli.c,v 1.37 2003/01/23 16:12:02 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -56,6 +56,7 @@ typedef struct {
   config_values_t     *config;
 
   fifo_buffer_t       *video_fifo;
+  fifo_buffer_t       *audio_fifo;
 
   input_plugin_t      *input;
 
@@ -88,7 +89,7 @@ typedef struct {
   config_values_t  *config;
 } demux_fli_class_t;
 
-/* returns 1 if the CIN file was opened successfully, 0 otherwise */
+/* returns 1 if the FLI file was opened successfully, 0 otherwise */
 static int open_fli_file(demux_fli_t *this) {
 
   /* read the whole header */
@@ -131,6 +132,11 @@ static int open_fli_file(demux_fli_t *this) {
      this->frame_pts_inc = this->speed * 90;
   }
 
+  /* sanity check: the FLI file must have non-zero values for width, height,
+   * and frame count */
+  if ((!this->width) || (!this->height) || (!this->frame_count))
+    return 0;
+
   return 1;
 }
 
@@ -138,19 +144,12 @@ static int demux_fli_send_chunk(demux_plugin_t *this_gen) {
 
   demux_fli_t *this = (demux_fli_t *) this_gen;
   buf_element_t *buf = NULL;
-  int i = 0;
   unsigned char fli_buf[6];
   unsigned int chunk_size;
   unsigned int chunk_magic;
   int64_t pts_counter = 0;
   off_t current_file_pos;
 
-  /* check if all the frames have been sent */
-  if (i >= this->frame_count) {
-    this->status = DEMUX_FINISHED;
-    return this->status;
-  }
-  
   current_file_pos = this->input->get_current_pos(this->input);
   
   /* get the chunk size nd magic number */
@@ -169,9 +168,9 @@ static int demux_fli_send_chunk(demux_plugin_t *this_gen) {
     while (chunk_size) {
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
       buf->type = BUF_VIDEO_FLI;
-      buf->input_pos = current_file_pos;
-      buf->input_time = pts_counter / 90000;
-      buf->input_length = this->stream_len;
+      buf->extra_info->input_pos = current_file_pos;
+      buf->extra_info->input_time = pts_counter / 90;
+      buf->extra_info->input_length = this->stream_len;
       buf->pts = pts_counter;
   
       if (chunk_size > buf->max_size)
@@ -182,6 +181,7 @@ static int demux_fli_send_chunk(demux_plugin_t *this_gen) {
   
       if (this->input->read(this->input, buf->content, buf->size) !=
         buf->size) {
+        buf->free_buffer(buf);
         this->status = DEMUX_FINISHED;
         break;
       }
@@ -194,8 +194,6 @@ static int demux_fli_send_chunk(demux_plugin_t *this_gen) {
   } else
     this->input->seek(this->input, chunk_size, SEEK_CUR);
   
-  i++;
-  
   return this->status;
 }
 
@@ -204,8 +202,8 @@ static void demux_fli_send_headers(demux_plugin_t *this_gen) {
   demux_fli_t *this = (demux_fli_t *) this_gen;
   buf_element_t *buf;
 
-  this->video_fifo  = this->stream->video_fifo;
-  /* video-only format, don't worry about audio_fifo */
+  this->video_fifo = this->stream->video_fifo;
+  this->audio_fifo = this->stream->audio_fifo;
 
   this->status = DEMUX_OK;
 
@@ -231,7 +229,7 @@ static void demux_fli_send_headers(demux_plugin_t *this_gen) {
 }
 
 static int demux_fli_seek (demux_plugin_t *this_gen,
-                             off_t start_pos, int start_time) {
+                           off_t start_pos, int start_time) {
 
   demux_fli_t *this = (demux_fli_t *) this_gen;
 
@@ -351,19 +349,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   strncpy (this->last_mrl, input->get_mrl (input), 1024);
 
-  /* print vital stats */
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_fli: FLI type: %04X, speed: %d/%d\n"),
-    this->magic_number, this->speed,
-    (this->magic_number == FLI_FILE_MAGIC_1) ? 70 : 1000);
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_fli: %d frames, %dx%d\n"), 
-    this->frame_count, this->width, this->height);
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_fli: running time: %d min, %d sec\n"), 
-    this->frame_count * this->frame_pts_inc / 90000 / 60,
-    this->frame_count * this->frame_pts_inc / 90000 % 60);
-
   return &this->demux_plugin;
 }
 
@@ -380,7 +365,7 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
+  return "video/x-flic: fli,flc: Autodesk FLIC files;";
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -414,6 +399,6 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "fli", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "fli", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
