@@ -63,7 +63,7 @@
  *     - if any bytes exceed 63, do not shift the bytes at all before
  *       transmitting them to the video decoder
  *
- * $Id: demux_idcin.c,v 1.30 2002/12/08 21:43:51 miguelfreitas Exp $
+ * $Id: demux_idcin.c,v 1.37 2003/01/25 03:04:15 tmmm Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -112,8 +112,10 @@ typedef struct {
 
   int                  audio_chunk_size1;
   int                  audio_chunk_size2;
+  int                  current_audio_chunk;
 
   unsigned char        huffman_table[HUFFMAN_TABLE_SIZE];
+  uint64_t             pts_counter;
 
   char                 last_mrl[1024];
 } demux_idcin_t;
@@ -128,6 +130,16 @@ typedef struct {
   config_values_t  *config;
 } demux_idcin_class_t;
 
+/* set DEBUG_IDCIN to output information about the A/V chunks that the
+ * demuxer is dispatching to the engine */
+#define DEBUG_IDCIN 0
+
+#if DEBUG_IDCIN
+#define debug_idcin printf
+#else
+static inline void debug_idcin(const char *format, ...) { }
+#endif
+
 static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
 
   demux_idcin_t *this = (demux_idcin_t *) this_gen;
@@ -138,8 +150,6 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
   palette_entry_t palette[PALETTE_SIZE];
   int i;
   unsigned int remaining_sample_bytes;
-  uint64_t pts_counter = 0;
-  int current_audio_chunk = 1;
   int scale_bits;
 
   /* figure out what the next data is */
@@ -149,11 +159,15 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
   }
 
   command = le2me_32(command);
+  debug_idcin("  demux_idcin: command %d: ", command);
   if (command == 2) {
+    debug_idcin("demux finished\n");
     this->status = DEMUX_FINISHED;
     return this->status;
   } else {
     if (command == 1) {
+      debug_idcin("load palette\n");
+
       /* load a 768-byte palette and pass it to the demuxer */
       if (this->input->read(this->input, disk_palette, PALETTE_SIZE * 3) !=
         PALETTE_SIZE * 3) {
@@ -183,11 +197,12 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       buf->decoder_flags = BUF_FLAG_SPECIAL;
       buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
       buf->decoder_info[2] = PALETTE_SIZE;
-      buf->decoder_info[3] = (unsigned int)&palette;
+      buf->decoder_info_ptr[2] = &palette;
       buf->size = 0;
       buf->type = BUF_VIDEO_IDCIN;
       this->video_fifo->put (this->video_fifo, buf);
-    }
+    } else
+      debug_idcin("load video and audio\n");
   }
 
   /* load the video frame */
@@ -197,13 +212,15 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
   }
   remaining_sample_bytes = LE_32(&preamble[0]) - 4;
 
+  debug_idcin("  demux_idcin: dispatching %d video bytes\n",
+    remaining_sample_bytes);
   while (remaining_sample_bytes) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
     buf->type = BUF_VIDEO_IDCIN;
-    buf->input_pos = this->input->get_current_pos(this->input);
-    buf->input_length = this->filesize;
-    buf->input_time = pts_counter / 90000;
-    buf->pts = pts_counter;
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_length = this->filesize;
+    buf->extra_info->input_time = this->pts_counter / 90;
+    buf->pts = this->pts_counter;
 
     if (remaining_sample_bytes > buf->max_size)
       buf->size = buf->max_size;
@@ -223,27 +240,31 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
     if (!remaining_sample_bytes)
       buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
+    debug_idcin("    sending video buf with %d bytes, %lld pts\n",
+        buf->size, buf->pts);
     this->video_fifo->put(this->video_fifo, buf);
   }
 
   /* load the audio frame */
   if (this->audio_fifo && this->audio_sample_rate) {
 
-    if (current_audio_chunk == 1) {
+    if (this->current_audio_chunk == 1) {
       remaining_sample_bytes = this->audio_chunk_size1;
-      current_audio_chunk = 2;
+      this->current_audio_chunk = 2;
     } else {
       remaining_sample_bytes = this->audio_chunk_size2;
-      current_audio_chunk = 1;
+      this->current_audio_chunk = 1;
     }
 
+    debug_idcin("  demux_idcin: dispatching %d audio bytes\n",
+      remaining_sample_bytes);
     while (remaining_sample_bytes) {
       buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
       buf->type = BUF_AUDIO_LPCM_LE;
-      buf->input_pos = this->input->get_current_pos(this->input);
-      buf->input_length = this->filesize;
-      buf->input_time = pts_counter / 90000;
-      buf->pts = pts_counter;
+      buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+      buf->extra_info->input_length = this->filesize;
+      buf->extra_info->input_time = this->pts_counter / 90;
+      buf->pts = this->pts_counter;
 
       if (remaining_sample_bytes > buf->max_size)
         buf->size = buf->max_size;
@@ -261,11 +282,15 @@ static int demux_idcin_send_chunk(demux_plugin_t *this_gen) {
       if (!remaining_sample_bytes)
         buf->decoder_flags |= BUF_FLAG_FRAME_END;
 
+      debug_idcin("    sending audio buf with %d bytes, %lld pts\n",
+          buf->size, buf->pts);
       this->audio_fifo->put(this->audio_fifo, buf);
     }
   }
 
-  pts_counter += IDCIN_FRAME_PTS_INC;
+  debug_idcin("\n");
+
+  this->pts_counter += IDCIN_FRAME_PTS_INC;
 
   return this->status;
 }
@@ -319,6 +344,11 @@ static int open_idcin_file(demux_idcin_t *this) {
 
   /* if execution got this far, qualify it as a valid Id CIN file 
    * and continue loading */
+  debug_idcin("  demux_idcin: %dx%d video, %d Hz, %d channels, %d bits PCM audio\n",
+    this->video_width, this->video_height,
+    this->audio_sample_rate, 
+    this->audio_channels,
+    this->audio_bytes_per_sample * 8);
 
   /* read the Huffman table */
   if (this->input->read(this->input, this->huffman_table,
@@ -375,7 +405,8 @@ static void demux_idcin_send_headers(demux_plugin_t *this_gen) {
   buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
   buf->decoder_flags = BUF_FLAG_SPECIAL;
   buf->decoder_info[1] = BUF_SPECIAL_IDCIN_HUFFMAN_TABLE;
-  buf->decoder_info[2] = (unsigned int)&this->huffman_table;
+  buf->decoder_info[2] = sizeof(this->huffman_table);
+  buf->decoder_info_ptr[2] = &this->huffman_table;
   buf->size = 0;
   buf->type = BUF_VIDEO_IDCIN;
   this->video_fifo->put (this->video_fifo, buf);
@@ -393,6 +424,8 @@ static void demux_idcin_send_headers(demux_plugin_t *this_gen) {
         (this->audio_sample_rate / 14) * this->audio_bytes_per_sample *
         this->audio_channels;
     }
+    debug_idcin("  demux_idcin: audio_chunk_size[1,2] = %d, %d\n",
+      this->audio_chunk_size1, this->audio_chunk_size2);
 
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = BUF_AUDIO_LPCM_LE;
@@ -421,6 +454,8 @@ static int demux_idcin_seek (demux_plugin_t *this_gen,
     /* reposition stream past the Huffman tables */
     this->input->seek(this->input, 0x14 + 0x10000, SEEK_SET);
 
+    this->pts_counter = 0;
+    this->current_audio_chunk = 1;
   }
 
   return this->status;
@@ -525,18 +560,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   strncpy (this->last_mrl, input->get_mrl (input), 1024);
 
-  /* print vital stats */
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_idcin: Id CIN file, video is %dx%d, 14 frames/sec\n"),
-    this->video_width,
-    this->video_height);
-  if (this->audio_channels)
-    xine_log (this->stream->xine, XINE_LOG_MSG,
-      _("demux_idcin: %d-bit, %d Hz %s PCM audio\n"),
-      this->audio_bytes_per_sample * 8,
-      this->audio_sample_rate,
-      (this->audio_channels == 1) ? "monaural" : "stereo");
-
   return &this->demux_plugin;
 }
 
@@ -564,7 +587,7 @@ static void class_dispose (demux_class_t *this_gen) {
   free (this);
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+void *demux_idcin_init_plugin (xine_t *xine, void *data) {
 
   demux_idcin_class_t     *this;
 
@@ -586,8 +609,10 @@ static void *init_plugin (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
+#if 0
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "idcin", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "idcin", XINE_VERSION_CODE, NULL, demux_idcin_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
+#endif
