@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpeg.c,v 1.99 2002/12/08 21:43:51 miguelfreitas Exp $
+ * $Id: demux_mpeg.c,v 1.105 2003/01/16 22:25:53 miguelfreitas Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  * reads streams of variable blocksizes
@@ -307,10 +307,9 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
 
-    buf->input_pos = this->input->get_current_pos (this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos (this->input);
 
-    if(this->audio_fifo)
-      this->audio_fifo->put (this->audio_fifo, buf);
+    this->audio_fifo->put (this->audio_fifo, buf);
 
   } else if ((stream_id & 0xe0) == 0xc0) {
     int track = stream_id & 0x1f;
@@ -357,7 +356,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
     
-    buf->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
 
     if(this->audio_fifo)
       this->audio_fifo->put (this->audio_fifo, buf);
@@ -402,7 +401,7 @@ static void parse_mpeg2_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
     
-    buf->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
 
     this->video_fifo->put (this->video_fifo, buf);
 
@@ -527,9 +526,10 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
     
-    buf->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
     if (this->rate)
-      buf->input_time = buf->input_pos / (this->rate * 50);
+      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
+                                          * 1000 / (this->rate * 50));
 
     if(this->audio_fifo)
       this->audio_fifo->put (this->audio_fifo, buf);
@@ -550,9 +550,10 @@ static void parse_mpeg1_packet (demux_mpeg_t *this, int stream_id, int64_t scr)
     if (this->preview_mode)
       buf->decoder_flags = BUF_FLAG_PREVIEW;
     
-    buf->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
     if (this->rate)
-      buf->input_time = buf->input_pos / (this->rate * 50);
+      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
+                                          * 1000 / (this->rate * 50));
 
     this->video_fifo->put (this->video_fifo, buf);
 
@@ -637,33 +638,6 @@ static uint32_t parse_pack(demux_mpeg_t *this) {
        buf = read_bytes (this, 3) ;
   }
 
-  /* discontinuity ? */
-#if 0
-  /* scr-wrap detection disabled due bad streams */
-  if( scr && !this->preview_mode )
-  {
-    int64_t scr_diff = scr - this->last_scr;
-
-    if (abs(scr_diff) > 60000 && !this->send_newpts) {
-
-      buf_element_t *buf;
-
-      buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-      buf->type = BUF_CONTROL_DISCONTINUITY;
-      buf->disc_off = scr_diff;
-      this->video_fifo->put (this->video_fifo, buf);
-
-      if (this->audio_fifo) {
-        buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-        buf->type = BUF_CONTROL_DISCONTINUITY;
-        buf->disc_off = scr_diff;
-        this->audio_fifo->put (this->audio_fifo, buf);
-      }
-    }
-    this->last_scr = scr;
-  }
-#endif
-
   /* system header */
 
   buf = read_bytes (this, 4) ;
@@ -875,7 +849,8 @@ static int demux_mpeg_get_stream_length (demux_plugin_t *this_gen) {
   demux_mpeg_t *this = (demux_mpeg_t *) this_gen;
 
   if (this->rate)
-    return this->input->get_length (this->input) / (this->rate * 50);
+    return (int)((int64_t) 1000 * this->input->get_length (this->input) / 
+                 (this->rate * 50));
   else
     return 0;
 
@@ -1034,9 +1009,11 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
       /* disregard the RIFF file if it is certainly a better known
        * format like AVI or WAVE */
       if ((fourcc_tag == WAVE_TAG) ||
-	  (fourcc_tag == AVI_TAG))
+	  (fourcc_tag == AVI_TAG)) {
+	free (this);
 	return DEMUX_CANNOT_HANDLE;
-	
+      }
+
       /* Iterate through first n kilobytes of RIFF file searching for
        * MPEG video marker. No, it's not a very efficient approach, but
        * if execution has reached this special case, this is currently
@@ -1139,6 +1116,6 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "mpeg", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "mpeg", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
