@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_mpeg_block.c,v 1.147 2002/12/08 21:43:51 miguelfreitas Exp $
+ * $Id: demux_mpeg_block.c,v 1.158 2003/01/26 15:56:21 tmmm Exp $
  *
  * demultiplexer for mpeg 1/2 program streams
  *
@@ -30,7 +30,6 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <sched.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -186,10 +185,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 
       cbuf->type = buf->type;
       cbuf->decoder_flags = buf->decoder_flags;
-      cbuf->decoder_info[0] = buf->decoder_info[0];
-      cbuf->decoder_info[1] = buf->decoder_info[1];
-      cbuf->decoder_info[2] = buf->decoder_info[2];
-      cbuf->decoder_info[3] = buf->decoder_info[3];
+      memcpy( cbuf->decoder_info, buf->decoder_info, sizeof(cbuf->decoder_info) );
+      memcpy( cbuf->decoder_info_ptr, buf->decoder_info_ptr, sizeof(cbuf->decoder_info_ptr) );
 
       this->audio_fifo->put (this->audio_fifo, cbuf);
     }
@@ -207,11 +204,12 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
   else
     buf->decoder_flags = 0;
     
-  buf->input_pos = this->input->get_current_pos (this->input);
-  buf->input_length = this->input->get_length (this->input);
+  buf->extra_info->input_pos = this->input->get_current_pos (this->input);
+  buf->extra_info->input_length = this->input->get_length (this->input);
 
   if (this->rate)
-    buf->input_time = buf->input_pos / (this->rate * 50);
+    buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
+                                        * 1000 / (this->rate * 50));
 
   if (p[3] == 0xBA) { /* program stream pack header */
 
@@ -238,7 +236,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
 	this->rate |= (p[11] >> 1);
       }
 
-      buf->input_time = buf->input_pos / (this->rate * 50);
+      buf->extra_info->input_time = (int)((int64_t)buf->extra_info->input_pos 
+                                          * 1000 / (this->rate * 50));
 
       p   += 12;
 
@@ -350,8 +349,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     buf->decoder_info[1] = BUF_SPECIAL_SPU_DVD_SUBTYPE;
     buf->decoder_info[2] = SPU_DVD_SUBTYPE_NAV;
     buf->pts       = 0;   /* NAV packets do not have PES values */
-    buf->input_pos = this->input->get_current_pos(this->input);
-    buf->input_length = this->input->get_length (this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_length = this->input->get_length (this->input);
     this->video_fifo->put (this->video_fifo, buf);
 
     return ;
@@ -471,8 +470,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       if( !preview_mode )
         check_newpts( this, pts, PTS_VIDEO );
       
-      buf->input_pos = this->input->get_current_pos(this->input);
-      buf->input_length = this->input->get_length (this->input);
+      buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+      buf->extra_info->input_length = this->input->get_length (this->input);
       
       this->video_fifo->put (this->video_fifo, buf);    
 #ifdef LOG
@@ -499,8 +498,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       if( !preview_mode )
         check_newpts( this, pts, PTS_AUDIO );
 
-      buf->input_pos = this->input->get_current_pos(this->input);
-      buf->input_length = this->input->get_length (this->input);
+      buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+      buf->extra_info->input_length = this->input->get_length (this->input);
 
       if(this->audio_fifo) {
 	this->audio_fifo->put (this->audio_fifo, buf);
@@ -566,8 +565,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
       if( !preview_mode )
         check_newpts( this, pts, PTS_AUDIO );
 
-      buf->input_pos = this->input->get_current_pos(this->input);
-      buf->input_length = this->input->get_length (this->input);
+      buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+      buf->extra_info->input_length = this->input->get_length (this->input);
 
       if(this->audio_fifo) {
 	this->audio_fifo->put (this->audio_fifo, buf);
@@ -589,8 +588,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     if( !preview_mode )
       check_newpts( this, pts, PTS_VIDEO );
 
-    buf->input_pos = this->input->get_current_pos(this->input);
-    buf->input_length = this->input->get_length (this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_length = this->input->get_length (this->input);
 
     this->video_fifo->put (this->video_fifo, buf);
 #ifdef LOG
@@ -611,8 +610,8 @@ static void demux_mpeg_block_parse_pack (demux_mpeg_block_t *this, int preview_m
     if( !preview_mode )
         check_newpts( this, pts, PTS_AUDIO );
 
-    buf->input_pos = this->input->get_current_pos(this->input);
-    buf->input_length = this->input->get_length (this->input);
+    buf->extra_info->input_pos = this->input->get_current_pos(this->input);
+    buf->extra_info->input_length = this->input->get_length (this->input);
 
     if(this->audio_fifo) {
       this->audio_fifo->put (this->audio_fifo, buf);
@@ -640,35 +639,59 @@ static int demux_mpeg_block_send_chunk (demux_plugin_t *this_gen) {
   return this->status;
 }
 
-/* estimate bitrate */
+#ifdef ESTIMATE_RATE_FIXED
+/*! 
+   Estimate bitrate by looking inside the MPEG file for presentation 
+   time stamps (PTS) and computing how far apart these are 
+   in bytes and in time. 
+
+   On failure return 0.
+
+   This might be used after deciding that mux_rate in a stream is faulty.
+
+*/
+
+/* How many *sucessful* PTS samples do we take? */
+#define MAX_SAMPLES 5
+
+/* How many times we read blocks before giving up. */
+#define MAX_READS 30
+
+/* TRUNCATE x to the nearest multiple of y. */
+#define TRUNC(x,y) (((x) / (y)) * (y))
 
 static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
 
   buf_element_t *buf = NULL;
   unsigned char *p;
   int            is_mpeg1=0;
-  off_t          pos, last_pos;
-  off_t          step;
-  int64_t        pts, last_pts;
-  int            rate;
-  int            count;
+  off_t          pos, last_pos=0;
+  off_t          step, mpeg_length;
+  off_t          blocksize = this->blocksize;
+  int64_t        pts, last_pts=0;
+  int            reads=0    /* Number of blocks read so far */;
+  int            count=0;   /* Number of sucessful PTS found so far */
+  int            rate=0;    /* The return rate value */
   int            stream_id;
 
+  /* We can't estimate by sampling if we don't thave the ability to 
+     randomly access the and more importantly reset after accessessing.  */
   if (!(this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE))
     return 0;
 
-  last_pos = 0;
-  last_pts = 0;
-  rate     = 0;
-  step     = this->input->get_length (this->input) / 10;
-  step     = (step / this->blocksize) * this->blocksize;
-  if (step <= 0) step = this->blocksize; /* avoid endless loop for tiny files */
-  pos      = step;
-  count    = 0;
+  mpeg_length= this->input->get_length (this->input);
+  step = TRUNC((mpeg_length/MAX_SAMPLES), blocksize); 
+  if (step <= 0) step = blocksize; /* avoid endless loop for tiny files */
+  pos = step;
+
+  /* At this point "pos", and "step" are a multiple of blocksize and
+     they should continue to be so throughout.
+   */
   
   this->input->seek (this->input, pos, SEEK_SET);
 
-  while ((buf = this->input->read_block (this->input, this->video_fifo, this->blocksize)) ) {
+  while ( (buf = this->input->read_block (this->input, this->video_fifo, blocksize)) 
+	  && count < MAX_SAMPLES && reads++ < MAX_READS ) {
 
     p = buf->content; /* len = this->mnBlocksize; */
 
@@ -698,7 +721,7 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
     pts = 0; 
 
     if ((stream_id < 0xbc) || ((stream_id & 0xf0) != 0xe0)) {
-      pos += (off_t) this->blocksize;
+      pos += (off_t) blocksize;
       buf->free_buffer (buf);
       continue; /* only use video packets */
     }
@@ -762,11 +785,11 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
       last_pts = pts;
       pos += step;
     } else
-      pos += (off_t) this->blocksize;
+      pos += blocksize;
 
     buf->free_buffer (buf);
 
-    if (this->input->seek (this->input, pos, SEEK_SET) == (off_t)-1)
+    if (pos > mpeg_length || this->input->seek (this->input, pos, SEEK_SET) == (off_t)-1)
       break;
 
   }
@@ -777,6 +800,7 @@ static int demux_mpeg_block_estimate_rate (demux_mpeg_block_t *this) {
   return rate;
   
 }
+#endif /*ESTIMATE_RATE_FIXED*/
 
 static void demux_mpeg_block_dispose (demux_plugin_t *this_gen) {
 
@@ -835,8 +859,13 @@ static void demux_mpeg_block_send_headers (demux_plugin_t *this_gen) {
   
   xine_demux_control_start(this->stream);
   
-  if (!this->rate)
+#ifdef USE_ILL_ADVISED_ESTIMATE_RATE_INITIALLY
+  if (!this->rate) 
     this->rate = demux_mpeg_block_estimate_rate (this);
+#else 
+  /* Set to Use rate given in by stream initially. */
+  this->rate = 0; 
+#endif
   
   if((this->input->get_capabilities(this->input) & INPUT_CAP_SEEKABLE) != 0) {
     
@@ -915,12 +944,17 @@ static void demux_mpeg_block_accept_input (demux_mpeg_block_t *this,
 
     strncpy (this->cur_mrl, input->get_mrl(input), 256);
 
-    printf ("demux_mpeg_block: mrl %s is new, will estimated bitrate\n",
+#ifdef LOG
+    printf ("demux_mpeg_block: mrl %s is new\n",
 	    this->cur_mrl);
+#endif
 
   } else
-    printf ("demux_mpeg_block: mrl %s is known, estimated bitrate: %d\n",
-	    this->cur_mrl, this->rate * 50 * 8);
+#ifdef LOG    
+    printf ("demux_mpeg_block: mrl %s is known, bitrate: %d\n",
+	    this->cur_mrl, this->rate * 50 * 8)
+#endif
+      ;
 }
 
 static int demux_mpeg_block_get_stream_length (demux_plugin_t *this_gen) {
@@ -931,7 +965,8 @@ static int demux_mpeg_block_get_stream_length (demux_plugin_t *this_gen) {
    */
 
   if (this->rate) 
-    return this->input->get_length (this->input) / (this->rate * 50);
+    return (int)((int64_t) 1000 * this->input->get_length (this->input) /
+                 (this->rate * 50));
   else
     return 0;
 }
@@ -1053,6 +1088,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
           return NULL;
         }
 
+        input->seek(input, 0, SEEK_SET);
+
         demux_mpeg_block_accept_input (this, input);
 #ifdef LOG
         printf("demux_mpeg_block:open_plugin:Accepting detection_method XINE_DEMUX_CONTENT_STRATEGY blocksize=%d\n",this->blocksize);
@@ -1142,7 +1179,7 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return "";
+  return NULL;
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -1176,6 +1213,6 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "mpeg_block", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "mpeg_block", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
