@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_ogg.c,v 1.55 2002/12/08 21:43:51 miguelfreitas Exp $
+ * $Id: demux_ogg.c,v 1.63 2003/01/26 23:48:47 tmattern Exp $
  *
  * demultiplexer for ogg streams
  *
@@ -30,7 +30,6 @@
 #include <stdio.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <sched.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -250,8 +249,9 @@ static void send_ogg_buf (demux_ogg_t *this,
 	    op->granulepos, buf->pts);
 #endif
 
-    buf->input_pos     = this->input->get_current_pos (this->input);
-    buf->input_time    = buf->input_pos * 8 / this->avg_bitrate;
+    buf->extra_info->input_pos     = this->input->get_current_pos (this->input);
+    buf->extra_info->input_time    = (int)((uint64_t)buf->extra_info->input_pos 
+                                           * 8 * 1000 / this->avg_bitrate);
     buf->type          = this->buf_types[stream_num] ;
     buf->decoder_flags = decoder_flags;
     
@@ -299,8 +299,8 @@ static void send_ogg_buf (demux_ogg_t *this,
       printf ("demux_ogg: video granulepos %lld, pts %lld\n", op->granulepos, buf->pts);
 #endif
       
-      buf->input_pos  = this->input->get_current_pos (this->input);
-      buf->input_time = 0;
+      buf->extra_info->input_pos  = this->input->get_current_pos (this->input);
+      buf->extra_info->input_time = 0;
       buf->type       = this->buf_types[stream_num] ;
 	
       done += buf->size;
@@ -432,7 +432,10 @@ static void demux_ogg_send_header (demux_ogg_t *this) {
 
 	    channel = this->num_video_streams++;
 
-	    this->buf_types[stream_num] = fourcc_to_buf_video (oggh->subtype) | channel;
+	    this->buf_types[stream_num] = fourcc_to_buf_video (oggh->subtype);
+	    if( !this->buf_types[stream_num] )
+	      this->buf_types[stream_num] = BUF_VIDEO_UNKNOWN;
+	    this->buf_types[stream_num] |= channel;
 	    this->preview_buffers[stream_num] = 5; /* FIXME: don't know */
 
 #ifdef LOG
@@ -589,7 +592,10 @@ static void demux_ogg_send_header (demux_ogg_t *this) {
 
 	      printf ("demux_ogg: fourcc %08x\n", fcc);
 
-	      this->buf_types[stream_num] = fourcc_to_buf_video (fcc) | channel;
+	      this->buf_types[stream_num] = fourcc_to_buf_video (fcc);
+	      if( !this->buf_types[stream_num] )
+	        this->buf_types[stream_num] = BUF_VIDEO_UNKNOWN;
+	      this->buf_types[stream_num] |= channel;
 
 	      bih.biSize          = sizeof(xine_bmiheader);
 	      bih.biWidth         = *(int32_t*)(op.packet+176);
@@ -914,7 +920,8 @@ static int demux_ogg_get_stream_length (demux_plugin_t *this_gen) {
   demux_ogg_t *this = (demux_ogg_t *) this_gen; 
 
   if (this->avg_bitrate)
-    return this->input->get_length (this->input) * 8 / this->avg_bitrate;
+    return (int)((int64_t)1000 * this->input->get_length (this->input) * 8 /
+           this->avg_bitrate);
   else
     return 0;
 }
@@ -951,6 +958,8 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
 	      || (buf[2] != 'g')
 	      || (buf[3] != 'S')) 
 	    return NULL;
+	} else {
+	  return NULL;
 	}
       } else if (input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW)) {
 	if ((buf[0] != 'O')
@@ -1037,7 +1046,8 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return "audio/x-ogg: ogg: OggVorbis Audio;";
+  return "audio/x-ogg: ogg: OggVorbis Audio;"
+         "application/x-ogg: ogg: OggVorbis Audio;"; 
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -1071,6 +1081,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "ogg", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_DEMUX, 20, "ogg", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
