@@ -30,7 +30,7 @@
  *    build_frame_table
  *  free_qt_info
  *
- * $Id: demux_qt.c,v 1.125 2002/12/08 21:43:51 miguelfreitas Exp $
+ * $Id: demux_qt.c,v 1.142 2003/01/19 23:33:33 tmmm Exp $
  *
  */
 
@@ -95,6 +95,7 @@ typedef unsigned int qt_atom;
 #define CO64_ATOM QT_ATOM('c', 'o', '6', '4')
 
 #define ESDS_ATOM QT_ATOM('e', 's', 'd', 's')
+#define WAVE_ATOM QT_ATOM('w', 'a', 'v', 'e')
 
 #define IMA4_FOURCC QT_ATOM('i', 'm', 'a', '4')
 #define MP4A_FOURCC QT_ATOM('m', 'p', '4', 'a')
@@ -178,6 +179,8 @@ typedef struct {
       unsigned int channels;
       unsigned int bits;
       unsigned int vbr;
+      unsigned int wave_present;
+      xine_waveformatex wave;
     } audio;
 
   } media_description;
@@ -228,6 +231,10 @@ typedef struct {
   unsigned int bytes_per_sample;
   unsigned int samples_per_frame;
 
+  /* verbatim copy of the stsd atom */
+  int          stsd_size;
+  void        *stsd;
+
 } qt_sample_table;
 
 typedef struct {
@@ -242,22 +249,29 @@ typedef struct {
   int64_t moov_first_offset;
   int64_t moov_last_offset;
 
-  qt_atom audio_codec;
-  unsigned int audio_type;
-  unsigned int audio_sample_rate;
-  unsigned int audio_channels;
-  unsigned int audio_bits;
-  unsigned int audio_vbr;    /* flag to indicate if audio is VBR */
-  void *audio_decoder_config;
-  int audio_decoder_config_len;
+  qt_atom           audio_codec;
+  unsigned int      audio_type;
+  unsigned int      audio_sample_rate;
+  unsigned int      audio_channels;
+  unsigned int      audio_bits;
+  unsigned int      audio_vbr;    /* flag to indicate if audio is VBR */
+  void             *audio_decoder_config;
+  int               audio_decoder_config_len;
+  int               wave_present;
+  xine_waveformatex wave;
+  int               audio_stsd_size;
+  void             *audio_stsd;
+
+  qt_atom           video_codec;
+  unsigned int      video_type;
+  unsigned int      video_width;
+  unsigned int      video_height;
+  unsigned int      video_depth;
+  void             *video_decoder_config;
+  int               video_decoder_config_len;
+  int               video_stsd_size;
+  void             *video_stsd;
 
-  qt_atom video_codec;
-  unsigned int video_type;
-  unsigned int video_width;
-  unsigned int video_height;
-  unsigned int video_depth;
-  void *video_decoder_config;
-  int video_decoder_config_len;
 
   qt_frame *frames;
   unsigned int frame_count;
@@ -373,6 +387,37 @@ static inline void debug_video_demux(const char *format, ...) { }
 static inline void debug_audio_demux(const char *format, ...) { }
 #endif
 
+static void hexdump (char *buf, int length) {
+
+  int i;
+
+  printf ("demux_qt: ascii contents>");
+  for (i = 0; i < length; i++) {
+    unsigned char c = buf[i];
+
+    if ((c >= 32) && (c <= 128))
+      printf ("%c", c);
+    else
+      printf (".");
+  }
+  printf ("\n");
+
+  printf ("demux_qt: complete hexdump of package follows:\ndemux_qt 0x0000:  ");
+  for (i = 0; i < length; i++) {
+    unsigned char c = buf[i];
+
+    printf ("%02x", c);
+
+    if ((i % 16) == 15)
+      printf ("\ndemux_qt 0x%04x: ", i);
+
+    if ((i % 2) == 1)
+      printf (" ");
+
+  }
+  printf ("\n");
+}
+
 void dump_moov_atom(unsigned char *moov_atom, int moov_atom_size) {
 #if DEBUG_DUMP_MOOV
 
@@ -435,6 +480,28 @@ static void find_moov_atom(input_plugin_t *input, off_t *moov_offset,
       break;
     }
 
+    /* special case alert: 'free' atoms are known to contain 'cmov' atoms.
+     * If this is a free atom, check for cmov immediately following.
+     * QT Player can handle it, so xine should too. */
+    if (atom == FREE_ATOM) {
+
+      /* get the next atom preamble */
+      if (input->read(input, atom_preamble, ATOM_PREAMBLE_SIZE) !=
+        ATOM_PREAMBLE_SIZE)
+        break;
+
+      /* if there is a cmov, qualify this free atom as the moov atom */
+      if (BE_32(&atom_preamble[4]) == CMOV_ATOM) {
+        /* pos = current pos minus 2 atom preambles */
+        *moov_offset = input->get_current_pos(input) - ATOM_PREAMBLE_SIZE * 2;
+        *moov_size = atom_size;
+        break;
+      } else {
+        /* otherwise, rewind the stream */
+        input->seek(input, -ATOM_PREAMBLE_SIZE, SEEK_CUR);
+      }
+    }
+
     /* if this atom is not the moov atom, make sure that it is at least one
      * of the other top-level QT atom */
     if ((atom != FREE_ATOM) &&
@@ -490,6 +557,7 @@ qt_info *create_qt_info(void) {
   info->audio_vbr = 0;
   info->audio_decoder_config = NULL;
   info->audio_decoder_config_len = 0;
+  info->audio_stsd = NULL;
 
   info->video_codec = 0;
   info->video_type = 0;
@@ -498,6 +566,7 @@ qt_info *create_qt_info(void) {
   info->video_depth = 0;
   info->video_decoder_config = NULL;
   info->video_decoder_config_len = 0;
+  info->video_stsd = NULL;
 
   info->frames = NULL;
   info->frame_count = 0;
@@ -523,6 +592,8 @@ void free_qt_info(qt_info *info) {
       free(info->audio_decoder_config);
     if(info->video_decoder_config)
       free(info->video_decoder_config);
+    free(info->audio_stsd);
+    free(info->video_stsd);
     free(info->copyright);
     free(info->description);
     free(info->comment);
@@ -589,12 +660,13 @@ static int mp4_read_descr_len(unsigned char *s, uint32_t *length) {
  * This function traverses through a trak atom searching for the sample
  * table atoms, which it loads into an internal sample table structure.
  */
-static qt_error parse_trak_atom(qt_sample_table *sample_table,
-  unsigned char *trak_atom) {
+static qt_error parse_trak_atom (qt_sample_table *sample_table,
+				 unsigned char *trak_atom) {
 
   int i, j;
   unsigned int trak_atom_size = BE_32(&trak_atom[0]);
   qt_atom current_atom;
+  unsigned int current_atom_size;
   qt_error last_error = QT_OK;
 
   /* for palette traversal */
@@ -625,6 +697,9 @@ static qt_error parse_trak_atom(qt_sample_table *sample_table,
   sample_table->frames = NULL;
   sample_table->decoder_config = NULL;
   sample_table->decoder_config_len = 0;
+  sample_table->stsd = NULL;
+  memset(&sample_table->media_description, 0, 
+    sizeof(sample_table->media_description));
 
   /* special audio parameters */
   sample_table->samples_per_packet = 0;
@@ -636,14 +711,6 @@ static qt_error parse_trak_atom(qt_sample_table *sample_table,
   /* default type */
   sample_table->type = MEDIA_OTHER;
 
-  /* video size and depth not yet known */
-  sample_table->media_description.video.width = 0;
-  sample_table->media_description.video.height = 0;
-  sample_table->media_description.video.depth = 0;
-
-  /* assume no palette at first */
-  sample_table->media_description.video.palette_count = 0;
-
   /* search for media type atoms */
   for (i = ATOM_PREAMBLE_SIZE; i < trak_atom_size - 4; i++) {
     current_atom = BE_32(&trak_atom[i]);
@@ -663,6 +730,7 @@ static qt_error parse_trak_atom(qt_sample_table *sample_table,
 
   /* search for the useful atoms */
   for (i = ATOM_PREAMBLE_SIZE; i < trak_atom_size - 4; i++) {
+    current_atom_size = BE_32(&trak_atom[i - 4]);	
     current_atom = BE_32(&trak_atom[i]);
 
     if (current_atom == TKHD_ATOM) {
@@ -715,8 +783,27 @@ static qt_error parse_trak_atom(qt_sample_table *sample_table,
       sample_table->timescale = BE_32(&trak_atom[i + 0x10]);
     else if (current_atom == STSD_ATOM) {
 
+      debug_atom_load ("demux_qt: stsd atom\n");
+#if DEBUG_ATOM_LOAD
+      hexdump (&trak_atom[i], current_atom_size);
+#endif
+
+      /* copy whole stsd atom so it can later be sent to the decoder */
+
+      sample_table->stsd_size = current_atom_size;
+      sample_table->stsd = xine_xmalloc (current_atom_size);
+      memcpy (sample_table->stsd, &trak_atom[i], current_atom_size);
+      
       if (sample_table->type == MEDIA_VIDEO) {
 
+        /* initialize to sane values */
+        sample_table->media_description.video.width = 0;
+        sample_table->media_description.video.height = 0;
+        sample_table->media_description.video.depth = 0;
+
+        /* assume no palette at first */
+        sample_table->media_description.video.palette_count = 0;
+
         /* fetch video parameters */
         if( BE_16(&trak_atom[i + 0x2C]) && BE_16(&trak_atom[i + 0x2E]) ) {
           sample_table->media_description.video.width =
@@ -897,6 +984,18 @@ static qt_error parse_trak_atom(qt_sample_table *sample_table,
         if (BE_32(&trak_atom[i + 0x10]) == MP4A_FOURCC)
           sample_table->media_description.audio.vbr = 1;
 
+        /* check for a MS-style WAVE format header */
+        if ((current_atom_size >= 0x48) && 
+            (BE_32(&trak_atom[i + 0x44]) == WAVE_ATOM)) {
+          sample_table->media_description.audio.wave_present = 1;
+          memcpy(&sample_table->media_description.audio.wave, 
+            &trak_atom[i + 0x5C],
+            sizeof(sample_table->media_description.audio.wave));
+          xine_waveformatex_le2me(&sample_table->media_description.audio.wave);
+        } else {
+          sample_table->media_description.audio.wave_present = 0;
+        }
+
         debug_atom_load("    audio description\n");
         debug_atom_load("      %d Hz, %d bits, %d channels, %saudio fourcc = '%c%c%c%c' (%02X%02X%02X%02X)\n",
           sample_table->media_description.audio.sample_rate,
@@ -1152,13 +1251,60 @@ free_sample_table:
   free(sample_table->sample_to_chunk_table);
   free(sample_table->time_to_sample_table);
   free(sample_table->decoder_config);
+  free(sample_table->stsd);
 
   return last_error;
 }
 
-static qt_error build_frame_table(qt_sample_table *sample_table,
+/* This is a little support function used to process the edit list when
+ * building a frame table. */
+#define MAX_DURATION 0x7FFFFFFFFFFFFFFF
+static void get_next_edit_list_entry(qt_sample_table *sample_table, 
+  int *edit_list_index,
+  unsigned int *edit_list_media_time, 
+  int64_t *edit_list_duration,
   unsigned int global_timescale) {
 
+  /* if there is no edit list, set to max duration and get out */
+  if (!sample_table->edit_list_table) {
+
+    *edit_list_media_time = 0;
+    *edit_list_duration = MAX_DURATION;
+    debug_edit_list("  qt: no edit list table, initial = %d, %lld\n", *edit_list_media_time, *edit_list_duration);
+    return;
+
+  } else while (*edit_list_index < sample_table->edit_list_count) {
+
+    /* otherwise, find an edit list entries whose media time != -1 */
+    if (sample_table->edit_list_table[*edit_list_index].media_time != -1) {
+
+      *edit_list_media_time = 
+        sample_table->edit_list_table[*edit_list_index].media_time;
+      *edit_list_duration = 
+        sample_table->edit_list_table[*edit_list_index].track_duration;
+
+      /* duration is in global timescale units; convert to trak timescale */
+      *edit_list_duration *= sample_table->timescale;
+      *edit_list_duration /= global_timescale;
+
+      *edit_list_index = *edit_list_index + 1;
+      break;
+    }
+
+    *edit_list_index = *edit_list_index + 1;
+  }
+
+  /* on the way out, check if this is the last edit list entry; if so, 
+   * don't let the duration expire (so set it to an absurdly large value) 
+   */
+  if (*edit_list_index == sample_table->edit_list_count)
+    *edit_list_duration = MAX_DURATION;
+  debug_edit_list("  qt: edit list table exists, initial = %d, %lld\n", *edit_list_media_time, *edit_list_duration);
+}
+
+static qt_error build_frame_table(qt_sample_table *sample_table,
+				  unsigned int global_timescale) {
+
   int i, j;
   unsigned int frame_counter;
   unsigned int chunk_start, chunk_end;
@@ -1262,27 +1408,8 @@ static qt_error build_frame_table(qt_sample_table *sample_table,
 
     /* initialize edit list considerations */
     edit_list_index = 0;
-    if (sample_table->edit_list_table) {
-      edit_list_media_time = 
-        sample_table->edit_list_table[edit_list_index].media_time;
-      edit_list_duration = 
-        sample_table->edit_list_table[edit_list_index].track_duration;
-
-      /* duration is in global timescale units; convert to trak timescale */
-      edit_list_duration *= sample_table->timescale;
-      edit_list_duration /= global_timescale;
-
-      edit_list_index++;
-      /* if this is the last edit list entry, don't let the duration
-       * expire (so set it to an absurdly large value) */
-      if (edit_list_index == sample_table->edit_list_count)
-        edit_list_duration = 0xFFFFFFFFFFFF;
-      debug_edit_list("  qt: edit list table exists, initial = %d, %lld\n", edit_list_media_time, edit_list_duration);
-    } else {
-      edit_list_media_time = 0;
-      edit_list_duration = 0xFFFFFFFFFFFF;
-      debug_edit_list("  qt: no edit list table, initial = %d, %lld\n", edit_list_media_time, edit_list_duration);
-    }
+    get_next_edit_list_entry(sample_table, &edit_list_index,
+      &edit_list_media_time, &edit_list_duration, global_timescale);
 
     /* fix up pts information w.r.t. the edit list table */
     edit_list_pts_counter = 0;
@@ -1297,7 +1424,7 @@ static qt_error build_frame_table(qt_sample_table *sample_table,
           frame_duration = 
             (sample_table->frames[i + 1].pts - sample_table->frames[i].pts);
 
-            debug_edit_list("duration = %lld...", frame_duration);
+        debug_edit_list("duration = %lld...", frame_duration);
         sample_table->frames[i].pts = edit_list_pts_counter;
         edit_list_pts_counter += frame_duration;
         edit_list_duration -= frame_duration;
@@ -1307,29 +1434,8 @@ static qt_error build_frame_table(qt_sample_table *sample_table,
 
       /* reload media time and duration */
       if (edit_list_duration <= 0) {
-        if ((sample_table->edit_list_table) &&
-            (edit_list_index < sample_table->edit_list_count)) {
-          debug_edit_list("\n  edit list index = %d, ", edit_list_index);
-          edit_list_media_time = 
-            sample_table->edit_list_table[edit_list_index].media_time;
-          edit_list_duration = 
-            sample_table->edit_list_table[edit_list_index].track_duration;
-
-          /* duration is in global timescale units; convert to trak timescale */
-          edit_list_duration *= sample_table->timescale;
-          edit_list_duration /= global_timescale;
-
-          edit_list_index++;
-          /* if this is the last edit list entry, don't let the duration
-           * expire (so set it to an absurdly large value) */
-          if (edit_list_index == sample_table->edit_list_count)
-            edit_list_duration = 0xFFFFFFFFFFFF;
-          debug_edit_list("entry: %d, %lld\n      ", edit_list_media_time, edit_list_duration);
-        } else {
-          edit_list_media_time = 0;
-          edit_list_duration = 0xFFFFFFFFFFFF;
-          debug_edit_list("no edit list table (or expired): %d, %lld\n", edit_list_media_time, edit_list_duration);
-        }
+        get_next_edit_list_entry(sample_table, &edit_list_index,
+          &edit_list_media_time, &edit_list_duration, global_timescale);
       }
 
       debug_edit_list("(after) pts = %lld...\n", sample_table->frames[i].pts);
@@ -1445,8 +1551,8 @@ static void parse_moov_atom(qt_info *info, unsigned char *moov_atom) {
       sample_tables = (qt_sample_table *)realloc(sample_tables,
         sample_table_count * sizeof(qt_sample_table));
 
-      parse_trak_atom(&sample_tables[sample_table_count - 1],
-        &moov_atom[i - 4]);
+      parse_trak_atom (&sample_tables[sample_table_count - 1],
+		       &moov_atom[i - 4]);
       if (info->last_error != QT_OK)
         return;
       i += BE_32(&moov_atom[i - 4]) - 4;
@@ -1510,6 +1616,11 @@ static void parse_moov_atom(qt_info *info, unsigned char *moov_atom) {
           PALETTE_COUNT * sizeof(palette_entry_t));
       }
 
+      /* stsd atom */
+      info->video_stsd      = realloc (info->video_stsd, sample_tables[i].stsd_size);
+      info->video_stsd_size = sample_tables[i].stsd_size;
+      memcpy(info->video_stsd, sample_tables[i].stsd, info->video_stsd_size);
+
     } else if (sample_tables[i].type == MEDIA_AUDIO) {
 
       info->audio_sample_rate =
@@ -1525,6 +1636,16 @@ static void parse_moov_atom(qt_info *info, unsigned char *moov_atom) {
 
       info->audio_decoder_config = sample_tables[i].decoder_config;
       info->audio_decoder_config_len = sample_tables[i].decoder_config_len;
+
+      info->wave_present = 
+        sample_tables[i].media_description.audio.wave_present;
+      memcpy(&info->wave, &sample_tables[i].media_description.audio.wave,
+        sizeof(xine_waveformatex));
+
+      /* stsd atom */
+      info->audio_stsd      = realloc (info->audio_stsd, sample_tables[i].stsd_size);
+      info->audio_stsd_size = sample_tables[i].stsd_size;
+      memcpy(info->audio_stsd, sample_tables[i].stsd, info->audio_stsd_size);
     }
   }
   debug_frame_table("  qt: finished building frame tables, merging into one...\n");
@@ -1598,6 +1719,7 @@ static void parse_moov_atom(qt_info *info, unsigned char *moov_atom) {
     free(sample_tables[i].sample_to_chunk_table);
     free(sample_tables[i].sync_sample_table);
     free(sample_tables[i].frames);
+    free(sample_tables[i].stsd);
   }
   free(sample_tables);
   free(sample_table_indices);
@@ -1792,9 +1914,9 @@ static int demux_qt_send_chunk(demux_plugin_t *this_gen) {
     while (remaining_sample_bytes) {
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
       buf->type = this->qt->video_type;
-      buf->input_pos = this->qt->frames[i].offset - this->data_start;
-      buf->input_length = this->data_size;
-      buf->input_time = this->qt->frames[i].pts / 90000;
+      buf->extra_info->input_pos = this->qt->frames[i].offset - this->data_start;
+      buf->extra_info->input_length = this->data_size;
+      buf->extra_info->input_time = this->qt->frames[i].pts / 90;
       buf->pts = this->qt->frames[i].pts;
 
       buf->decoder_flags |= BUF_FLAG_FRAMERATE;
@@ -1837,8 +1959,8 @@ static int demux_qt_send_chunk(demux_plugin_t *this_gen) {
     while (remaining_sample_bytes) {
       buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
       buf->type = this->qt->audio_type;
-      buf->input_pos = this->qt->frames[i].offset - this->data_start;
-      buf->input_length = this->data_size;
+      buf->extra_info->input_pos = this->qt->frames[i].offset - this->data_start;
+      buf->extra_info->input_length = this->data_size;
       /* The audio chunk is often broken up into multiple 8K buffers when
        * it is sent to the audio decoder. Only attach the proper timestamp
        * to the first buffer. This is for the linear PCM decoder which
@@ -1848,15 +1970,15 @@ static int demux_qt_send_chunk(demux_plugin_t *this_gen) {
       if ((buf->type == BUF_AUDIO_LPCM_BE) || 
           (buf->type == BUF_AUDIO_LPCM_LE)) { 
         if (first_buf) {
-          buf->input_time = this->qt->frames[i].pts / 90000;
+          buf->extra_info->input_time = this->qt->frames[i].pts / 90;
           buf->pts = this->qt->frames[i].pts;
           first_buf = 0;
         } else {
-          buf->input_time = 0;
+          buf->extra_info->input_time = 0;
           buf->pts = 0;
         }
       } else {
-        buf->input_time = this->qt->frames[i].pts / 90000;
+        buf->extra_info->input_time = this->qt->frames[i].pts / 90;
         buf->pts = this->qt->frames[i].pts;
       }
 
@@ -1919,18 +2041,15 @@ static void demux_qt_send_headers(demux_plugin_t *this_gen) {
    */
   if( this->qt->video_type == BUF_VIDEO_MSMPEG4_V1 )
     this->qt->video_type = BUF_VIDEO_MPEG4;
-#if 0
+  
   if( !this->qt->video_type && this->qt->video_codec )
-    xine_report_codec( this->stream, XINE_CODEC_VIDEO, this->bih.biCompression, 0, 0);
-#endif
+    this->qt->video_type = BUF_VIDEO_UNKNOWN;
 
   this->qt->audio_type = formattag_to_buf_audio(this->qt->audio_codec);
 
-#if 0
   if( !this->qt->audio_type && this->qt->audio_codec )
-    xine_report_codec( this->stream, XINE_CODEC_AUDIO, this->qt->audio_codec, 0, 0);
-#endif
-
+    this->qt->audio_type = BUF_AUDIO_UNKNOWN;
+  
   /* load stream information */
   this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] =
     (this->qt->video_type) ? 1 : 0;
@@ -1976,6 +2095,8 @@ static void demux_qt_send_headers(demux_plugin_t *this_gen) {
   /* send header info to decoder. some mpeg4 streams need this */
   if( this->qt->video_decoder_config ) {
     buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+    buf->decoder_flags = BUF_FLAG_SPECIAL;
+    buf->decoder_info[0] = BUF_SPECIAL_DECODER_CONFIG;
     buf->type = this->qt->video_type;
     buf->size = this->qt->video_decoder_config_len;
     buf->content = this->qt->video_decoder_config;      
@@ -1988,12 +2109,23 @@ static void demux_qt_send_headers(demux_plugin_t *this_gen) {
     buf->decoder_flags = BUF_FLAG_SPECIAL;
     buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
     buf->decoder_info[2] = this->qt->palette_count;
-    buf->decoder_info[3] = (unsigned int)&this->qt->palette;
+    buf->decoder_info_ptr[2] = &this->qt->palette;
     buf->size = 0;
     buf->type = this->qt->video_type;
     this->video_fifo->put (this->video_fifo, buf);
   }
 
+  /* send stsd to the decoder */
+  buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
+  buf->decoder_flags = BUF_FLAG_SPECIAL;
+  buf->decoder_info[1] = BUF_SPECIAL_STSD_ATOM;
+  buf->decoder_info[2] = this->qt->video_stsd_size;
+  buf->decoder_info_ptr[2] = this->qt->video_stsd;
+  buf->size = 0;
+  buf->type = this->qt->video_type;
+  this->video_fifo->put (this->video_fifo, buf);
+
+
   if (this->audio_fifo && this->qt->audio_type) {
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = this->qt->audio_type;
@@ -2002,6 +2134,8 @@ static void demux_qt_send_headers(demux_plugin_t *this_gen) {
     buf->decoder_info[1] = this->qt->audio_sample_rate;
     buf->decoder_info[2] = this->qt->audio_bits;
     buf->decoder_info[3] = this->qt->audio_channels;
+    buf->content = (void *)&this->qt->wave;
+    buf->size = sizeof(this->qt->wave);
     this->audio_fifo->put (this->audio_fifo, buf);
     
     if( this->qt->audio_decoder_config ) {
@@ -2011,9 +2145,20 @@ static void demux_qt_send_headers(demux_plugin_t *this_gen) {
       buf->decoder_flags = BUF_FLAG_SPECIAL;
       buf->decoder_info[1] = BUF_SPECIAL_DECODER_CONFIG;
       buf->decoder_info[2] = this->qt->audio_decoder_config_len;
-      buf->decoder_info[3] = (uint32_t)this->qt->audio_decoder_config;
+      buf->decoder_info_ptr[2] = this->qt->audio_decoder_config;
       this->audio_fifo->put (this->audio_fifo, buf);
     }
+
+    /* send stsd to the decoder */
+    buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+    buf->decoder_flags = BUF_FLAG_SPECIAL;
+    buf->decoder_info[1] = BUF_SPECIAL_STSD_ATOM;
+    buf->decoder_info[2] = this->qt->audio_stsd_size;
+    buf->decoder_info_ptr[2] = this->qt->audio_stsd;
+    buf->size = 0;
+    buf->type = this->qt->audio_type;
+    this->audio_fifo->put (this->audio_fifo, buf);
+
   }
 }
 
@@ -2030,30 +2175,54 @@ static int demux_qt_seek (demux_plugin_t *this_gen,
 
   /* perform a binary search on the sample table, testing the offset
    * boundaries first */
-  if (start_pos <= this->qt->frames[0].offset)
-    best_index = 0;
-  else if (start_pos >= this->qt->frames[this->qt->frame_count - 1].offset) {
-    this->status = DEMUX_FINISHED;
-    return this->status;
-  } else {
-    left = 0;
-    right = this->qt->frame_count - 1;
-    found = 0;
+  if (start_pos) {
+    if (start_pos <= this->qt->frames[0].offset)
+      best_index = 0;
+    else if (start_pos >= this->qt->frames[this->qt->frame_count - 1].offset) {
+      this->status = DEMUX_FINISHED;
+      return this->status;
+    } else {
+      left = 0;
+      right = this->qt->frame_count - 1;
+      found = 0;
 
-    while (!found) {
-      middle = (left + right) / 2;
-      if ((start_pos >= this->qt->frames[middle].offset) &&
-          (start_pos <= this->qt->frames[middle].offset +
-           this->qt->frames[middle].size)) {
-        found = 1;
-      } else if (start_pos < this->qt->frames[middle].offset) {
-        right = middle;
-      } else {
-        left = middle;
+      while (!found) {
+	middle = (left + right) / 2;
+	if ((start_pos >= this->qt->frames[middle].offset) &&
+	    (start_pos <= this->qt->frames[middle].offset +
+	     this->qt->frames[middle].size)) {
+	  found = 1;
+	} else if (start_pos < this->qt->frames[middle].offset) {
+	  right = middle;
+	} else {
+	  left = middle;
+        }
       }
-    }
 
-    best_index = middle;
+      best_index = middle;
+    }
+  } else {
+    int64_t pts = 90000 * start_time;
+
+    if (pts <= this->qt->frames[0].pts)
+      best_index = 0;
+    else if (pts >= this->qt->frames[this->qt->frame_count - 1].pts) {
+      this->status = DEMUX_FINISHED;
+      return this->status;
+    } else {
+      left = 0;
+      right = this->qt->frame_count - 1;
+      do {
+	middle = (left + right + 1) / 2;
+	if (pts < this->qt->frames[middle].pts) {
+	  right = (middle - 1);
+	} else {
+	  left = middle;
+	}
+      } while (left < right);
+
+      best_index = left;
+    }
   }
 
   /* search back in the table for the nearest keyframe */
@@ -2106,7 +2275,7 @@ static int demux_qt_get_stream_length (demux_plugin_t *this_gen) {
 
   demux_qt_t *this = (demux_qt_t *) this_gen;
 
-  return (this->qt->duration / this->qt->timescale);
+  return (int)((int64_t) 1000 * this->qt->duration / this->qt->timescale);
 }
 
 static uint32_t demux_qt_get_capabilities(demux_plugin_t *this_gen) {
@@ -2216,33 +2385,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   strncpy (this->last_mrl, input->get_mrl (input), 1024);
 
-  /* print vital stats */
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_qt: Apple Quicktime file, %srunning time: %d min, %d sec\n"),
-    (this->qt->compressed_header) ? "compressed header, " : "",
-    this->qt->duration / this->qt->timescale / 60,
-    this->qt->duration / this->qt->timescale % 60);
-  if (this->qt->video_codec)
-    xine_log (this->stream->xine, XINE_LOG_MSG,
-      _("demux_qt: '%c%c%c%c' video @ %dx%d\n"),
-      *((char *)&this->qt->video_codec + 0),
-      *((char *)&this->qt->video_codec + 1),
-      *((char *)&this->qt->video_codec + 2),
-      *((char *)&this->qt->video_codec + 3),
-      this->qt->video_width,
-      this->qt->video_height);
-  if (this->qt->audio_codec)
-    xine_log (this->stream->xine, XINE_LOG_MSG,
-      _("demux_qt: '%c%c%c%c' audio @ %d Hz, %d bits, %d %s\n"),
-      *((char *)&this->qt->audio_codec + 0),
-      *((char *)&this->qt->audio_codec + 1),
-      *((char *)&this->qt->audio_codec + 2),
-      *((char *)&this->qt->audio_codec + 3),
-      this->qt->audio_sample_rate,
-      this->qt->audio_bits,
-      this->qt->audio_channels,
-      ngettext("channel", "channels", this->qt->audio_channels));
-
   return &this->demux_plugin;
 }
 
@@ -2294,6 +2436,6 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "quicktime", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "quicktime", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
