@@ -21,7 +21,14 @@
  * For more information regarding the Real file format, visit:
  *   http://www.pcisys.net/~melanson/codecs/
  *
- * $Id: demux_real.c,v 1.21 2002/12/10 19:47:30 guenter Exp $
+ * video packet sub-demuxer ported from mplayer code (www.mplayerhq.hu):
+ *   Real parser & demuxer
+ *   
+ *   (C) Alex Beregszaszi <alex@naxine.org>
+ *   
+ *   Based on FFmpeg's libav/rm.c.
+ *
+ * $Id: demux_real.c,v 1.39 2003/01/29 18:53:56 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +40,7 @@
 #include <unistd.h>
 #include <string.h>
 #include <stdlib.h>
+#include <ctype.h>
 
 #include "xine_internal.h"
 #include "xineutils.h"
@@ -92,7 +100,6 @@ typedef struct {
   int                  status;
   unsigned int         duration;
   int                  packet_count;
-  int                  bitrate;
 
   int                  video_stream_num;
   uint32_t             video_buf_type;
@@ -115,6 +122,7 @@ typedef struct {
 
   int                  fragment_size; /* video sub-demux */
 
+  int                  reference_mode;
 } demux_real_t ;
 
 typedef struct {
@@ -257,7 +265,9 @@ static void real_parse_headers (demux_real_t *this) {
   this->data_start = 0;
   this->data_size = 0;
 
-  this->input->seek (this->input, 0, SEEK_SET);
+  if ((this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE) != 0) 
+    this->input->seek (this->input, 0, SEEK_SET);
+
   if (this->input->read(this->input, signature, REAL_SIGNATURE_SIZE) !=
       REAL_SIGNATURE_SIZE) {
 
@@ -276,7 +286,7 @@ static void real_parse_headers (demux_real_t *this) {
 
   /* skip to the start of the first chunk (the first chunk is 0x12 bytes
    * long) and start traversing */
-  this->input->seek(this->input, 0x12, SEEK_SET);
+  this->input->seek(this->input, 14, SEEK_CUR);
 
   /* iterate through chunks and gather information until the first DATA
    * chunk is located */
@@ -326,9 +336,6 @@ static void real_parse_headers (demux_real_t *this) {
 
 	mdpr = pnm_parse_mdpr (chunk_buffer);
 
-	this->bitrate = mdpr->avg_bit_rate;
-	this->stream->stream_info[XINE_STREAM_INFO_BITRATE] = mdpr->avg_bit_rate;
-
 #ifdef LOG
 	printf ("demux_real: parsing type specific data...\n");
 #endif
@@ -357,6 +364,8 @@ static void real_parse_headers (demux_real_t *this) {
 		printf ("demux_real: audio detected %.3s\n", 
 			mdpr->type_specific_data+off+4);
 #endif
+		this->stream->stream_info[XINE_STREAM_INFO_AUDIO_BITRATE] = mdpr->avg_bit_rate;
+
 		
 		version = BE_16 (mdpr->type_specific_data+off);
 		
@@ -374,13 +383,13 @@ static void real_parse_headers (demux_real_t *this) {
 		  printf ("demux_real: sample_rate %d\n", sample_rate);
 #endif
 		  
-		  str_len = *(mdpr->type_specific_data+off+50);
+		  str_len = *(mdpr->type_specific_data+off+52);
 		  
 #ifdef LOG
 		  printf ("demux_real: str_len = %d\n", str_len);
 #endif
 		  
-		  memcpy (fourcc, mdpr->type_specific_data+off+53+str_len, 4);
+		  memcpy (fourcc, mdpr->type_specific_data+off+54+str_len, 4);
 		  fourcc[4]=0;
 		  
 #ifdef LOG
@@ -399,20 +408,19 @@ static void real_parse_headers (demux_real_t *this) {
 		} else {
 		  printf ("demux_real: error, unknown audio data header version %d\n",
 			  version);
-		  abort();
+
+		  free (mdpr->stream_name);
+		  free (mdpr->mime_type);
+		  free (mdpr->type_specific_data);
+		  free (mdpr);
+
+		  this->status = DEMUX_FINISHED;
+		  return;
 		}
 	      
-		if (!strncmp (fourcc, "dnet", 4)) 
-		  this->audio_buf_type = BUF_AUDIO_DNET;
-		else if (!strncmp (fourcc, "sipr", 4)) 
-		  this->audio_buf_type = BUF_AUDIO_SIPRO;
-		else if (!strncmp (fourcc, "cook", 4))
-		  this->audio_buf_type = BUF_AUDIO_COOK;
-		else if (!strncmp (fourcc, "atrc", 4))
-		  this->audio_buf_type = BUF_AUDIO_ATRK;
-		else 
-		  this->audio_buf_type = 0;
-		
+		this->stream->stream_info[XINE_STREAM_INFO_AUDIO_FOURCC] = *(uint32_t *)fourcc;
+		this->audio_buf_type = formattag_to_buf_audio(*(uint32_t *)fourcc);
+
 #ifdef LOG
 		printf ("demux_real: audio codec, buf type %08x\n",
 			this->audio_buf_type);
@@ -420,7 +428,7 @@ static void real_parse_headers (demux_real_t *this) {
 
 		this->audio_stream_num = mdpr->stream_number;
 
-		if (this->audio_buf_type) {
+		if (this->audio_buf_type && this->audio_fifo) {
 		  buf_element_t *buf;
 		  
 		  /* send header */
@@ -434,8 +442,8 @@ static void real_parse_headers (demux_real_t *this) {
 		  
 		  buf->size = mdpr->type_specific_len-off;
 		  
-		  buf->input_pos     = 0 ; 
-		  buf->input_time    = 0 ; 
+		  buf->extra_info->input_pos     = 0 ; 
+		  buf->extra_info->input_time    = 0 ; 
 		  buf->type          = this->audio_buf_type;
 		  buf->decoder_flags = BUF_FLAG_HEADER;
 		  
@@ -448,92 +456,72 @@ static void real_parse_headers (demux_real_t *this) {
 		break;  /* audio */
 	      } 
 	      if (!strncmp (mdpr->type_specific_data+off, "VIDO", 4)) {
+                const char *video_fmt = (mdpr->type_specific_data + off + 4);
 #ifdef LOG
 		printf ("demux_real: video detected\n");
 #endif
-		/* FIXME: insert video codec detection code here */
-		
+		this->stream->stream_info[XINE_STREAM_INFO_VIDEO_BITRATE] = mdpr->avg_bit_rate;
+
+                this->stream->stream_info[XINE_STREAM_INFO_VIDEO_FOURCC] = *(uint32_t *)video_fmt;
+
+                this->video_buf_type = fourcc_to_buf_video(*(uint32_t *)video_fmt);
+
+		if( this->video_buf_type ) {
+
+                  this->video_stream_num = mdpr->stream_number;
+                  this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
+                } else {
+                  printf("demux_real: video codec [%c%c%c%c] not recognized\n", 
+                         video_fmt[0],video_fmt[1],video_fmt[2],video_fmt[3]);
+                }
+
+                if ( this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] ) {
+                  buf_element_t *buf;
+
+                  buf = this->video_fifo->buffer_pool_alloc(this->video_fifo);
+
+                  buf->content = buf->mem;
+
+		  if (this->video_buf_type == BUF_VIDEO_RV10) {
+		    xine_bmiheader bih;
+
+		    memcpy(&bih.biWidth, mdpr->type_specific_data+off+8, 2);
+		    memcpy(&bih.biHeight, mdpr->type_specific_data+off+10, 2);
+		    bih.biWidth=bswap_16(bih.biWidth);
+		    bih.biHeight=bswap_16(bih.biHeight);
+		    bih.biSize=sizeof(bih);
+#ifdef LOG
+		    printf("demux_real: setting size to w:%u h:%u for RV10\n", bih.biWidth, bih.biHeight);
+#endif
+		    memcpy(buf->content, &bih, bih.biSize);
+
+		    
+		  } else {
+		    memcpy(buf->content, mdpr->type_specific_data,
+			   mdpr->type_specific_len);
+
+		    buf->size = mdpr->type_specific_len;
+		  }
+
+                  buf->type = this->video_buf_type;
+                  buf->decoder_flags = BUF_FLAG_HEADER;
+                  buf->extra_info->input_pos  = 0;
+                  buf->extra_info->input_time = 0;
+
+                  this->video_fifo->put (this->video_fifo, buf);
+                }
+
 		break;  /* video */
 	      }
 	      off++;
-	    }
+	    } /* while */
 	  }
-	
-	
 
-	  /* detect video streamtype */
-
-	  if (!strncmp (mdpr->type_specific_data+4, "VIDORV20", 8)) {
-
-	    buf_element_t *buf;
-
-	    this->video_stream_num = mdpr->stream_number;
-	    this->video_buf_type   = BUF_VIDEO_RV20;
-
-#ifdef LOG
-	    printf ("demux_real: RV20 video detected\n");
-#endif
-
-	    this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-
-	    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-
-	    buf->content = buf->mem;
-
-	    memcpy (buf->content, mdpr->type_specific_data, 
-		    mdpr->type_specific_len);
-
-	    buf->size = mdpr->type_specific_len;
-
-	    buf->input_pos     = 0 ; 
-	    buf->input_time    = 0 ; 
-	    buf->type          = BUF_VIDEO_RV20;
-	    buf->decoder_flags = BUF_FLAG_HEADER;
-    
-	    this->video_fifo->put (this->video_fifo, buf);  
-	
-	    this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-
-	  } else if (!strncmp (mdpr->type_specific_data+4, "VIDORV30", 8)) {
-
-	    buf_element_t *buf;
-
-	    this->video_stream_num = mdpr->stream_number;
-	    this->video_buf_type   = BUF_VIDEO_RV30;
-
-#ifdef LOG
-	    printf ("demux_real: RV30 video detected\n");
-#endif
-
-	    this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-
-	    buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
-
-	    buf->content = buf->mem;
-
-	    memcpy (buf->content, mdpr->type_specific_data, 
-		    mdpr->type_specific_len);
-
-	    buf->size = mdpr->type_specific_len;
-
-	    buf->input_pos     = 0 ; 
-	    buf->input_time    = 0 ; 
-	    buf->type          = BUF_VIDEO_RV30;
-	    buf->decoder_flags = BUF_FLAG_HEADER;
-    
-	    this->video_fifo->put (this->video_fifo, buf);  
-	
-	    this->stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] = 1;
-
-	  }  else {
-
-#ifdef LOG
-	    printf ("demux_real: codec not recognized as video\n");
-#endif
-
-	  }
 	}
 
+	free (mdpr->stream_name);
+	free (mdpr->mime_type);
+	free (mdpr->type_specific_data);
 	free (mdpr);
 
       } else if (chunk_type == CONT_TAG) {
@@ -597,6 +585,9 @@ static void real_parse_headers (demux_real_t *this) {
 
     default:
       /* this should not occur, but in case it does, skip the chunk */
+#ifdef LOG
+      printf("demux_real: skipping a chunk!\n");
+#endif
       this->input->seek(this->input, chunk_size - PREAMBLE_SIZE, SEEK_CUR);
       break;
 
@@ -604,6 +595,77 @@ static void real_parse_headers (demux_real_t *this) {
   }
 }
 
+
+/* very naive approach for parsing ram files. it will extract known
+ * mrls directly so it should work for simple smil files too.
+ * no attempt is made to support smil features:
+ * http://service.real.com/help/library/guides/production/htmfiles/smil.htm
+ */
+static int demux_real_parse_references( demux_real_t *this) {
+
+  char           *buf = NULL;
+  int             buf_size = 0;
+  int             buf_used = 0;
+  int             len, i, j;
+  int             alternative = 0;
+  xine_mrl_reference_data_t *data;
+  xine_event_t    uevent;
+
+
+  /* read file to memory. 
+   * warning: dumb code, but hopefuly ok since reference file is small */
+  do {
+    buf_size += 1024;
+    buf = realloc(buf, buf_size+1);
+    
+    len = this->input->read(this->input, &buf[buf_used], buf_size-buf_used);
+
+    if( len > 0 )
+      buf_used += len;
+      
+    /* 50k of reference file? no way. something must be wrong */
+    if( buf_used > 50*1024 )
+      break;
+  } while( len > 0 );
+  
+  if(buf_used)
+    buf[buf_used] = '\0';
+
+  for(i=0;i<buf_used;i++) {
+    
+    /* "--stop--" is used to have pnm alternative for old real clients 
+     * new real clients will stop processing the file and thus use
+     * rtsp protocol.
+     */
+    if( !strncmp(&buf[i],"--stop--",8) )
+      alternative++;
+      
+    if( !strncmp(&buf[i],"pnm://",6) || !strncmp(&buf[i],"rtsp://",7) ) {
+      for(j=i; buf[j] && buf[j] != '"' && !isspace(buf[j]); j++ )
+        ;
+      j--;
+      buf[j]='\0';
+      printf("demux_real: ref=%s\n", &buf[i]);
+      
+      uevent.type = XINE_EVENT_MRL_REFERENCE;
+      uevent.stream = this->stream;
+      uevent.data_length = strlen(&buf[i])+sizeof(xine_mrl_reference_data_t);
+      data = malloc(uevent.data_length);
+      uevent.data = data;
+      strcpy(data->mrl, &buf[i]);
+      data->alternative = alternative;
+      xine_event_send(this->stream, &uevent);
+      
+      i = j;
+    }
+  }  
+  
+  free(buf);
+  
+  this->status = DEMUX_FINISHED;
+  return this->status;
+}
+        
 /* redefine abs as macro to handle 64-bit diffs.
    i guess llabs may not be available everywhere */
 #define abs(x) ( ((x)<0) ? -(x) : (x) )
@@ -640,18 +702,6 @@ static void check_newpts (demux_real_t *this, int64_t pts, int video, int previe
 
   if (!preview && pts )
     this->last_pts[video] = pts;
-
-#if 0
-  /* use pts for bitrate measurement */
-
-  if (pts>180000) {
-    this->avg_bitrate = this->input->get_current_pos (this->input) * 8 * 90000/ pts;
-
-    if (this->avg_bitrate<1)
-      this->avg_bitrate = 1;
-
-  }
-#endif
 }
 
 static int stream_read_char (demux_real_t *this) {
@@ -677,6 +727,9 @@ static int demux_real_send_chunk(demux_plugin_t *this_gen) {
   int64_t         pts;
   off_t           offset;
 
+  if(this->reference_mode)
+    return demux_real_parse_references(this);
+    
   /* load a header from wherever the stream happens to be pointing */
   if ( (size=this->input->read(this->input, header, DATA_PACKET_HEADER_SIZE)) !=
       DATA_PACKET_HEADER_SIZE) {
@@ -801,9 +854,11 @@ static int demux_real_send_chunk(demux_plugin_t *this_gen) {
       buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
 
       buf->content       = buf->mem;
-      buf->pts           = timestamp;
-      buf->input_pos     = this->input->get_current_pos (this->input);
-      buf->input_time    = buf->input_pos * 8 / this->avg_bitrate ; 
+      buf->pts           = pts;
+      buf->extra_info->input_pos     = this->input->get_current_pos (this->input);
+
+      buf->extra_info->input_time    = (int)((int64_t)buf->extra_info->input_pos 
+                                             * 8 * 1000 / this->avg_bitrate); 
       buf->type          = this->video_buf_type;
       
       check_newpts (this, pts, PTS_VIDEO, 0);
@@ -873,17 +928,22 @@ static int demux_real_send_chunk(demux_plugin_t *this_gen) {
       
     } /* while(size>2) */
 
-  } else if (stream == this->audio_stream_num) {
+  } else if (stream == this->audio_stream_num && this->audio_fifo) {
 
     buf_element_t *buf;
     int            n;
 
+#ifdef LOG
+    printf ("demux_real: audio chunk detected.\n");
+#endif
+
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
 
     buf->content       = buf->mem;
-    buf->pts           = timestamp;
-    buf->input_pos     = this->input->get_current_pos (this->input);
-    buf->input_time    = buf->input_pos * 8 / this->avg_bitrate ; 
+    buf->pts           = pts;
+    buf->extra_info->input_pos     = this->input->get_current_pos (this->input);
+    buf->extra_info->input_time    = (int)((int64_t)buf->extra_info->input_pos 
+                                           * 8 * 1000 / this->avg_bitrate); 
     buf->type          = this->audio_buf_type;
     buf->decoder_flags = 0;
     buf->size          = size;
@@ -892,7 +952,7 @@ static int demux_real_send_chunk(demux_plugin_t *this_gen) {
 
     n = this->input->read (this->input, buf->content, size);
 
-    if (n<size) {
+    if (n<size || size < 0) {
       printf ("demux_real: read error 44\n");
 
       buf->free_buffer(buf);
@@ -907,7 +967,9 @@ static int demux_real_send_chunk(demux_plugin_t *this_gen) {
   } else {
 
     /* discard */
-
+#ifdef LOG
+    printf ("demux_real: chunk not detected; discarding.\n");
+#endif
     this->input->seek(this->input, size, SEEK_CUR);
 
   }
@@ -981,14 +1043,12 @@ static void demux_real_send_headers(demux_plugin_t *this_gen) {
   this->video_stream_num = -1;
   this->audio_stream_num = -1;
 
-  real_parse_headers (this);
-
-
-  /* print vital stats */
-  xine_log (this->stream->xine, XINE_LOG_MSG,
-    _("demux_real: Real media file, running time: %d min, %d sec\n"),
-    this->duration / 1000 / 60,
-    this->duration / 1000 % 60);
+  if( !this->reference_mode ) {
+    real_parse_headers (this);
+  } else {
+    if ((this->input->get_capabilities (this->input) & INPUT_CAP_SEEKABLE) != 0) 
+      this->input->seek (this->input, 0, SEEK_SET);
+  }
 }
 
 static int demux_real_seek (demux_plugin_t *this_gen,
@@ -1031,7 +1091,7 @@ static int demux_real_get_stream_length (demux_plugin_t *this_gen) {
   demux_real_t *this = (demux_real_t *) this_gen;
 
   /* duration is stored in the file as milliseconds */
-  return this->duration / 1000;
+  return this->duration;
 }
 
 static uint32_t demux_real_get_capabilities(demux_plugin_t *this_gen) {
@@ -1043,6 +1103,26 @@ static int demux_real_get_optional_data(demux_plugin_t *this_gen,
   return DEMUX_OPTIONAL_UNSUPPORTED;
 }
 
+/* help function to discover stream type. returns:
+ *  0 if not known.
+ *  1 if normal stream.
+ *  2 if reference stream.
+ */
+static int real_check_stream_type(uint8_t *buf, int len)
+{
+  if ((buf[0] == 0x2e)
+      && (buf[1] == 'R')
+      && (buf[2] == 'M')
+      && (buf[3] == 'F'))
+    return 1;
+  
+  buf[len] = '\0';
+  if( strstr(buf,"pnm://") || strstr(buf,"rtsp://") || strstr(buf,"<smil>") )
+    return 2;
+    
+  return 0;
+}
+
 static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *stream,
                                     input_plugin_t *input_gen) {
 
@@ -1053,46 +1133,40 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   case METHOD_BY_CONTENT:
     {
-      uint8_t buf[4096];
+      uint8_t buf[4096], len;
 
       if ((input->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0) {
 
 	input->seek(input, 0, SEEK_SET);
 
-	if (input->read(input, buf, 4)) {
+	if ((len = input->read(input, buf, 1024))) {
 
 #ifdef LOG
 	  printf ("demux_real: input seekable, read 4 bytes: %02x %02x %02x %02x\n",
 		  buf[0], buf[1], buf[2], buf[3]);
 #endif
 
-	  if ((buf[0] != 0x2e)
-	      || (buf[1] != 'R')
-	      || (buf[2] != 'M')
-	      || (buf[3] != 'F')) 
+	  if (!real_check_stream_type(buf,len))
 	    return NULL;
 	} else
 	  return NULL;
 
-      } else if (input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW)) {
+      } else if ((len = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW))) {
 	
 #ifdef LOG
 	printf ("demux_real: input provides preview, read 4 bytes: %02x %02x %02x %02x\n",
 		buf[0], buf[1], buf[2], buf[3]);
 #endif
 
-	if ((buf[0] != 0x2e)
-	    || (buf[1] != 'R')
-	    || (buf[2] != 'M')
-	    || (buf[3] != 'F')) 
+	if (!real_check_stream_type(buf,len))
 	  return NULL;
       } else
 	return NULL;
     }
     
-
+#ifdef LOG
     printf ("demux_real: by content accepted.\n");
-
+#endif
   break;
 
   case METHOD_BY_EXTENSION: {
@@ -1138,6 +1212,32 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
   this->stream = stream;
   this->input  = input;
 
+  
+  /* discover stream type */
+  {
+    uint8_t buf[4096], len;
+    
+    this->reference_mode = 0;
+    if ((len = input->get_capabilities(input) & INPUT_CAP_SEEKABLE) != 0) {
+
+      input->seek(input, 0, SEEK_SET);
+
+      if ( (len = input->read(input, buf, 1024)) > 0) {
+        if (real_check_stream_type(buf,len) == 2)
+          this->reference_mode = 1;
+      }
+
+    } else if ((len = input->get_optional_data (input, buf, INPUT_OPTIONAL_DATA_PREVIEW))) {
+   
+      if (real_check_stream_type(buf,len) == 2)
+        this->reference_mode = 1;
+    }
+    
+    if(this->reference_mode)
+      printf("demux_real: reference stream detected\n");
+  }
+  
+  
   this->demux_plugin.send_headers      = demux_real_send_headers;
   this->demux_plugin.send_chunk        = demux_real_send_chunk;
   this->demux_plugin.seek              = demux_real_seek;
@@ -1152,7 +1252,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   strncpy (this->last_mrl, input->get_mrl (input), 1024);
 
-
   return &this->demux_plugin;
 }
 
@@ -1169,8 +1268,8 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return "audio/x-pn-realaudio: ra, rm, ram: Real Media File;"
-         "audio/x-realaudio: ra: Real Media File;"; 
+  return "audio/x-pn-realaudio: ra, rm, ram: Real Media file;"
+         "audio/x-pn-realaudio-plugin: rpm: Real Media plugin file;"; 
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -1204,6 +1303,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_DEMUX, 18, "real", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_DEMUX, 20, "real", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
