@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: demux_ts.c,v 1.69 2002/12/09 17:33:36 guenter Exp $
+ * $Id: demux_ts.c,v 1.77 2003/01/26 15:56:21 tmmm Exp $
  *
  * Demultiplexer for MPEG2 Transport Streams.
  *
@@ -111,7 +111,7 @@
  * used to pass control information to the decoder.
  *
  * If decoder_info[1] == BUF_SPECIAL_SPU_DVB_DESCRIPTOR then
- * decoder_info[2] either points to a spu_dvb_descriptor_t or is 0.
+ * decoder_info_ptr[2] either points to a spu_dvb_descriptor_t or is NULL.
  *
  * If it is 0, the user has disabled the subtitling, or has selected a
  * channel that is not present in the stream.  The decoder should
@@ -132,16 +132,12 @@
 #include <stdlib.h>
 #include <fcntl.h>
 #include <unistd.h>
-#include <pthread.h>
-#include <sched.h>
 #include <string.h>
 
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "demux.h"
 
-#define VALID_MRLS   "fifo,stdin,dvb,tcp"
-
 /*
 #define TS_LOG
 */
@@ -190,6 +186,11 @@
 #define ISO13522_STREAM  0xF3
 #define PROG_STREAM_DIR  0xFF
 
+#define WRAP_THRESHOLD       120000
+
+#define PTS_AUDIO 0
+#define PTS_VIDEO 1
+
 /*
 **
 ** DATA STRUCTURES
@@ -257,8 +258,6 @@ typedef struct {
    */
   unsigned int     programNumber;
   unsigned int     pcrPid;
-  int64_t          PCR;
-  int64_t          last_PCR;
   unsigned int     pid;
   unsigned int     pid_count;
   unsigned int     videoPid;
@@ -268,7 +267,7 @@ typedef struct {
   char             audioLang[4];
   
   int              send_end_buffers;
-  int              ignore_scr_discont;
+  int64_t          last_pts[2];
   int              send_newpts;
   int              buf_flag_seek;
 
@@ -285,6 +284,9 @@ typedef struct {
   demux_ts_spu_lang spu_langs[MAX_NO_SPU_LANGS];
   int               no_spu_langs;
   int               current_spu_channel;
+
+  /* dvb */
+  xine_event_queue_t *event_queue;
 } demux_ts_t;
 
 typedef struct {
@@ -320,14 +322,23 @@ static uint32_t demux_ts_compute_crc32(demux_ts_t*this, uint8_t *data,
   return crc32;
 }
 
-static void check_newpts( demux_ts_t*this, int64_t pts ) {
+/* redefine abs as macro to handle 64-bit diffs.
+   i guess llabs may not be available everywhere */
+#define abs(x) ( ((x)<0) ? -(x) : (x) )
+
+static void check_newpts( demux_ts_t *this, int64_t pts, int video )
+{
+  int64_t diff;
 
 #ifdef TS_LOG
   printf ("demux_ts: check_newpts %lld, send_newpts %d, buf_flag_seek %d\n",
 	  pts, this->send_newpts, this->buf_flag_seek);
 #endif
-  
-  if (this->send_newpts && pts) {
+
+  diff = pts - this->last_pts[video];
+
+  if( pts &&
+      (this->send_newpts || (this->last_pts[video] && abs(diff)>WRAP_THRESHOLD) ) ) {
 
     if (this->buf_flag_seek) {
       xine_demux_control_newpts(this->stream, pts, BUF_FLAG_SEEK);
@@ -336,8 +347,11 @@ static void check_newpts( demux_ts_t*this, int64_t pts ) {
       xine_demux_control_newpts(this->stream, pts, 0);
     }
     this->send_newpts = 0;
-    this->ignore_scr_discont = 1;
+    this->last_pts[1-video] = 0;
   }
+
+  if( pts )
+    this->last_pts[video] = pts;
 }
 
 
@@ -367,7 +381,8 @@ static void demux_ts_update_spu_channel(demux_ts_t *this)
     {
       demux_ts_spu_lang *lang = &this->spu_langs[this->current_spu_channel];
 
-      buf->decoder_info[2] = (uint32_t) &(lang->desc);
+      buf->decoder_info[2] = sizeof(lang->desc);
+      buf->decoder_info_ptr[2] = &(lang->desc);
 
       this->spu_pid = lang->pid;
       this->spu_media = lang->media_index;
@@ -379,7 +394,7 @@ static void demux_ts_update_spu_channel(demux_ts_t *this)
     }
   else
     {
-      buf->decoder_info[2] = 0;
+      buf->decoder_info_ptr[2] = NULL;
 
       this->spu_pid = INVALID_PID;
 
@@ -748,7 +763,7 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
       }
       m->buf->pts = m->pts;
       m->buf->decoder_info[0] = 1;
-      m->buf->input_pos = this->input->get_current_pos(this->input);
+      m->buf->extra_info->input_pos = this->input->get_current_pos(this->input);
       m->fifo->put(m->fifo, m->buf);
       m->buffered_bytes = 0;
       m->buf = NULL; /* forget about buf -- not our responsibility anymore */
@@ -763,7 +778,6 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
 
     } else {
 
-      check_newpts(this, m->pts);
       m->corrupted_pes = 0;
       m->buf = m->fifo->buffer_pool_alloc(m->fifo);
       memcpy(m->buf->mem, ts+len-m->size, m->size);
@@ -778,7 +792,7 @@ static void demux_ts_buffer_pes(demux_ts_t*this, unsigned char *ts,
       m->buf->type = m->type;
       m->buf->pts = m->pts;
       m->buf->decoder_info[0] = 1;
-      m->buf->input_pos = this->input->get_current_pos(this->input);
+      m->buf->extra_info->input_pos = this->input->get_current_pos(this->input);
       m->fifo->put(m->fifo, m->buf);
       m->buffered_bytes = 0;
       m->buf = m->fifo->buffer_pool_alloc(m->fifo);
@@ -1453,35 +1467,7 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
   if( adaptation_field_control & 0x2 ){
     uint32_t adaptation_field_length = originalPkt[4];
     if (adaptation_field_length > 0) {
-      this->PCR = demux_ts_adaptation_field_parse (originalPkt+5,
-						   adaptation_field_length);
-
-      if (pid == this->pcrPid && this->PCR)  {  
-	int64_t scr_diff = this->PCR - this->last_PCR;
-	
-	/* note: comparing (abs(scr_diff) > 90000) isn't reliable
-         * below because abs() give inconsistent results on int64_t
-         * types */
-	if ( (scr_diff > 90000 || scr_diff < -90000) && !this->send_newpts && 
-	     !this->ignore_scr_discont ) {
-	  
-	  buf_element_t *buf;
-	  
-	  buf = this->video_fifo->buffer_pool_alloc(this->video_fifo);
-	  buf->type = BUF_CONTROL_DISCONTINUITY;
-	  buf->disc_off = scr_diff;
-	  this->video_fifo->put (this->video_fifo, buf);
-	  
-	  if (this->audio_fifo) {
-	    buf = this->audio_fifo->buffer_pool_alloc(this->audio_fifo);
-	    buf->type = BUF_CONTROL_DISCONTINUITY;
-	    buf->disc_off = scr_diff;
-	    this->audio_fifo->put (this->audio_fifo, buf);
-	  }
-	}
-	this->last_PCR = this->PCR;
-	this->ignore_scr_discont = 0;
-      }
+      demux_ts_adaptation_field_parse (originalPkt+5, adaptation_field_length);
     }
     /*
      * Skip adaptation header.
@@ -1503,8 +1489,7 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
     
     int t = originalPkt[data_offset+3];
     
-    switch (t) {
-    case VIDEO_STREAM_S ... VIDEO_STREAM_E:
+    if (t >= VIDEO_STREAM_S && t <= VIDEO_STREAM_E) {
       if ( this->videoPid == INVALID_PID) {
 
 	printf ("demux_ts: auto-detected video pid %d\n",
@@ -1514,9 +1499,8 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 	this->videoMedia = this->media_num;
 	demux_ts_pes_new(this, this->media_num++, pid, this->video_fifo, t);
       }
-      break;
-    case PRIVATE_STREAM1:
-    case AUDIO_STREAM_S ... AUDIO_STREAM_E:
+    } else if (t == PRIVATE_STREAM1 ||
+	       (t >= AUDIO_STREAM_S && t <= AUDIO_STREAM_E)) {
       if ( this->audioPid == INVALID_PID) {
 
 	printf ("demux_ts: auto-detected audio pid %d\n",
@@ -1526,7 +1510,6 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 	this->audioMedia = this->media_num;
 	demux_ts_pes_new(this, this->media_num++, pid, this->audio_fifo, t);
       }
-      break;
     }
   }
   
@@ -1544,6 +1527,7 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 #ifdef TS_LOG
       printf ("demux_ts: Video pid: %.4x\n", pid);
 #endif
+      check_newpts(this, this->media[this->videoMedia].pts, PTS_VIDEO);
       demux_ts_buffer_pes (this, originalPkt+data_offset, this->videoMedia,
 			   payload_unit_start_indicator, continuity_counter,
 			   data_len);
@@ -1553,6 +1537,7 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
 #ifdef TS_LOG
       printf ("demux_ts: Audio pid: %.4x\n", pid);
 #endif
+      check_newpts(this, this->media[this->audioMedia].pts, PTS_AUDIO);
       demux_ts_buffer_pes (this, originalPkt+data_offset, this->audioMedia,
 			   payload_unit_start_indicator, continuity_counter,
 			   data_len);
@@ -1599,6 +1584,33 @@ static void demux_ts_parse_packet (demux_ts_t*this) {
   }
 }
 
+/*
+ * check for pids change events
+ */
+
+static void demux_ts_event_handler (demux_ts_t *this) {
+
+  xine_event_t *event;
+
+  while ((event = xine_event_get (this->event_queue))) {
+
+
+    switch (event->type) {
+
+    case XINE_EVENT_PIDS_CHANGE:
+
+      this->videoPid    = INVALID_PID;
+      this->audioPid    = INVALID_PID;
+      this->media_num   = 0;
+      this->send_newpts = 1;
+
+      break;
+      
+    }
+
+    xine_event_free (event);
+  }
+}
 
 /*
  * send a piece of data down the fifos
@@ -1608,14 +1620,15 @@ static int demux_ts_send_chunk (demux_plugin_t *this_gen) {
 
   demux_ts_t*this = (demux_ts_t*)this_gen;
 
+  demux_ts_event_handler (this);
+
   demux_ts_parse_packet(this);
 
   /* DVBSUB: check if channel has changed.  Dunno if I should, or
    * even could, lock the xine object. */
-  if (this->stream->spu_channel != this->current_spu_channel)
-    {
-      demux_ts_update_spu_channel(this);
-    }
+  if (this->stream->spu_channel != this->current_spu_channel) {
+    demux_ts_update_spu_channel(this);
+  }
 
   return this->status;
 }
@@ -1631,6 +1644,9 @@ static void demux_ts_dispose (demux_plugin_t *this_gen) {
     if (this->media[i].buf != NULL) 
       this->media[i].buf->free_buffer(this->media[i].buf);
   }
+
+  xine_event_dispose_queue (this->event_queue);
+
   free(this_gen);
 }
 
@@ -1663,14 +1679,12 @@ static void demux_ts_send_headers (demux_plugin_t *this_gen) {
   this->input->seek (this->input, 0, SEEK_SET);
 
   this->send_newpts = 1;
-  this->ignore_scr_discont = 0;
   
   demux_ts_build_crc32_table (this);
   
   this->status = DEMUX_OK ;
 
   this->send_end_buffers  = 1;
-  this->last_PCR          = 0;
   this->scrambled_npids   = 0;
   
   /* DVBSUB */
@@ -1699,7 +1713,6 @@ static int demux_ts_seek (demux_plugin_t *this_gen,
   }
 
   this->send_newpts = 1;
-  this->ignore_scr_discont = 0;
   
   for (i=0; i<MAX_PIDS; i++) {
     demux_ts_media *m = &this->media[i];
@@ -1732,7 +1745,8 @@ static int demux_ts_get_stream_length (demux_plugin_t *this_gen) {
   demux_ts_t*this = (demux_ts_t*)this_gen;
 
   if (this->rate)
-    return this->input->get_length (this->input) / (this->rate * 50);
+    return (int)((int64_t) this->input->get_length (this->input) 
+                 * 1000 / (this->rate * 50));
   else
     return 0;
 }
@@ -1936,7 +1950,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
 
   this->programNumber = INVALID_PROGRAM;
   this->pcrPid = INVALID_PID;
-  this->PCR    = 0;
   this->scrambled_npids = 0;
   this->videoPid = INVALID_PID;
   this->audioPid = INVALID_PID;
@@ -1955,6 +1968,9 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen,
   this->spu_pid = INVALID_PID;
   this->no_spu_langs = 0;
   this->current_spu_channel = this->stream->spu_channel;
+
+  /* dvb */
+  this->event_queue = xine_event_new_queue (this->stream);
   
   return &this->demux_plugin;
 }
@@ -2011,6 +2027,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "mpeg-ts", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_DEMUX, 20, "mpeg-ts", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
