@@ -20,7 +20,7 @@
  * MS WAV File Demuxer by Mike Melanson (melanson@pcisys.net)
  * based on WAV specs that are available far and wide
  *
- * $Id: demux_wav.c,v 1.28 2002/12/08 21:43:52 miguelfreitas Exp $
+ * $Id: demux_wav.c,v 1.36 2003/01/23 16:12:15 miguelfreitas Exp $
  *
  */
 
@@ -124,8 +124,7 @@ static int open_wav_file(demux_wav_t *this) {
   xine_waveformatex_le2me(this->wave);
   this->audio_type = formattag_to_buf_audio(this->wave->wFormatTag);
   if(!this->audio_type) {
-    xine_report_codec(this->stream, XINE_CODEC_AUDIO, this->audio_type, 0, 0);
-    return 0;
+    this->audio_type = BUF_AUDIO_UNKNOWN;
   }
 
   /* traverse through the chunks to find the 'data' chunk */
@@ -172,11 +171,16 @@ static int demux_wav_send_chunk(demux_plugin_t *this_gen) {
   }
 
   while (remaining_sample_bytes) {
+    if(!this->audio_fifo){
+      this->status = DEMUX_FINISHED;
+      break;
+    }
+
     buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
     buf->type = this->audio_type;
-    buf->input_pos = current_file_pos;
-    buf->input_length = this->data_size;
-    buf->input_time = current_pts / 90000;
+    buf->extra_info->input_pos = current_file_pos;
+    buf->extra_info->input_length = this->data_size;
+    buf->extra_info->input_time = current_pts / 90;
     buf->pts = current_pts;
 
     if (remaining_sample_bytes > buf->max_size)
@@ -286,12 +290,12 @@ static int demux_wav_get_status (demux_plugin_t *this_gen) {
   return this->status;
 }
 
-/* return the approximate length in seconds */
+/* return the approximate length in miliseconds */
 static int demux_wav_get_stream_length (demux_plugin_t *this_gen) {
 
   demux_wav_t *this = (demux_wav_t *) this_gen;
 
-  return (int)(this->data_size / this->wave->nAvgBytesPerSec);
+  return (int)((int64_t) this->data_size * 1000 / this->wave->nAvgBytesPerSec);
 }
 
 static uint32_t demux_wav_get_capabilities(demux_plugin_t *this_gen) {
@@ -377,23 +381,6 @@ static demux_plugin_t *open_plugin (demux_class_t *class_gen, xine_stream_t *str
 
   strncpy (this->last_mrl, input->get_mrl (input), 1024);
 
-  /* print vital stats */
-  xine_log(this->stream->xine, XINE_LOG_MSG,
-    _("demux_wav: format 0x%X audio, %d Hz, %d bits/sample, %d %s\n"),
-    this->wave->wFormatTag,
-    this->wave->nSamplesPerSec,
-    this->wave->wBitsPerSample,
-    this->wave->nChannels,
-    ngettext("channel", "channels", this->wave->nChannels));
-  xine_log(this->stream->xine, XINE_LOG_MSG,
-    _("demux_wav: running time = %lld min, %lld sec\n"),
-    this->data_size / this->wave->nAvgBytesPerSec / 60,
-    this->data_size / this->wave->nAvgBytesPerSec % 60);
-  xine_log(this->stream->xine, XINE_LOG_MSG,
-    _("demux_wav: average bytes/sec = %d, block alignment = %d\n"),
-    this->wave->nAvgBytesPerSec,
-    this->wave->nBlockAlign);
-
   /* special block alignment hack so that the demuxer doesn't send
    * packets with individual PCM samples */
   if ((this->wave->nAvgBytesPerSec / this->wave->nBlockAlign) ==
@@ -416,7 +403,10 @@ static char *get_extensions (demux_class_t *this_gen) {
 }
 
 static char *get_mimetypes (demux_class_t *this_gen) {
-  return NULL;
+  return "audio/x-wav: wav: WAV audio;"
+         "audio/wav: wav: WAV audio;"
+         "audio/x-pn-wav: wav: WAV audio;"
+         "audio/x-pn-windows-acm: wav: WAV audio;";
 }
 
 static void class_dispose (demux_class_t *this_gen) {
@@ -426,7 +416,7 @@ static void class_dispose (demux_class_t *this_gen) {
   free (this);
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+void *demux_wav_init_plugin (xine_t *xine, void *data) {
 
   demux_wav_class_t     *this;
 
@@ -448,8 +438,10 @@ static void *init_plugin (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
+#if 0
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "wav", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "wav", XINE_VERSION_CODE, NULL, demux_wav_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
+#endif
