@@ -22,7 +22,7 @@
  * For more information on the MVE file format, visit:
  *   http://www.pcisys.net/~melanson/codecs/
  *
- * $Id: demux_wc3movie.c,v 1.28 2002/12/08 21:43:52 miguelfreitas Exp $
+ * $Id: demux_wc3movie.c,v 1.33 2003/01/17 16:52:40 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -53,7 +53,7 @@
 #define SOND_TAG FOURCC_TAG('S', 'O', 'N', 'D')
 #define PALT_TAG FOURCC_TAG('P', 'A', 'L', 'T')
 #define INDX_TAG FOURCC_TAG('I', 'N', 'D', 'X')
-#define BNAM_TAG FOURCC_TAG('B', 'N', 'A', 'N')
+#define BNAM_TAG FOURCC_TAG('B', 'N', 'A', 'M')
 #define SIZE_TAG FOURCC_TAG('S', 'I', 'Z', 'E')
 #define BRCH_TAG FOURCC_TAG('B', 'R', 'C', 'H')
 #define SHOT_TAG FOURCC_TAG('S', 'H', 'O', 'T')
@@ -217,51 +217,54 @@ static int demux_mve_send_chunk(demux_plugin_t *this_gen) {
       buf->decoder_flags = BUF_FLAG_SPECIAL;
       buf->decoder_info[1] = BUF_SPECIAL_PALETTE;
       buf->decoder_info[2] = PALETTE_SIZE;
-      buf->decoder_info[3] = 
-        (unsigned int)&this->palettes[PALETTE_SIZE * palette_number];
+      buf->decoder_info_ptr[2] = &this->palettes[PALETTE_SIZE * palette_number];
       buf->size = 0;
       buf->type = BUF_VIDEO_WC3;
       this->video_fifo->put (this->video_fifo, buf);
 
     } else if (chunk_tag == AUDI_TAG) {
 
-      audio_pts = this->video_pts - WC3_PTS_INC;
+      if( this->audio_fifo ) {
 
-      while (chunk_size) {
-        buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
-        buf->type = BUF_AUDIO_LPCM_LE;
-        buf->input_pos = current_file_pos;
-        buf->input_length = this->data_size;
-        buf->input_time = audio_pts / 90000;
-        buf->pts = audio_pts;
-
-        if (chunk_size > buf->max_size)
-          buf->size = buf->max_size;
-        else
-          buf->size = chunk_size;
-        chunk_size -= buf->size;
-
-        if (this->input->read(this->input, buf->content, buf->size) !=
-          buf->size) {
-          buf->free_buffer(buf);
-          this->status = DEMUX_FINISHED;
-          break;
+        audio_pts = this->video_pts - WC3_PTS_INC;
+  
+        while (chunk_size) {
+          buf = this->audio_fifo->buffer_pool_alloc (this->audio_fifo);
+          buf->type = BUF_AUDIO_LPCM_LE;
+          buf->extra_info->input_pos = current_file_pos;
+          buf->extra_info->input_length = this->data_size;
+          buf->extra_info->input_time = audio_pts / 90;
+          buf->pts = audio_pts;
+  
+          if (chunk_size > buf->max_size)
+            buf->size = buf->max_size;
+          else
+            buf->size = chunk_size;
+          chunk_size -= buf->size;
+  
+          if (this->input->read(this->input, buf->content, buf->size) !=
+            buf->size) {
+            buf->free_buffer(buf);
+            this->status = DEMUX_FINISHED;
+            break;
+          }
+  
+          if (!chunk_size)
+            buf->decoder_flags |= BUF_FLAG_FRAME_END;
+  
+          this->audio_fifo->put (this->audio_fifo, buf);
         }
-
-        if (!chunk_size)
-          buf->decoder_flags |= BUF_FLAG_FRAME_END;
-
-        this->audio_fifo->put (this->audio_fifo, buf);
+      }else{
+        this->input->seek(this->input, chunk_size, SEEK_CUR);
       }
-
     } else if (chunk_tag == VGA_TAG) {
 
       while (chunk_size) {
         buf = this->video_fifo->buffer_pool_alloc (this->video_fifo);
         buf->type = BUF_VIDEO_WC3;
-        buf->input_pos = current_file_pos;
-        buf->input_length = this->data_size;
-        buf->input_time = this->video_pts / 90000;
+        buf->extra_info->input_pos = current_file_pos;
+        buf->extra_info->input_length = this->data_size;
+        buf->extra_info->input_time = this->video_pts / 90;
         buf->pts = this->video_pts;
 
         if (chunk_size > buf->max_size)
@@ -763,7 +766,7 @@ static void class_dispose (demux_class_t *this_gen) {
   free (this);
 }
 
-static void *init_plugin (xine_t *xine, void *data) {
+void *demux_wc3movie_init_plugin (xine_t *xine, void *data) {
 
   demux_mve_class_t     *this;
 
@@ -785,8 +788,10 @@ static void *init_plugin (xine_t *xine, void *data) {
  * exported plugin catalog entry
  */
 
+#if 0
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_DEMUX, 18, "wc3movie", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_DEMUX, 20, "wc3movie", XINE_VERSION_CODE, NULL, demux_wc3movie_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
+#endif
