@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out_dxr3.c,v 1.64 2002/12/06 01:33:01 miguelfreitas Exp $
+ * $Id: video_out_dxr3.c,v 1.70 2003/01/25 12:07:34 mroi Exp $
  */
  
 /* mpeg1 encoding video out plugin for the dxr3.  
@@ -35,6 +35,9 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
+#if defined(__sun)
+#include <sys/ioccom.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -63,19 +66,31 @@
 #define LOG_OVR 0
 
 
-/* plugin class initialization function */
-static void       *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen);
+/* plugin class initialization functions */
+static void                *dxr3_x11_init_plugin(xine_t *xine, void *visual_gen);
+static void                *dxr3_aa_init_plugin(xine_t *xine, void *visual_gen);
+static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen);
 
 
 /* plugin catalog information */
-static vo_info_t   vo_info_dxr3 = {
+#ifdef HAVE_X11
+static vo_info_t   vo_info_dxr3_x11 = {
   10,                  /* priority        */
   XINE_VISUAL_TYPE_X11 /* visual type     */
 };
+#endif
+
+static vo_info_t   vo_info_dxr3_aa = {
+  10,                  /* priority        */
+  XINE_VISUAL_TYPE_AA  /* visual type     */
+};
 
 plugin_info_t      xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_OUT, 13, "dxr3", XINE_VERSION_CODE, &vo_info_dxr3, &dxr3_vo_init_plugin },
+#ifdef HAVE_X11
+  { PLUGIN_VIDEO_OUT, 14, "dxr3", XINE_VERSION_CODE, &vo_info_dxr3_x11, &dxr3_x11_init_plugin },
+#endif
+  { PLUGIN_VIDEO_OUT, 14, "aadxr3", XINE_VERSION_CODE, &vo_info_dxr3_aa, &dxr3_aa_init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
@@ -125,7 +140,27 @@ static void        dxr3_update_swap_fields(void *data, xine_cfg_entry_t *entry);
 static void        dxr3_update_enhanced_mode(void *this_gen, xine_cfg_entry_t *entry);
 
 
-static void *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen)
+#ifdef HAVE_X11
+static void *dxr3_x11_init_plugin(xine_t *xine, void *visual_gen)
+{
+  dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
+  
+  if (!this) return NULL;
+  this->visual_type = XINE_VISUAL_TYPE_X11;
+  return &this->video_driver_class;
+}
+#endif
+
+static void *dxr3_aa_init_plugin(xine_t *xine, void *visual_gen)
+{
+  dxr3_driver_class_t *this = dxr3_vo_init_plugin(xine, visual_gen);
+  
+  if (!this) return NULL;
+  this->visual_type = XINE_VISUAL_TYPE_AA;
+  return &this->video_driver_class;
+}
+
+static dxr3_driver_class_t *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen)
 {
   dxr3_driver_class_t *this;
   const char *confstr;
@@ -158,7 +193,7 @@ static void *dxr3_vo_init_plugin(xine_t *xine, void *visual_gen)
   
   this->instance                           = 0;
   
-  return &this->video_driver_class;
+  return this;
 }
 
 static char *dxr3_vo_get_identifier(video_driver_class_t *class_gen)
@@ -320,8 +355,11 @@ static vo_driver_t *dxr3_vo_open_plugin(video_driver_class_t *class_gen, const v
   /* overlay or tvout? */
   confnum = config->register_enum(config, "dxr3.videoout_mode", 0, videoout_modes,
     _("Dxr3: videoout mode (tv or overlay)"), NULL, 0, NULL, NULL);
+  if (!(class->visual_type == XINE_VISUAL_TYPE_X11) && confnum > 1)
+    /* no overlay modes when not using X11 -> switch to letterboxed tv */
+    confnum = 0;
 #if LOG_VID
-  printf("video_out_dxr3: overlaymode = %s\n", videoout_modes[confnum]);
+  printf("video_out_dxr3: videomode = %s\n", videoout_modes[confnum]);
 #endif
   switch (confnum) {
   case 0: /* letterboxed tv mode */
@@ -1078,7 +1116,20 @@ static void dxr3_dispose(vo_driver_t *this_gen)
   if(this->overlay_enabled)
     ioctl(this->fd_control, EM8300_IOCTL_OVERLAY_SETMODE, &val);
   close(this->fd_control);
-  if (this->fd_spu) close(this->fd_spu);
+  pthread_mutex_lock(&this->spu_device_lock);
+  if (this->fd_spu) {
+    uint8_t empty_spu[] = {
+      0x00, 0x26, 0x00, 0x08, 0x80, 0x00, 0x00, 0x80,
+      0x00, 0x00, 0x00, 0x20, 0x01, 0x03, 0x00, 0x00,
+      0x04, 0x00, 0x00, 0x05, 0x00, 0x00, 0x01, 0x00,
+      0x00, 0x01, 0x06, 0x00, 0x04, 0x00, 0x07, 0xFF,
+      0x00, 0x01, 0x00, 0x20, 0x02, 0xFF };
+    /* clear any remaining spu */
+    ioctl(this->fd_spu, EM8300_IOCTL_SPU_BUTTON, NULL);
+    write(this->fd_spu, empty_spu, sizeof(empty_spu));
+    close(this->fd_spu);
+  }
+  pthread_mutex_unlock(&this->spu_device_lock);
   pthread_mutex_destroy(&this->spu_device_lock);
   free(this);
 }
