@@ -18,7 +18,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_dvd.c,v 1.122 2002/12/06 18:44:40 miguelfreitas Exp $
+ * $Id: input_dvd.c,v 1.125 2002/12/27 16:47:10 miguelfreitas Exp $
  *
  */
 
@@ -129,9 +129,6 @@
 # endif
 #endif
 
-/* Globals */
-extern int errno;
-
 /* Array to hold MRLs returned by get_autoplay_list */
 #define MAX_DIR_ENTRIES 1250
 #define MAX_STR_LEN     255  
@@ -776,38 +773,8 @@ static char* dvd_plugin_get_mrl (input_plugin_t *this_gen) {
 }
 
 static void flush_buffers(dvd_input_plugin_t *this) {
-  /*
-   * This code comes from xine_demux_flush_engine
-   * ---> so why do it here again??
-   */
-  xine_stream_t *stream = this->stream;  
-  buf_element_t *buf;
-  
-  stream->video_fifo->clear(stream->video_fifo);
 
-  if( stream->audio_fifo )
-    stream->audio_fifo->clear(stream->audio_fifo);
-  
-  buf = stream->video_fifo->buffer_pool_alloc (stream->video_fifo);
-  buf->type            = BUF_CONTROL_RESET_DECODER;
-  stream->video_fifo->put (stream->video_fifo, buf);
-
-  if(stream->audio_fifo) {
-    buf = stream->audio_fifo->buffer_pool_alloc (stream->audio_fifo);
-    buf->type            = BUF_CONTROL_RESET_DECODER;
-    stream->audio_fifo->put (stream->audio_fifo, buf);
-  }
-
-  if (stream->video_out) {
-    stream->video_out->flush(stream->video_out);
-  }
-
-  if (stream->audio_out) {
-    stream->audio_out->flush(stream->audio_out);
-  }
-
- this->stream->xine->clock->adjust_clock(this->stream->xine->clock,
-   this->stream->xine->clock->get_current_time(this->stream->xine->clock) + 30 * 90000 );
+  xine_demux_flush_engine(this->stream);
 }
 
 static void xine_dvd_send_button_update(dvd_input_plugin_t *this, int mode) {
@@ -1678,6 +1645,25 @@ static void *init_class (xine_t *xine, void *data) {
 
 /*
  * $Log: input_dvd.c,v $
+ * Revision 1.125  2002/12/27 16:47:10  miguelfreitas
+ * man errno: "must not be  explicitly  declared; errno  may  be a macro"
+ * (thanks Chris Rankin for noticing)
+ *
+ * Revision 1.124  2002/12/22 23:35:42  miguelfreitas
+ * it doesn't make sense to reimplement flush here.
+ * (this is why xine_demux_flush_engine was created, to avoid redundant code)
+ *
+ * Revision 1.123  2002/12/21 12:56:47  miguelfreitas
+ * - add buf->decoder_info_ptr: portability for systems where pointer has
+ *   different sizeof than integer.
+ * - add extra_info structure to pass informations from input/demuxers down
+ *   to the output frame. this can be used, for example, to pass the frame
+ *   number of a frame (when known by decoder). also, immediate benefict is
+ *   that we now have a slider which really shows the current position of
+ *   the playing stream. new fields can be added to extra_info keeping
+ *   binary compatibility
+ * - bumpy everybody's api versions
+ *
  * Revision 1.122  2002/12/06 18:44:40  miguelfreitas
  * - add still frame hint (untested - i don't have dvd here)
  * - check mrl before allocating plugin context, so it doesn't get initialized for
@@ -1940,6 +1926,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 10, "DVD", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 11, "DVD", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
