@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_file.c,v 1.69 2002/11/20 11:57:43 mroi Exp $
+ * $Id: input_file.c,v 1.74 2003/01/29 18:53:59 miguelfreitas Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -39,8 +39,6 @@
 #include "compat.h"
 #include "input_plugin.h"
 
-extern int errno;
-
 #define MAXFILES      65535
 
 typedef struct {
@@ -66,14 +64,25 @@ typedef struct {
   int               fh;
   char             *mrl;
 
-  FILE             *sub;
-
 } file_input_plugin_t;
 
 
 static uint32_t file_plugin_get_capabilities (input_plugin_t *this_gen) {
 
-  return INPUT_CAP_SEEKABLE | INPUT_CAP_SPULANG;
+  struct stat          buf ;
+  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
+
+  if (this->fh <0)
+    return 0;
+
+  if (fstat (this->fh, &buf) == 0) {
+    if (S_ISREG(buf.st_mode))
+      return INPUT_CAP_SEEKABLE;
+    else
+      return 0;
+  } else
+    perror ("system call fstat");
+  return 0;
 }
 
 
@@ -183,37 +192,6 @@ static char* file_plugin_get_mrl (input_plugin_t *this_gen) {
 static int file_plugin_get_optional_data (input_plugin_t *this_gen, 
 					  void *data, int data_type) {
   
-  file_input_plugin_t *this = (file_input_plugin_t *) this_gen;
-
-#ifdef LOG
-  printf ("input_file: get optional data, type %08x, sub %p\n",
-	  data_type, this->sub);
-#endif
-
-  switch(data_type) {
-  case INPUT_OPTIONAL_DATA_TEXTSPU0:
-    if(this->sub) {
-      FILE **tmp;
-      
-      /* dirty hacks... */
-      tmp = data;
-      *tmp = this->sub;
-      
-      return INPUT_OPTIONAL_SUCCESS;
-    }
-    break;
-    
-  case INPUT_OPTIONAL_DATA_SPULANG:
-    sprintf(data, "%3s", (this->sub) ? "sub" : "none");
-    return INPUT_OPTIONAL_SUCCESS;
-    break;
-    
-  default:
-    return INPUT_OPTIONAL_UNSUPPORTED;
-    break;
-
-  }
-
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
@@ -222,9 +200,6 @@ static void file_plugin_dispose (input_plugin_t *this_gen ) {
 
   close(this->fh);
 
-  if (this->sub) 
-    fclose (this->sub);
-
   free (this->mrl);
 
   free (this);
@@ -260,11 +235,10 @@ static char *decode_uri (char *uri) {
 static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
-  file_input_class_t  *cls = (file_input_class_t *) cls_gen;
+  /* file_input_class_t  *cls = (file_input_class_t *) cls_gen; */
   file_input_plugin_t *this;
-  FILE                *sub;
   char                *mrl = strdup(data);
-  char                *filename, *subtitle;
+  char                *filename;
   int                  fh;
 
   if (!strncasecmp (mrl, "file:", 5)) 
@@ -272,22 +246,6 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   else
     filename = mrl;
 
-  /* FIXME: find a better solution (multiple streams) for text subtitles */
-  subtitle = strrchr (filename, '?');
-  if (subtitle) {
-    *subtitle = 0;
-    subtitle++;
-
-    xine_log (cls->xine, XINE_LOG_MSG,
-	      _("input_file: trying to open subtitle file '%s'\n"),
-	      subtitle);
-
-    sub = fopen (subtitle, "r");
-
-  } else
-    sub = NULL;
-
-
   fh = open (filename, O_RDONLY);
 
   if (fh == -1) {
@@ -299,7 +257,6 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   this->stream = stream;
   this->mrl    = mrl;
   this->fh     = fh;
-  this->sub    = sub;
 
   this->input_plugin.get_capabilities   = file_plugin_get_capabilities;
   this->input_plugin.read               = file_plugin_read;
@@ -876,7 +833,7 @@ static void *init_plugin (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 10, "file", XINE_VERSION_CODE, NULL, init_plugin },
+  { PLUGIN_INPUT, 11, "file", XINE_VERSION_CODE, NULL, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
