@@ -41,8 +41,6 @@
 #include "input_plugin.h"
 #include "net_buf_ctrl.h"
 
-extern int errno;
-
 /*
 #define LOG
 */
@@ -437,9 +435,9 @@ static off_t http_plugin_read (input_plugin_t *this_gen,
         n = read (this->fh, &buf[num_bytes], n);
       }
     }
-      
-    if (n <= 0) {
-      
+
+    /* read errors */
+    if (n < 0) {
       switch (errno) {
       case EAGAIN:
 	xine_log (this->stream->xine, XINE_LOG_MSG, _("input_http: EAGAIN\n"));
@@ -452,6 +450,9 @@ static off_t http_plugin_read (input_plugin_t *this_gen,
     
     num_bytes += n;
     this->curpos += n;
+    
+    /* end of stream */
+    if (!n) break;
   }
   return num_bytes;
 }
@@ -595,10 +596,10 @@ static off_t http_plugin_seek(input_plugin_t *this_gen, off_t offset, int origin
   if ((origin == SEEK_CUR) && (offset >= 0)) {
 
     for (;((int)offset) - BUFSIZE > 0; offset -= BUFSIZE) {
-      this->curpos += http_plugin_read (this_gen, this->seek_buf, BUFSIZE);
+      http_plugin_read (this_gen, this->seek_buf, BUFSIZE);
     }
 
-    this->curpos += http_plugin_read (this_gen, this->seek_buf, offset);
+    http_plugin_read (this_gen, this->seek_buf, offset);
   }
 
   return this->curpos;
@@ -880,13 +881,15 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
 
   this->nbc    = nbc_init (this->stream);
 
+  nbc_set_high_water_mark(this->nbc, 30);
+
   /*
    * fill preview buffer
    */
 
   this->preview_size = http_plugin_read (&this->input_plugin, this->preview,
 					 PREVIEW_SIZE);
-  
+
   this->preview_pos  = 0;
   this->curpos  = 0;
   
@@ -971,7 +974,7 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 10, "http", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 11, "http", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
