@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_mms.c,v 1.26 2002/11/20 11:57:43 mroi Exp $
+ * $Id: input_mms.c,v 1.34 2003/01/18 17:19:45 tmattern Exp $
  *
  * mms input plugin based on work from major mms
  */
@@ -43,29 +43,37 @@
 #include "input_plugin.h"
 
 #include "mms.h"
+#include "mmsh.h"
 #include "net_buf_ctrl.h"
 
 /*
 #define LOG
 */
 
-extern int errno;
+#define PROTOCOL_UNDEFINED 0
+#define PROTOCOL_MMST      1
+#define PROTOCOL_MMSH      2
 
 #if !defined(NDELAY) && defined(O_NDELAY)
 #define FNDELAY O_NDELAY
 #endif
 
-typedef struct {
+/* network bandwidth */
+const uint32_t mms_bandwidths[]={14400,19200,28800,33600,34430,57600,
+                                  115200,262200,393216,524300,1544000,10485800};
 
-  input_class_t     input_class;
-
-  xine_t           *xine;
-} mms_input_class_t;
+const char * mms_bandwidth_strs[]={"14.4 Kbps (Modem)", "19.2 Kbps (Modem)",
+                                   "28.8 Kbps (Modem)", "33.6 Kbps (Modem)",
+                                   "34.4 Kbps (Modem)", "57.6 Kbps (Modem)",
+                                   "115.2 Kbps (ISDN)", "262.2 Kbps (Cable/DSL)",
+                                   "393.2 Kbps (Cable/DSL)","524.3 Kbps (Cable/DSL)",
+                                   "1.5 Mbps (T1)", "10.5 Mbps (LAN)", NULL};
 
 typedef struct {
   input_plugin_t   input_plugin;
 
   mms_t           *mms;
+  mmsh_t          *mmsh;
 
   char            *mrl;
 
@@ -75,22 +83,41 @@ typedef struct {
 
   char             scratch[1025];
 
+  int              bandwidth;
+  int              protocol;       /* mmst or mmsh */
+  
 } mms_input_plugin_t;
 
+typedef struct {
+
+  input_class_t       input_class;
+  
+  mms_input_plugin_t *ip;
+
+  xine_t             *xine;
+} mms_input_class_t;
 
 static off_t mms_plugin_read (input_plugin_t *this_gen, 
                               char *buf, off_t len) {
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
-  off_t               n;
+  off_t               n = 0;
 
 #ifdef LOG
-  printf ("mms_plugin_read: %lld bytes ...\n",
+  printf ("input_mms: mms_plugin_read: %lld bytes ...\n",
           len);
 #endif
 
   nbc_check_buffers (this->nbc);
 
-  n = mms_read (this->mms, buf, len);
+  switch (this->protocol) {
+    case PROTOCOL_MMST:
+      n = mms_read (this->mms, buf, len);
+      break;
+    case PROTOCOL_MMSH:
+      n = mmsh_read (this->mmsh, buf, len);
+      break;
+  }
+              
   this->curpos += n;
 
   return n;
@@ -103,7 +130,7 @@ static buf_element_t *mms_plugin_read_block (input_plugin_t *this_gen,
   int                   total_bytes;
 
 #ifdef LOG
-  printf ("mms_plugin_read_block: %lld bytes...\n",
+  printf ("input_mms: mms_plugin_read_block: %lld bytes...\n",
           todo);
 #endif
 
@@ -127,7 +154,7 @@ static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
   off_t                 dest = this->curpos;
 
 #ifdef LOG
-  printf ("mms_plugin_seek: %lld offset, %d origin...\n",
+  printf ("input_mms: mms_plugin_seek: %lld offset, %d origin...\n",
           offset, origin);
 #endif
 
@@ -152,14 +179,23 @@ static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
   }
 
   while (this->curpos<dest) {
-    int n, diff;
+    int n = 0;
+    int diff;
 
     diff = dest - this->curpos;
 
     if (diff>1024)
       diff = 1024;
 
-    n = mms_read (this->mms, this->scratch, diff);
+    switch (this->protocol) {
+      case PROTOCOL_MMST:
+        n = mms_read (this->mms, this->scratch, diff);
+        break;
+      case PROTOCOL_MMSH:
+        n = mmsh_read (this->mmsh, this->scratch, diff);
+        break;
+    }
+    
     this->curpos += n;
 
     if (n < diff)
@@ -172,12 +208,19 @@ static off_t mms_plugin_seek (input_plugin_t *this_gen, off_t offset, int origin
 
 static off_t mms_plugin_get_length (input_plugin_t *this_gen) {
   mms_input_plugin_t   *this = (mms_input_plugin_t *) this_gen; 
-  off_t                 length;
+  off_t                 length = 0;
 
   if (!this->mms)
     return 0;
 
-  length = mms_get_length (this->mms);
+  switch (this->protocol) {
+    case PROTOCOL_MMST:
+      length = mms_get_length (this->mms);
+      break;
+    case PROTOCOL_MMSH:
+      length = mmsh_get_length (this->mmsh);
+      break;
+  }
 
 #ifdef LOG
   printf ("input_mms: length is %lld\n", length);
@@ -209,8 +252,16 @@ static void mms_plugin_dispose (input_plugin_t *this_gen) {
   mms_input_plugin_t *this = (mms_input_plugin_t *) this_gen;
 
   if (this->mms) {
-    mms_close (this->mms);
-    this->mms = NULL;
+    switch (this->protocol) {
+      case PROTOCOL_MMST:
+        mms_close (this->mms);
+        break;
+      case PROTOCOL_MMSH:
+        mmsh_close (this->mmsh);
+        break;
+    }
+    this->mms  = NULL;
+    this->mmsh = NULL;
   }
 
   if (this->nbc) {
@@ -237,7 +288,14 @@ static int mms_plugin_get_optional_data (input_plugin_t *this_gen,
   switch (data_type) {
 
   case INPUT_OPTIONAL_DATA_PREVIEW:
-    return mms_peek_header (this->mms, data);
+    switch (this->protocol) {
+      case PROTOCOL_MMST:
+        return mms_peek_header (this->mms, data);
+        break;
+      case PROTOCOL_MMSH:
+        return mmsh_peek_header (this->mmsh, data);
+        break;
+    }
     break;
     
   default:
@@ -249,36 +307,85 @@ static int mms_plugin_get_optional_data (input_plugin_t *this_gen,
   return INPUT_OPTIONAL_UNSUPPORTED;
 }
 
+void bandwidth_changed_cb (void *this_gen, xine_cfg_entry_t *entry) {
+  mms_input_class_t *class = (mms_input_class_t*) this_gen;
+
+#ifdef LOG
+  printf ("input_mms: bandwidth_changed_cb %d\n", entry->num_value);
+#endif
+  if(!class)
+   return;
+
+  if(class->ip && ((entry->num_value >= 0) && (entry->num_value <= 11))) {
+    mms_input_plugin_t *this = class->ip;
+
+    this->bandwidth = mms_bandwidths[entry->num_value];
+  }
+}
+
 static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *stream, 
 				    const char *data) {
 
-  /* mms_input_class_t  *cls = (mms_input_class_t *) cls_gen; */
+  mms_input_class_t  *cls = (mms_input_class_t *) cls_gen;
   mms_input_plugin_t *this;
-  mms_t              *mms;
-  char               *mrl = strdup(data);
-
+  mms_t              *mms  = NULL;
+  mmsh_t             *mmsh = NULL;
+  char               *mrl  = strdup(data);
+  xine_cfg_entry_t    bandwidth_entry;
+  int                 protocol;
+  
 #ifdef LOG
   printf ("input_mms: trying to open '%s'\n", mrl);
 #endif
 
-  if (strncasecmp (mrl, "mms://", 6)) {
-    free (mrl);
-    return NULL;
-  }
-
-  mms = mms_connect (stream, mrl);
-
-  if (!mms) {
+  if (!strncasecmp (mrl, "mms://", 6)) {
+    protocol = PROTOCOL_UNDEFINED;
+  } else if (!strncasecmp (mrl, "mmst://", 7)) {
+    protocol =   PROTOCOL_MMST;
+  } else if (!strncasecmp (mrl, "mmsh://", 7)) {
+    protocol =   PROTOCOL_MMSH;
+  } else {
     free (mrl);
     return NULL;
   }
 
   this = (mms_input_plugin_t *) malloc (sizeof (mms_input_plugin_t));
-
-  this->mms    = mms;
-  this->mrl    = mrl; 
-  this->curpos = 0;
-  this->nbc    = nbc_init (stream);
+  cls->ip = this;
+  
+  if (xine_config_lookup_entry (stream->xine, "input.mms_network_bandwidth", 
+                                &bandwidth_entry)) {
+    bandwidth_changed_cb(cls, &bandwidth_entry);
+  }
+    
+  switch (protocol) {
+    case PROTOCOL_UNDEFINED:
+      mms = mms_connect (stream, mrl, this->bandwidth);
+      if (mms) {
+        protocol = PROTOCOL_MMST;
+      } else {
+        mmsh = mmsh_connect (stream, mrl, this->bandwidth);
+        protocol = PROTOCOL_MMSH;
+      }
+      break;
+    case PROTOCOL_MMST:
+      mms = mms_connect (stream, mrl, this->bandwidth);
+      break;
+    case PROTOCOL_MMSH:
+      mmsh = mmsh_connect (stream, mrl, this->bandwidth);
+      break;
+  }
+  
+  if (!mms && !mmsh) {
+    free (mrl);
+    return NULL;
+  }
+  
+  this->mms      = mms;
+  this->mmsh     = mmsh;
+  this->protocol = protocol;
+  this->mrl      = mrl; 
+  this->curpos   = 0;
+  this->nbc      = nbc_init (stream);
 
   this->input_plugin.get_capabilities  = mms_plugin_get_capabilities;
   this->input_plugin.read              = mms_plugin_read;
@@ -291,6 +398,7 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   this->input_plugin.dispose           = mms_plugin_dispose;
   this->input_plugin.get_optional_data = mms_plugin_get_optional_data;
   this->input_plugin.input_class       = cls_gen;
+
   
   return &this->input_plugin;
 }
@@ -320,6 +428,7 @@ static void *init_class (xine_t *xine, void *data) {
   this = (mms_input_class_t *) xine_xmalloc (sizeof (mms_input_class_t));
 
   this->xine   = xine;
+  this->ip                             = NULL;
 
   this->input_class.open_plugin        = open_plugin;
   this->input_class.get_identifier     = mms_class_get_identifier;
@@ -329,6 +438,11 @@ static void *init_class (xine_t *xine, void *data) {
   this->input_class.dispose            = mms_class_dispose;
   this->input_class.eject_media        = NULL;
 
+  xine->config->register_enum(xine->config, "input.mms_network_bandwidth", 10,
+			      (char **)mms_bandwidth_strs,
+			      "Network bandwidth",
+			      NULL, 0, bandwidth_changed_cb, (void*) this);
+  
   return this;
 }
 
@@ -338,7 +452,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 10, "mms", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 11, "mms", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
-
