@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_plugin.h,v 1.39 2002/11/20 11:57:43 mroi Exp $
+ * $Id: input_plugin.h,v 1.43 2002/12/29 16:48:35 mroi Exp $
  */
 
 #ifndef HAVE_INPUT_PLUGIN_H
@@ -29,7 +29,7 @@
 #include "buffer.h"
 #include "configfile.h"
 
-#define INPUT_PLUGIN_IFACE_VERSION   10
+#define INPUT_PLUGIN_IFACE_VERSION   11
  
 typedef struct input_class_s input_class_t ;
 typedef struct input_plugin_s input_plugin_t;
@@ -80,9 +80,21 @@ struct input_class_s {
 struct input_plugin_s {
 
   /*
-   * return capabilities of input source
+   * return capabilities of the current playable entity. See
+   * get_current_pos below for a description of a "playable entity"
+   * Capabilities a created by "OR"ing a mask of constants listed
+   * below which start "INPUT_CAP".  
+   *
+   * depending on the values set, some of the functions below
+   * will or will not get called or should (not) be able to
+   * do certain tasks.
+   *
+   * for example if INPUT_CAP_SEEKABLE is set,
+   * the seek() function is expected to work fully at any time.
+   * however, if the flag is not set, the seek() function should
+   * make a best-effort attempt to seek, e.g. at least
+   * relative forward seeking should work.
    */
-
   uint32_t (*get_capabilities) (input_plugin_t *this);
 
   /*
@@ -115,13 +127,36 @@ struct input_plugin_s {
 
 
   /*
-   * return length of input (-1 => unlimited, e.g. stream)
+   * return number of bytes in the next playable entity or -1 if the
+   * input is unlimited, as would be the case in a network stream.
+   * 
+   * A "playable entity" tends to be the entities listed in a playback
+   * list or the units on which playback control generally works on.
+   * It might be the number of bytes in a VCD "segment" or "track" (if
+   * the track has no "entry" subdivisions), or the number of bytes in
+   * a PS (Program Segment or "Chapter") of a DVD. If there are no
+   * subdivisions of the input medium and it is considered one
+   * indivisible entity, it would be the byte count of that entity;
+   * for example, the length in bytes of an MPEG file.
+
+   * This length information is used, for example when in setting the
+   * absolute or relative play position or possibly calculating the
+   * bit rate.
    */
   off_t (*get_length) (input_plugin_t *this);
 
 
   /*
-   * return block size of input source (if supported, 0 otherwise)
+   * return block size in bytes of next complete playable entity (if
+   * supported, 0 otherwise). See the description above under
+   * get_length for a description of a "complete playable entity".
+   * 
+   * this block size is only used for mpeg streams stored on
+   * a block oriented storage media, e.g. DVDs and VCDs, to speed
+   * up the demuxing process. only set this (and the INPUT_CAP_BLOCK
+   * flag) if this is the case for your input plugin.
+   *
+   * make this function simply return 0 if unsure.
    */
 
   uint32_t (*get_blocksize) (input_plugin_t *this);
@@ -218,9 +253,6 @@ struct input_plugin_s {
 
 #define INPUT_OPTIONAL_DATA_AUDIOLANG 2
 #define INPUT_OPTIONAL_DATA_SPULANG   3
-#define INPUT_OPTIONAL_DATA_TEXTSPU0  4
-#define INPUT_OPTIONAL_DATA_TEXTSPU1  5
-#define INPUT_OPTIONAL_DATA_TEXTSPU2  6
 #define INPUT_OPTIONAL_DATA_PREVIEW   7
 
 #define MAX_MRL_ENTRIES 255
