@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_stdin_fifo.c,v 1.37 2002/11/20 11:57:43 mroi Exp $
+ * $Id: input_stdin_fifo.c,v 1.41 2003/01/17 17:50:20 mroi Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,10 +32,6 @@
 #include <sys/stat.h>
 #include <errno.h>
 
-#if defined(__linux__)
-#include <linux/config.h> /* Check for DEVFS */
-#endif
-
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "input_plugin.h"
@@ -50,8 +46,6 @@
 #define DEFAULT_LOW_WATER_MARK     1
 #define DEFAULT_HIGH_WATER_MARK    5
 
-extern int errno;
-
 typedef struct {
   input_plugin_t   input_plugin;
 
@@ -242,7 +236,8 @@ static void stdin_plugin_dispose (input_plugin_t *this_gen ) {
   if (this->nbc) 
     nbc_close (this->nbc);
 
-  close(this->fh);
+  if (this->fh != STDIN_FILENO)
+    close(this->fh);
   
   free (this->mrl);
   free (this);
@@ -271,7 +266,6 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
   /* stdin_input_class_t  *cls = (stdin_input_class_t *) cls_gen; */
   stdin_input_plugin_t *this;
   char                 *mrl = strdup(data);
-  char                 *filename;
   int                   fh;
 
 
@@ -282,30 +276,29 @@ static input_plugin_t *open_plugin (input_class_t *cls_gen, xine_stream_t *strea
 
   if (!strncasecmp(mrl, "stdin:/", 7) 
       || !strncmp(mrl, "-", 1)) {
-#if defined(CONFIG_DEVFS_FS)
-    filename = "/dev/vc/stdin";
-#else
-    filename = "/dev/stdin";
-#endif
 
-  } else if (!strncasecmp (mrl, "fifo:/", 6)) {
+    fh = STDIN_FILENO;
 
-    filename = (char *) &mrl[6];
+  } else if (!strncasecmp (mrl, "fifo:", 5)) {
+    char *filename;
+
+    filename = (char *) &mrl[5];
     
+    fh = open (filename, O_RDONLY);
+
+    printf("input_stdin_fifo: filename '%s'\n", filename);
+  
+    if (fh == -1) {
+      printf ("stdin: failed to open '%s'\n",
+	      filename);
+      free (mrl);
+      return NULL;
+    }
   } else {
     free (mrl);
     return NULL;
   }
   
-  printf("input_stdin_fifo: filename '%s'\n", filename);
-  fh = open (filename, O_RDONLY);
-  
-  if (fh == -1) {
-    printf ("stdin: failed to open '%s'\n",
-	    filename);
-    free (mrl);
-    return NULL;
-  }
 
   /* 
    * mrl accepted and opened successfully at this point
@@ -393,6 +386,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 10, "stdin", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 11, "stdin", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
