@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: input_vcd.c,v 1.59 2002/11/20 11:57:43 mroi Exp $
+ * $Id: input_vcd.c,v 1.63 2003/01/07 22:51:35 f1rmb Exp $
  *
  */
 
@@ -51,8 +51,6 @@
 #include "xineutils.h"
 #include "input_plugin.h"
 
-extern int errno;
-
 #if defined(__sun)
 #define	CDROM	       "/vol/dev/aliases/cdrom0"
 #else
@@ -229,8 +227,9 @@ static int sun_vcd_read(vcd_input_plugin_t *this, long lba, cdsector_t *data)
    * ATAPI devices.
    */
   if (this->controller_type == 0) {
-    if (ioctl(this->fd, DKIOCINFO, &cinfo) == 0
-	&& strcmp(cinfo.dki_cname, "ide") == 0)
+    if ( ioctl(this->fd, DKIOCINFO, &cinfo) == 0
+	 && ((strcmp(cinfo.dki_cname, "ide") == 0) 
+	     || (strncmp(cinfo.dki_cname, "pci", 3) == 0)) )
       this->controller_type = SUN_CTRL_ATAPI;
     else
       this->controller_type = SUN_CTRL_SCSI;    
@@ -345,7 +344,8 @@ static off_t vcd_plugin_read (input_plugin_t *this_gen,
 
   do
   {  
-    end_msf = &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
+    end_msf = (struct cdrom_msf0 *)
+      &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
 
     /*
     printf ("cur: %02d:%02d:%02d  end: %02d:%02d:%02d\n",
@@ -425,7 +425,8 @@ static off_t vcd_plugin_read (input_plugin_t *this_gen,
 
   do
   {  
-    end_msf = &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
+    end_msf = (struct cdrom_msf0 *) 
+      &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
 
     /*
     printf ("cur: %02d:%02d:%02d  end: %02d:%02d:%02d\n",
@@ -569,7 +570,8 @@ static buf_element_t *vcd_plugin_read_block (input_plugin_t *this_gen,
 
   do
   {  
-    end_msf = &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
+    end_msf = (struct cdrom_msf0 *) 
+      &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
 
     /*
     printf ("cur: %02d:%02d:%02d  end: %02d:%02d:%02d\n",
@@ -634,7 +636,8 @@ static off_t vcd_plugin_seek (input_plugin_t *this_gen,
   struct cdrom_msf0   *start_msf;
   off_t                sector_pos;
 
-  start_msf = &this->cls->tocent[this->cur_track].cdte_addr.msf;
+  start_msf = (struct cdrom_msf0 *)
+    &this->cls->tocent[this->cur_track].cdte_addr.msf;
 
   switch (origin) {
   case SEEK_SET:
@@ -728,24 +731,31 @@ static off_t vcd_plugin_get_length (input_plugin_t *this_gen) {
   vcd_input_plugin_t *this = (vcd_input_plugin_t *) this_gen;
   struct cdrom_msf0       *end_msf, *start_msf;
   off_t len ;
-
-  start_msf = &this->cls->tocent[this->cur_track].cdte_addr.msf;
-  end_msf   = &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
-
-  len = 75 - start_msf->frame;
-
-  if (start_msf->second<60)
-    len += (59 - start_msf->second) * 75;
-
-  if (end_msf->minute > start_msf->minute) {
-    len += (end_msf->minute - start_msf->minute-1) * 60 * 75;
-
-    len += end_msf->second * 60;
-
-    len += end_msf->frame ;
-  }
   
-  return len * VCDSECTORSIZE;
+  if(this->cls->total_tracks) {
+    
+    start_msf = (struct cdrom_msf0 *)
+      &this->cls->tocent[this->cur_track].cdte_addr.msf;
+    end_msf   = (struct cdrom_msf0 *)
+      &this->cls->tocent[this->cur_track+1].cdte_addr.msf;
+    
+    len = 75 - start_msf->frame;
+    
+    if (start_msf->second<60)
+      len += (59 - start_msf->second) * 75;
+    
+    if (end_msf->minute > start_msf->minute) {
+      len += (end_msf->minute - start_msf->minute-1) * 60 * 75;
+      
+      len += end_msf->second * 60;
+      
+      len += end_msf->frame ;
+    }
+    
+    return len * VCDSECTORSIZE;
+  }
+
+  return (off_t) 0;
 }
 #elif defined (__FreeBSD__)
 static off_t vcd_plugin_get_length (input_plugin_t *this_gen) {
@@ -1016,7 +1026,6 @@ static xine_mrl_t **vcd_class_get_dir (input_class_t *this_gen, const char *file
   fd = -1;
 
   *num_files = this->total_tracks - 1;
-  
   /* printf ("%d tracks\n", this->total_tracks); */
 
   for (i=1; i<this->total_tracks; i++) { /* FIXME: check if track 0 contains valid data */
@@ -1148,6 +1157,6 @@ static void *init_class (xine_t *xine, void *data) {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_INPUT, 10, "VCD", XINE_VERSION_CODE, NULL, init_class },
+  { PLUGIN_INPUT, 11, "VCD", XINE_VERSION_CODE, NULL, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
