@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.40 2002/11/20 11:57:43 mroi Exp $
+ * $Id: xine_decoder.c,v 1.49 2003/01/26 18:22:34 mroi Exp $
  *
  * stuff needed to turn liba52 into a xine decoder plugin
  */
@@ -42,6 +42,10 @@
 #include "buffer.h"
 #include "xineutils.h"
 
+/*
+#define LOG
+*/
+
 #undef DEBUG_A52
 #ifdef DEBUG_A52
 int a52file;
@@ -50,6 +54,11 @@ int a52file;
 typedef struct {
   audio_decoder_class_t   decoder_class;
   config_values_t *config;
+  
+  float            a52_level;
+  int              disable_dynrng;
+  int              enable_surround_downmix;
+  
 } a52dec_class_t;
 
 typedef struct a52dec_decoder_s {
@@ -69,21 +78,17 @@ typedef struct a52dec_decoder_s {
   int              a52_flags;
   int              a52_bit_rate;
   int              a52_sample_rate;
-  float            a52_level;
   int              have_lfe;
 
   int              a52_flags_map[11];
   int              ao_flags_map[11];
 
-  xine_audio_port_t *audio_out;
   int              audio_caps;
   int              bypass_mode;
   int              output_sampling_rate;
   int              output_open;
   int              output_mode;
 
-  int              disable_dynrng;
-  int              enable_surround_downmix;
 } a52dec_decoder_t;
 
 struct frmsize_s
@@ -180,7 +185,7 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
   if (!this->bypass_mode) {
 
     int              a52_output_flags, i;
-    sample_t         level = this->a52_level;
+    sample_t         level = this->class->a52_level;
     audio_buffer_t  *buf;
     int16_t         *int_samples;
     sample_t        *samples = a52_samples(this->a52_state);
@@ -201,7 +206,7 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
       return;
     }
 
-    if (this->disable_dynrng)
+    if (this->class->disable_dynrng)
       a52_dynrng (this->a52_state, NULL, NULL);
 
     this->have_lfe = a52_output_flags & A52_LFE;
@@ -219,10 +224,10 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
 	|| (output_mode != this->output_mode)) {
 
       if (this->output_open)
-	this->audio_out->close (this->audio_out, this->stream);
+	this->stream->audio_out->close (this->stream->audio_out, this->stream);
 
 
-      this->output_open = this->audio_out->open (this->audio_out, 
+      this->output_open = this->stream->audio_out->open (this->stream->audio_out, 
 						 this->stream, 16,
 						 this->a52_sample_rate,
 						 output_mode) ;
@@ -239,7 +244,7 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
      * decode a52 and convert/interleave samples
      */
 
-    buf = this->audio_out->get_buffer (this->audio_out);
+    buf = this->stream->audio_out->get_buffer (this->stream->audio_out);
     int_samples = buf->mem;
     buf->num_frames = 256*6;
 
@@ -283,11 +288,14 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
 	printf ("liba52: help - unsupported mode %08x\n", output_mode);
       }
     }
+#ifdef LOG
+    printf ("liba52: %d frames output\n", buf->num_frames);
+#endif
 
     /*  output decoded samples */
 
     buf->vpts       = pts;
-    this->audio_out->put_buffer (this->audio_out, buf, this->stream);
+    this->stream->audio_out->put_buffer (this->stream->audio_out, buf, this->stream);
 
   } else {
 
@@ -301,7 +309,7 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
 
       a52_syncinfo (this->frame_buffer, &flags, &sample_rate, &bit_rate);
 
-      this->output_open = this->audio_out->open (this->audio_out,
+      this->output_open = this->stream->audio_out->open (this->stream->audio_out,
 						 this->stream, 16,
 						 sample_rate,
 						 AO_CAP_MODE_A52) ;
@@ -314,7 +322,7 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
        */
       uint32_t syncword, crc1, fscod,frmsizecod,bsid,bsmod,frame_size;
       uint8_t *data_out,*data_in;
-      audio_buffer_t *buf = this->audio_out->get_buffer (this->audio_out);
+      audio_buffer_t *buf = this->stream->audio_out->get_buffer (this->stream->audio_out);
       data_in=(uint8_t *) this->frame_buffer;
       data_out=(uint8_t *) buf->mem;
       syncword = data_in[0] | (data_in[1] << 8);
@@ -336,7 +344,7 @@ static void a52dec_decode_frame (a52dec_decoder_t *this, int64_t pts) {
       buf->num_frames = 1536;
       buf->vpts       = pts;
 
-      this->audio_out->put_buffer (this->audio_out, buf, this->stream);
+      this->stream->audio_out->put_buffer (this->stream->audio_out, buf, this->stream);
 
     }
   }
@@ -349,8 +357,21 @@ void a52dec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
   uint8_t          *end = buf->content + buf->size;
   uint8_t           byte;
 
+#ifdef LOG
+  printf ("liba52: decode data %d bytes of type %08x, pts=%lld\n",
+	  buf->size, buf->type, buf->pts);
+#endif
+
+  if (buf->decoder_flags & BUF_FLAG_HEADER)
+    return;
+
   /* swap byte pairs if this is RealAudio DNET data */
   if (buf->type == BUF_AUDIO_DNET) {
+
+#ifdef LOG
+    printf ("liba52: byte-swapping dnet\n");
+#endif
+
     while (current != end) {
       byte = *current++;
       *(current - 1) = *current;
@@ -367,9 +388,19 @@ void a52dec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
   while (current != end) {
 
+#ifdef LOG
+    printf ("liba52: processing...\n");
+#endif
+
     if ( (this->sync_todo == 0) && (this->frame_todo == 0) ) {
-      if ((buf->decoder_flags & BUF_FLAG_PREVIEW)==0)
+      if ((buf->decoder_flags & BUF_FLAG_PREVIEW)==0) {
 	a52dec_decode_frame (this, this->pts);
+
+#ifdef LOG
+	printf ("liba52: decode frame\n");
+#endif
+      }
+
 #ifdef DEBUG_A52
       write (a52file, this->frame_buffer, this->frame_length);
 #endif
@@ -383,6 +414,11 @@ void a52dec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 
       if (this->sync_todo>0) {
 
+#ifdef LOG
+	printf ("liba52: looking for syncword %04x\n", this->syncword);
+#endif
+
+
 	/* search and collect syncinfo */
 
 	if (this->syncword != 0x0b77) {
@@ -401,25 +437,39 @@ void a52dec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
 	  this->sync_todo--;
 
 	  if (this->sync_todo==0) {
+	    int a52_flags_old       = this->a52_flags;
+	    int a52_sample_rate_old = this->a52_sample_rate;
+	    int a52_bit_rate_old    = this->a52_bit_rate;
 
 	    this->frame_length = a52_syncinfo (this->frame_buffer,
 					       &this->a52_flags,
 					       &this->a52_sample_rate,
 					       &this->a52_bit_rate);
+#ifdef LOG
+	    printf ("liba52: syncinfo frame_length=%d\n", this->frame_length);
+#endif
+
 	    if (this->frame_length) {
 	      this->frame_todo = this->frame_length - 7;
 
-	      if (!this->stream->meta_info[XINE_META_INFO_AUDIOCODEC]) {
+	      if (!this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] ||
+	          a52_flags_old       != this->a52_flags ||
+		  a52_sample_rate_old != this->a52_sample_rate ||
+		  a52_bit_rate_old    != this->a52_bit_rate) {
 
-		if ((this->a52_flags & A52_3F2R) && (this->a52_flags & A52_LFE))
+		if (((this->a52_flags & A52_CHANNEL_MASK) == A52_3F2R) && (this->a52_flags & A52_LFE))
 		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 5.1");
-		else if ((this->a52_flags & A52_2F2R) || (this->a52_flags & A52_3F1R))
+		else if (((this->a52_flags & A52_CHANNEL_MASK) == A52_2F2R) ||
+			((this->a52_flags & A52_CHANNEL_MASK) == A52_3F1R))
 		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 4.0");
-		else if ((this->a52_flags & A52_2F1R) || (this->a52_flags & A52_3F))
+		else if (((this->a52_flags & A52_CHANNEL_MASK) == A52_2F1R) ||
+			((this->a52_flags & A52_CHANNEL_MASK) == A52_3F))
 		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 3.0");
-		else if (this->a52_flags & A52_STEREO)
-		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 2.0");
-		else if (this->a52_flags & A52_MONO)
+		else if ((this->a52_flags & A52_CHANNEL_MASK) == A52_STEREO)
+		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 2.0 (stereo)");
+		else if ((this->a52_flags & A52_CHANNEL_MASK) == A52_DOLBY)
+		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 2.0 (dolby)");
+		else if ((this->a52_flags & A52_CHANNEL_MASK) == A52_MONO)
 		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52 1.0");
 		else
 		  this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] = strdup ("A/52");
@@ -459,7 +509,7 @@ static void a52dec_dispose (audio_decoder_t *this_gen) {
   a52dec_decoder_t *this = (a52dec_decoder_t *) this_gen;
 
   if (this->output_open)
-    this->audio_out->close (this->audio_out, this->stream);
+    this->stream->audio_out->close (this->stream->audio_out, this->stream);
 
   this->output_open = 0;
 
@@ -475,8 +525,9 @@ static void a52dec_dispose (audio_decoder_t *this_gen) {
 static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stream_t *stream) {
 
   a52dec_decoder_t *this ;
-  config_values_t *cfg;
+#ifdef LOG
   printf ("liba52:open_plugin called\n");
+#endif
 
   this = (a52dec_decoder_t *) malloc (sizeof (a52dec_decoder_t));
   memset(this, 0, sizeof (a52dec_decoder_t));
@@ -487,22 +538,9 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
   this->audio_decoder.dispose             = a52dec_dispose;
   this->stream                            = stream;
   this->class                             = (a52dec_class_t *) class_gen;
-  cfg = this->class->config;
-
-  this->a52_level = (float) cfg->register_range (cfg, "codec.a52_level", 100,
-						 0, 200,
-						 _("a/52 volume control"),
-						 NULL, 0, NULL, NULL) / 100.0;
-  this->disable_dynrng = !cfg->register_bool (cfg, "codec.a52_dynrng", 0,
-					      _("enable a/52 dynamic range compensation"),
-					      NULL, 0, NULL, NULL);
-  this->enable_surround_downmix = cfg->register_bool (cfg, "codec.a52_surround_downmix", 0,
-						      _("enable audio downmixing to 2.0 surround stereo"),
-						      NULL, 0, NULL, NULL);
 
   /* int i; */
 
-  this->audio_out     = stream->audio_out;
   this->audio_caps    = stream->audio_out->get_capabilities(stream->audio_out);
   this->syncword      = 0;
   this->sync_todo     = 7;
@@ -523,13 +561,13 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
     this->bypass_mode = 0;
 
     this->a52_flags_map[A52_MONO]   = A52_MONO;
-    this->a52_flags_map[A52_STEREO] = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
-    this->a52_flags_map[A52_3F]     = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
-    this->a52_flags_map[A52_2F1R]   = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
-    this->a52_flags_map[A52_3F1R]   = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
-    this->a52_flags_map[A52_2F2R]   = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
-    this->a52_flags_map[A52_3F2R]   = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
-    this->a52_flags_map[A52_DOLBY]  = ((this->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_STEREO] = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_3F]     = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_2F1R]   = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_3F1R]   = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_2F2R]   = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_3F2R]   = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
+    this->a52_flags_map[A52_DOLBY]  = ((this->class->enable_surround_downmix ? A52_DOLBY : A52_STEREO));
 
     this->ao_flags_map[A52_MONO]    = AO_CAP_MODE_MONO;
     this->ao_flags_map[A52_STEREO]  = AO_CAP_MODE_STEREO;
@@ -599,23 +637,30 @@ static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, xine_stre
 }
 
 static char *get_identifier (audio_decoder_class_t *this) {
+#ifdef LOG
   printf ("liba52:get_identifier called\n");
+#endif
   return "a/52dec";
 }
 
 static char *get_description (audio_decoder_class_t *this) {
+#ifdef LOG
   printf ("liba52:get_description called\n");
+#endif
   return "liba52 based a52 audio decoder plugin";
 }
 
 static void dispose_class (audio_decoder_class_t *this) {
+#ifdef LOG
   printf ("liba52:dispose_class called\n");
+#endif
   free (this);
 }
 
 static void *init_plugin (xine_t *xine, void *data) {
 
   a52dec_class_t *this;
+  config_values_t *cfg;
 
   this = (a52dec_class_t *) malloc (sizeof (a52dec_class_t));
 
@@ -624,9 +669,23 @@ static void *init_plugin (xine_t *xine, void *data) {
   this->decoder_class.get_description = get_description;
   this->decoder_class.dispose         = dispose_class;
 
-  this->config = xine->config;
+  cfg = this->config = xine->config;
 
+  this->a52_level = (float) cfg->register_range (cfg, "codec.a52_level", 100,
+						 0, 200,
+						 _("a/52 volume control"),
+						 NULL, 0, NULL, NULL) / 100.0;
+  this->disable_dynrng = !cfg->register_bool (cfg, "codec.a52_dynrng", 0,
+					      _("enable a/52 dynamic range compensation"),
+					      NULL, 0, NULL, NULL);
+  this->enable_surround_downmix = cfg->register_bool (cfg, "codec.a52_surround_downmix", 0,
+						      _("enable audio downmixing to 2.0 surround stereo"),
+						      NULL, 0, NULL, NULL);
+  
+  
+#ifdef LOG
   printf ("liba52:init_plugin called\n");
+#endif
   return this;
 }
 
@@ -644,6 +703,6 @@ static decoder_info_t dec_info_audio = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */
-  { PLUGIN_AUDIO_DECODER, 12, "a/52", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
+  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 13, "a/52", XINE_VERSION_CODE, &dec_info_audio, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
