@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_decoder.c,v 1.76 2002/12/08 22:24:28 miguelfreitas Exp $
+ * $Id: xine_decoder.c,v 1.87 2003/01/10 23:33:09 holstsn Exp $
  *
  * xine decoder plugin using ffmpeg
  *
@@ -68,8 +68,9 @@ typedef struct ff_decoder_s {
   int               size;
   int               skipframes;
 
-  AVVideoFrame      *av_picture;
+  AVFrame           *av_frame;
   AVCodecContext    *context;
+  AVCodec           *codec;
 
   /* mpeg sequence header parsing, stolen from libmpeg2 */
 
@@ -101,6 +102,8 @@ typedef struct ff_audio_decoder_s {
   int               size;
 
   AVCodecContext    *context;
+  AVCodec           *codec;
+  
   char              *decode_buffer;
   int               decoder_ok;
 
@@ -114,16 +117,14 @@ static pthread_once_t once_control = PTHREAD_ONCE_INIT;
 #define AUDIOBUFSIZE VIDEOBUFSIZE
 
 
-static void init_video_codec (ff_video_decoder_t *this, AVCodec *codec, xine_bmiheader *bih) {
+static void init_video_codec (ff_video_decoder_t *this, xine_bmiheader *bih) {
 
-#if 0  /* now we have strides, i don't think this is needed anymore */ 
   /* force (width % 8 == 0), otherwise there will be 
    * display problems with Xv. 
    */ 
-  this->bih.biWidth = (this->bih.biWidth + 7) & (~7);
-#endif
+  this->bih.biWidth = (this->bih.biWidth + 1) & (~1);
 
-  this->av_picture = avcodec_alloc_picture();
+  this->av_frame = avcodec_alloc_frame();
   this->context = avcodec_alloc_context();
   this->context->width = this->bih.biWidth;
   this->context->height = this->bih.biHeight;
@@ -137,12 +138,13 @@ static void init_video_codec (ff_video_decoder_t *this, AVCodec *codec, xine_bmi
             this->context->extradata_size ); 
   }
 
-  if (avcodec_open (this->context, codec) < 0) {
+  if (avcodec_open (this->context, this->codec) < 0) {
     printf ("ffmpeg: couldn't open decoder\n");
     free(this->context);
+    this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = 0;
     return;
   }
-  
+
   this->decoder_ok = 1;
   this->stream->video_out->open (this->stream->video_out, this->stream);
 
@@ -217,7 +219,6 @@ static void find_sequence_header (ff_video_decoder_t *this,
     
     if (code == 0xb3) {	/* sequence_header_code */
 
-      AVCodec *codec = NULL;
       int width, height, frame_rate_code;
 
 #ifdef LOG  
@@ -277,14 +278,14 @@ static void find_sequence_header (ff_video_decoder_t *this,
        * init codec
        */
 
-      codec = avcodec_find_decoder (CODEC_ID_MPEG1VIDEO); 
-      if (!codec) {
+      this->codec = avcodec_find_decoder (CODEC_ID_MPEG1VIDEO); 
+      if (!this->codec) {
 	printf ("avcodec_find_decoder (CODEC_ID_MPEG1VIDEO) failed.\n");
 	abort();
       }
 
       this->is_continous = 1;
-      init_video_codec (this, codec, NULL);
+      init_video_codec (this, NULL);
     }
   }
 }
@@ -292,11 +293,13 @@ static void find_sequence_header (ff_video_decoder_t *this,
 static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
   ff_video_decoder_t *this = (ff_video_decoder_t *) this_gen;
   int ratio;
+  int codec_type;
   
 #ifdef LOG
   printf ("ffmpeg: processing packet type = %08x, buf : %p, buf->decoder_flags=%08x\n", 
 	  buf->type, buf, buf->decoder_flags);
 #endif
+  codec_type = buf->type & 0xFFFF0000;
 
   if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
 
@@ -312,9 +315,6 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
   if (buf->decoder_flags & BUF_FLAG_HEADER) {
 
-    AVCodec *codec = NULL;
-    int codec_type;
-
 #ifdef LOG
     printf ("ffmpeg: header\n");
 #endif
@@ -330,69 +330,69 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
     /* init codec */
 
-    codec_type = buf->type & 0xFFFF0000;
+    this->codec = NULL;
 
     switch (codec_type) {
     case BUF_VIDEO_MSMPEG4_V1:
-      codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V1);
+      this->codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V1);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("ms mpeg-4 v1 (ffmpeg)");
       break;
     case BUF_VIDEO_MSMPEG4_V2:
-      codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V2);
+      this->codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V2);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("ms mpeg-4 v2 (ffmpeg)");
       break;
     case BUF_VIDEO_MSMPEG4_V3:
-      codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V3);
+      this->codec = avcodec_find_decoder (CODEC_ID_MSMPEG4V3);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("ms mpeg-4 v3 (ffmpeg)");
       break;
     case BUF_VIDEO_WMV7:
-      codec = avcodec_find_decoder (CODEC_ID_WMV1);
+      this->codec = avcodec_find_decoder (CODEC_ID_WMV1);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("ms wmv 7 (ffmpeg)");
       break;
     case BUF_VIDEO_WMV8:
-      codec = avcodec_find_decoder (CODEC_ID_WMV2);
+      this->codec = avcodec_find_decoder (CODEC_ID_WMV2);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("ms wmv 8 (ffmpeg)");
       break;
     case BUF_VIDEO_MPEG4 :
     case BUF_VIDEO_XVID :
     case BUF_VIDEO_DIVX5 :
-      codec = avcodec_find_decoder (CODEC_ID_MPEG4);
+      this->codec = avcodec_find_decoder (CODEC_ID_MPEG4);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("mpeg-4 (ffmpeg)");
       break;
     case BUF_VIDEO_JPEG:
     case BUF_VIDEO_MJPEG:
-      codec = avcodec_find_decoder (CODEC_ID_MJPEG);
+      this->codec = avcodec_find_decoder (CODEC_ID_MJPEG);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("mjpeg (ffmpeg)");
       break;
     case BUF_VIDEO_I263:
-      codec = avcodec_find_decoder (CODEC_ID_H263I);
+      this->codec = avcodec_find_decoder (CODEC_ID_H263I);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("i263 (ffmpeg)");
       break;
     case BUF_VIDEO_H263:
-      codec = avcodec_find_decoder (CODEC_ID_H263);
+      this->codec = avcodec_find_decoder (CODEC_ID_H263);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("h263 (ffmpeg)");
       break;
     case BUF_VIDEO_RV10:
-      codec = avcodec_find_decoder (CODEC_ID_RV10);
+      this->codec = avcodec_find_decoder (CODEC_ID_RV10);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("real video 1.0 (ffmpeg)");
       break;
     case BUF_VIDEO_SORENSON_V1:
-      codec = avcodec_find_decoder (CODEC_ID_SVQ1);
+      this->codec = avcodec_find_decoder (CODEC_ID_SVQ1);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("sorenson svq 1 (ffmpeg)");
       break;
     case BUF_VIDEO_DV:
-      codec = avcodec_find_decoder (CODEC_ID_DVVIDEO);
+      this->codec = avcodec_find_decoder (CODEC_ID_DVVIDEO);
       this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
 	= strdup ("DV (ffmpeg)");
       break;
@@ -403,12 +403,12 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	= strdup ("unknown (ffmpeg)");
     }
 
-    if (!codec) {
+    if (!this->codec) {
       printf ("ffmpeg: couldn't find decoder\n");
       return;
     }
 
-    init_video_codec (this, codec, (xine_bmiheader *)buf->content );
+    init_video_codec (this, (xine_bmiheader *)buf->content );
 
   } else if (this->decoder_ok) {
 
@@ -418,15 +418,18 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
         this->bufsize);
       this->buf = realloc( this->buf, this->bufsize );
     }
-    
-    xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
+   
+    if(!(buf->decoder_flags & BUF_FLAG_FRAME_START)) {
+      xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
 
-    this->size += buf->size;
+      this->size += buf->size;
+    }
 
     if (buf->decoder_flags & BUF_FLAG_FRAMERATE)
       this->video_step = buf->decoder_info[0];
 
-    if ( (buf->decoder_flags & BUF_FLAG_FRAME_END) || this->is_continous) {
+    if ( (buf->decoder_flags & (BUF_FLAG_FRAME_END|BUF_FLAG_FRAME_START))
+	  || this->is_continous) {
 
       vo_frame_t *img;
       int         got_picture, len, y;
@@ -440,9 +443,14 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 
       offset = 0;
       while (this->size>0) {
-	len = avcodec_decode_video (this->context, this->av_picture,
-				    &got_picture, &this->buf[offset],
-				    this->size);
+        
+        /* DV frames can be completely skipped */
+        if( codec_type == BUF_VIDEO_DV && this->skipframes )
+          len = this->size;
+        else
+	  len = avcodec_decode_video (this->context, this->av_frame,
+				      &got_picture, &this->buf[offset],
+				      this->size);
 	if (len<0) {
 	  printf ("ffmpeg: error decompressing frame\n");
 	  this->size=0;
@@ -494,7 +502,7 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 #endif
 	
 	img = this->stream->video_out->get_frame (this->stream->video_out,
-						  /* this->av_picture.linesize[0],  */
+						  /* this->av_frame.linesize[0],  */
 						  this->bih.biWidth,
 						  this->bih.biHeight,
 						  ratio, 
@@ -514,26 +522,26 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	  dy = img->base[0];
 	  du = img->base[1];
 	  dv = img->base[2];
-	  sy = this->av_picture->data[0];
-	  su = this->av_picture->data[1];
-	  sv = this->av_picture->data[2];
+	  sy = this->av_frame->data[0];
+	  su = this->av_frame->data[1];
+	  sv = this->av_frame->data[2];
 	  
           if (this->context->pix_fmt == PIX_FMT_YUV410P) {
 
             yuv9_to_yv12(
              /* Y */
-              this->av_picture->data[0],
-              this->av_picture->linesize[0],
+              this->av_frame->data[0],
+              this->av_frame->linesize[0],
               img->base[0],
               img->pitches[0],
              /* U */
-              this->av_picture->data[1],
-              this->av_picture->linesize[1],
+              this->av_frame->data[1],
+              this->av_frame->linesize[1],
               img->base[1],
               img->pitches[1],
              /* V */
-              this->av_picture->data[2],
-              this->av_picture->linesize[2],
+              this->av_frame->data[2],
+              this->av_frame->linesize[2],
               img->base[2],
               img->pitches[2],
              /* width x height */
@@ -547,7 +555,7 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	    
 	    dy += img->pitches[0];
 	  
-	    sy += this->av_picture->linesize[0];
+	    sy += this->av_frame->linesize[0];
 	  }
 	
           if (this->context->pix_fmt != PIX_FMT_YUV410P)
@@ -585,11 +593,11 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
 	    dv += img->pitches[2];
 	    
 	    if (this->context->pix_fmt != PIX_FMT_YUV420P) {
-	      su += 2*this->av_picture->linesize[1];
-	      sv += 2*this->av_picture->linesize[2];
+	      su += 2*this->av_frame->linesize[1];
+	      sv += 2*this->av_frame->linesize[2];
 	    } else {
-	      su += this->av_picture->linesize[1];
-	      sv += this->av_picture->linesize[2];
+	      su += this->av_frame->linesize[1];
+	      sv += this->av_frame->linesize[2];
 	    }
 	  }
 	}
@@ -601,6 +609,13 @@ static void ff_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
       
       }
     }
+
+    if(buf->decoder_flags & BUF_FLAG_FRAME_START) {
+      xine_fast_memcpy (&this->buf[this->size], buf->content, buf->size);
+
+      this->size += buf->size;
+    }
+    
   } else {
 #ifdef LOG
     printf ("ffmpeg: data but decoder not initialized (headers missing)\n");
@@ -622,6 +637,7 @@ static void ff_reset (video_decoder_t *this_gen) {
 #endif
 
   this->size = 0;
+  avcodec_flush_buffers(this->context);
 }
 
 static void ff_discontinuity (video_decoder_t *this_gen) {
@@ -654,9 +670,13 @@ void avcodec_register_all(void)
     //    register_avcodec(&dvaudio_decoder);
     register_avcodec(&mjpeg_decoder);
     register_avcodec(&mjpegb_decoder);
+    register_avcodec(&mp2_decoder);
+    register_avcodec(&mp3_decoder);
     register_avcodec(&wmav1_decoder);
     register_avcodec(&wmav2_decoder);
-
+    register_avcodec(&mace3_decoder);
+    register_avcodec(&mace6_decoder);
+    register_avcodec(&huffyuv_decoder);
 }
 
 static void ff_dispose (video_decoder_t *this_gen) {
@@ -679,8 +699,8 @@ static void ff_dispose (video_decoder_t *this_gen) {
   if( this->context )
     free( this->context );
 
-  if( this->av_picture )
-    free( this->av_picture );
+  if( this->av_frame )
+    free( this->av_frame );
   
   if (this->buf)
     free(this->buf);
@@ -773,26 +793,26 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 
   if (buf->decoder_flags & BUF_FLAG_HEADER) {
 
-    AVCodec *codec = NULL;
     int codec_type;
     xine_waveformatex *audio_header = (xine_waveformatex *)buf->content;
 
     codec_type = buf->type & 0xFFFF0000;
+    this->codec = NULL;
 
     switch (codec_type) {
     case BUF_AUDIO_WMAV1:
-      codec = avcodec_find_decoder (CODEC_ID_WMAV1);
+      this->codec = avcodec_find_decoder (CODEC_ID_WMAV1);
       this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
 	= strdup ("Windows Media Audio v1 (ffmpeg)");
       break;
     case BUF_AUDIO_WMAV2:
-      codec = avcodec_find_decoder (CODEC_ID_WMAV2);
+      this->codec = avcodec_find_decoder (CODEC_ID_WMAV2);
       this->stream->meta_info[XINE_META_INFO_AUDIOCODEC] 
 	= strdup ("Windows Media Audio v2 (ffmpeg)");
       break;
     }
 
-    if (!codec) {
+    if (!this->codec) {
       printf (" could not open ffmpeg decoder for buf type 0x%X\n",
         codec_type);
       return;
@@ -805,7 +825,7 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
     this->context->channels = this->audio_channels = buf->decoder_info[3];
     this->context->block_align = audio_header->nBlockAlign;
     this->context->bit_rate = audio_header->nAvgBytesPerSec * 8;
-    this->context->codec_id = codec->id;
+    this->context->codec_id = this->codec->id;
     this->context->fourcc = this->stream->stream_info[XINE_STREAM_INFO_AUDIO_FOURCC];
     if( audio_header->cbSize > 0 ) {
       this->context->extradata = malloc(audio_header->cbSize);
@@ -821,15 +841,16 @@ static void ff_audio_decode_data (audio_decoder_t *this_gen, buf_element_t *buf)
 
     this->decode_buffer = xine_xmalloc(100000);
 
-    if (avcodec_open (this->context, codec) < 0) {
+    if (avcodec_open (this->context, this->codec) < 0) {
       printf ("ffmpeg: couldn't open decoder\n");
+      this->stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = 0;
       return;
     }
 
     this->decoder_ok = 1;
 
     return;
-  } else if (this->decoder_ok) {
+  } else if (this->decoder_ok && !(buf->decoder_flags & BUF_FLAG_SPECIAL)) {
 
     if (!this->output_open) {
       this->output_open = this->stream->audio_out->open(this->stream->audio_out,
@@ -924,6 +945,10 @@ static void ff_audio_reset (audio_decoder_t *this_gen) {
   ff_audio_decoder_t *this = (ff_audio_decoder_t *) this_gen;
   
   this->size = 0;
+
+  /* try to reset the wma decoder */  
+  avcodec_close (this->context);
+  avcodec_open (this->context, this->codec);
 }
 
 static void ff_audio_discontinuity (audio_decoder_t *this_gen) {
@@ -932,6 +957,8 @@ static void ff_audio_discontinuity (audio_decoder_t *this_gen) {
 static void ff_audio_dispose (audio_decoder_t *this_gen) {
 
   ff_audio_decoder_t *this = (ff_audio_decoder_t *) this_gen;
+  
+  avcodec_close (this->context);
 
   if (this->output_open)
     this->stream->audio_out->close (this->stream->audio_out, this->stream);
@@ -1039,7 +1066,7 @@ static decoder_info_t dec_info_ffmpeg_audio = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_DECODER, 13, "ffmpegvideo", XINE_VERSION_CODE, &dec_info_ffmpeg_video, init_video_plugin },
-  { PLUGIN_AUDIO_DECODER, 12, "ffmpegaudio", XINE_VERSION_CODE, &dec_info_ffmpeg_audio, init_audio_plugin },
+  { PLUGIN_VIDEO_DECODER, 14, "ffmpegvideo", XINE_VERSION_CODE, &dec_info_ffmpeg_video, init_video_plugin },
+  { PLUGIN_AUDIO_DECODER, 13, "ffmpegaudio", XINE_VERSION_CODE, &dec_info_ffmpeg_audio, init_audio_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
