@@ -50,8 +50,7 @@
 
 static void process_userdata(mpeg2dec_t *mpeg2dec, uint8_t *buffer);
 
-void mpeg2_init (mpeg2dec_t * mpeg2dec, 
-		 xine_video_port_t * output)
+void mpeg2_init (mpeg2dec_t * mpeg2dec)
 {
   static int do_init = 1;
   uint32_t mm_accel;
@@ -76,7 +75,6 @@ void mpeg2_init (mpeg2dec_t * mpeg2dec,
     mpeg2dec->frames_to_drop = 0;
     mpeg2dec->drop_frame = 0;
     mpeg2dec->in_slice = 0;
-    mpeg2dec->output = output;
     mpeg2dec->chunk_ptr = mpeg2dec->chunk_buffer;
     mpeg2dec->code = 0xb4;
     mpeg2dec->seek_mode = 0;
@@ -214,6 +212,7 @@ static void remember_metainfo (mpeg2dec_t *mpeg2dec) {
   else
     mpeg2dec->stream->meta_info[XINE_META_INFO_VIDEOCODEC] 
       = strdup ("MPEG-2");
+
 }
 
 
@@ -241,8 +240,10 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
     picture = mpeg2dec->picture;
     is_frame_done = mpeg2dec->in_slice && ((!code) || (code >= 0xb0));
 
-    if (is_frame_done && picture->current_frame != NULL) {
+    if (is_frame_done)
 	mpeg2dec->in_slice = 0;
+    
+    if (is_frame_done && picture->current_frame != NULL) {
 
 	if (((picture->picture_structure == FRAME_PICTURE) ||
 	     (picture->second_field)) ) {
@@ -332,10 +333,10 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 	    } else if (!picture->backward_reference_frame || picture->backward_reference_frame->bad_frame) {
 	      mpeg2dec->drop_frame = 1;
 #ifdef LOG
-	      if (picture->backward_reference_frame->bad_frame)
-		printf ("libmpeg2: dropping p-frame because ref %d is bad\n", picture->backward_reference_frame->id);
-	      else
+	      if (!picture->backward_reference_frame)
 		printf ("libmpeg2: dropping p-frame because no ref frame\n");
+	      else
+		printf ("libmpeg2: dropping p-frame because ref %d is bad\n", picture->backward_reference_frame->id);
 #endif
 	    }
 	    break;
@@ -425,7 +426,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 
 	if (!(mpeg2dec->in_slice)) {
 	    mpeg2dec->in_slice = 1;
-
+     
 	    if (picture->second_field) {
 	      if (picture->current_frame)
 		picture->current_frame->field(picture->current_frame, 
@@ -441,7 +442,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 		
 		if (picture->picture_coding_type == B_TYPE)
 		    picture->current_frame =
-		        mpeg2dec->output->get_frame (mpeg2dec->output,
+		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
 						     picture->aspect_ratio_information,
@@ -449,7 +450,7 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 						     picture->picture_structure);
 		else {
 		    picture->current_frame =
-		        mpeg2dec->output->get_frame (mpeg2dec->output,
+		        mpeg2dec->stream->video_out->get_frame (mpeg2dec->stream->video_out,
 						     picture->coded_picture_width,
 						     picture->coded_picture_height,
 						     picture->aspect_ratio_information,
@@ -480,8 +481,13 @@ static inline int parse_chunk (mpeg2dec_t * mpeg2dec, int code,
 
 	if (!mpeg2dec->drop_frame && picture->current_frame != NULL) {
 	  mpeg2_slice (picture, code, buffer);
-	  if( picture->v_offset > picture->limit_y ) 
+
+	  if( picture->v_offset > picture->limit_y ) { 
 	    picture->current_frame->bad_frame = 0;
+#ifdef LOG
+	    printf("libmpeg2: frame %d successfuly decoded\n",picture->current_frame->id); 
+#endif
+	  }
 	}
     }
 
@@ -625,8 +631,10 @@ void mpeg2_flush (mpeg2dec_t * mpeg2dec) {
   if (picture->current_frame && !picture->current_frame->drawn &&
       !picture->current_frame->bad_frame) {
     
+#ifdef LOG
     printf ("libmpeg2: blasting out current frame %d on flush\n",
 	    picture->current_frame->id);
+#endif
     
     picture->current_frame->drawn = 1;
     get_frame_duration(mpeg2dec, picture->current_frame);
@@ -655,7 +663,9 @@ void mpeg2_close (mpeg2dec_t * mpeg2dec)
  
     if ( picture->current_frame ) {
       if( !picture->current_frame->drawn ) {
+#ifdef LOG
         printf ("libmpeg2: blasting out current frame on close\n");
+#endif
         picture->current_frame->pts = 0;
         get_frame_duration(mpeg2dec, picture->current_frame);
         picture->current_frame->draw (picture->current_frame, mpeg2dec->stream);
@@ -677,7 +687,9 @@ void mpeg2_close (mpeg2dec_t * mpeg2dec)
     
     if (picture->backward_reference_frame) {
       if( !picture->backward_reference_frame->drawn) {
+#ifdef LOG
         printf ("libmpeg2: blasting out backward reference frame on close\n");
+#endif
         picture->backward_reference_frame->pts = 0;
         get_frame_duration(mpeg2dec, picture->backward_reference_frame);
         picture->backward_reference_frame->draw (picture->backward_reference_frame, mpeg2dec->stream);
