@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_decoder.c,v 1.9 2002/12/10 19:47:02 guenter Exp $
+ * $Id: audio_decoder.c,v 1.20 2003/01/23 12:47:24 miguelfreitas Exp $
  *
  * thin layer to use real binary-only codecs in xine
  *
@@ -43,14 +43,14 @@
 */
 
 typedef struct {
-  video_decoder_class_t   decoder_class;
+  audio_decoder_class_t   decoder_class;
 
   char                   *real_codec_path;
 
 } real_class_t;
 
 typedef struct realdec_decoder_s {
-  video_decoder_t  video_decoder;
+  audio_decoder_t  audio_decoder;
 
   real_class_t    *cls;
 
@@ -83,6 +83,10 @@ typedef struct realdec_decoder_s {
 
   uint64_t         pts;
 
+  int              output_open;
+  
+  int              decoder_ok;
+
 } realdec_decoder_t;
 
 typedef struct {
@@ -133,6 +137,11 @@ void *__builtin_new(unsigned long size) {
   return malloc(size);
 }
 
+void __builtin_delete (void *foo) {
+  /* printf ("libareal: __builtin_delete called\n"); */
+  free (foo);
+}
+
 static int load_syms_linux (realdec_decoder_t *this, char *codec_name) {
 
   char path[1024];
@@ -311,17 +320,16 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
   }
 
   { 
-    ra_init_t init_data={
-      samples_per_sec,
-      bits_per_sample,
-      num_channels,
-      100, /* ??? */
-      subpacket_size,    /* subpacket size    */
-      coded_frame_size,  /* coded frame size  */
-      data_len,          /* codec data length */
-      extras             /* extras            */
-    };
+    ra_init_t init_data;
 
+    init_data.samplerate = samples_per_sec;
+    init_data.bits = bits_per_sample;
+    init_data.channels = num_channels;
+    init_data.unk1 = 100; /* ??? */
+    init_data.subpacket_size = subpacket_size; /* subpacket size */
+    init_data.coded_frame_size = coded_frame_size; /* coded frame size */
+    init_data.codec_data_length = data_len; /* codec data length */
+    init_data.extras = extras; /* extras */
 
 #ifdef LOG
     printf ("libareal: init_data:\n");
@@ -393,6 +401,8 @@ static int init_codec (realdec_decoder_t *this, buf_element_t *buf) {
 				samples_per_sec,
 				mode) ;
 
+  this->output_open = 1;
+
   this->sample_size = num_channels * (bits_per_sample>>3);
 
   return 1;
@@ -405,11 +415,12 @@ static unsigned char sipr_swaps[38][2]={
     {42,87},{43,65},{45,59},{48,79},{49,93},{51,89},{55,95},{61,76},{67,83},
     {77,80} };
 
-static void realdec_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
+static void realdec_decode_data (audio_decoder_t *this_gen, buf_element_t *buf) {
   realdec_decoder_t *this = (realdec_decoder_t *) this_gen;
 
 #ifdef LOG
-  printf ("libareal: decode_data, flags=0x%08x ...\n", buf->decoder_flags);
+  printf ("libareal: decode_data %d bytes, flags=0x%08x, pts=%lld ...\n", 
+	  buf->size, buf->decoder_flags, buf->pts);
 #endif
 
   if (buf->decoder_flags & BUF_FLAG_PREVIEW) {
@@ -418,9 +429,11 @@ static void realdec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 
   } else if (buf->decoder_flags & BUF_FLAG_HEADER) {
 
-    init_codec (this, buf) ;
+    this->decoder_ok = init_codec (this, buf) ;
+    if( !this->decoder_ok )
+      this->stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = 0;
 
-  } else {
+  } else if( this->decoder_ok ) {
 
     int size;
 
@@ -557,7 +570,7 @@ static void realdec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 	  printf ("libareal: raDecode result %d, len=%d\n", result, len);
 #endif
 
-	  audio_buffer->vpts       = this->pts*90; /* FIXME */
+	  audio_buffer->vpts       = this->pts; 
 
 	  this->pts = 0;
 
@@ -577,40 +590,57 @@ static void realdec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 #endif
 }
 
-static void realdec_flush (video_decoder_t *this_gen) {
-  /* realdec_decoder_t *this = (realdec_decoder_t *) this_gen; */
-
-#ifdef LOG
-  printf ("libareal: flush\n");
-#endif
-
+static void realdec_reset (audio_decoder_t *this_gen) {
+  realdec_decoder_t *this = (realdec_decoder_t *) this_gen;
+  
+  this->frame_num_bytes = 0;
 }
 
-static void realdec_reset (video_decoder_t *this_gen) {
-  /* realdec_decoder_t *this = (realdec_decoder_t *) this_gen; */
-
+static void realdec_discontinuity (audio_decoder_t *this_gen) {
+  realdec_decoder_t *this = (realdec_decoder_t *) this_gen;
+  
+  this->pts = 0;
 }
 
-static void realdec_discontinuity (video_decoder_t *this_gen) {
-  /* realdec_decoder_t *this = (realdec_decoder_t *) this_gen; */
-
-}
-
-static void realdec_dispose (video_decoder_t *this_gen) {
+static void realdec_dispose (audio_decoder_t *this_gen) {
 
   realdec_decoder_t *this = (realdec_decoder_t *) this_gen;
 
 #ifdef LOG
-  printf ("libareal: close\n");
+  printf ("libareal: dispose\n");
+#endif
+
+  if (this->context)
+    this->raCloseCodec (this->context);
+
+#if 0
+  printf ("libareal: FreeDecoder...\n");
+
+  if (this->context)
+    this->raFreeDecoder (this->context);
+#endif
+
+#ifdef LOG
+  printf ("libareal: dlclose...\n");
 #endif
 
   if (this->ra_handle)
     dlclose (this->ra_handle);
 
+  if (this->output_open)
+     this->stream->audio_out->close (this->stream->audio_out, this->stream);
+
+  if (this->frame_buffer)
+    free (this->frame_buffer);
+
   free (this);
+
+#ifdef LOG
+  printf ("libareal: dispose done\n");
+#endif
 }
 
-static video_decoder_t *open_plugin (video_decoder_class_t *class_gen, 
+static audio_decoder_t *open_plugin (audio_decoder_class_t *class_gen, 
 				     xine_stream_t *stream) {
 
   real_class_t      *cls = (real_class_t *) class_gen;
@@ -619,30 +649,31 @@ static video_decoder_t *open_plugin (video_decoder_class_t *class_gen,
   this = (realdec_decoder_t *) malloc (sizeof (realdec_decoder_t));
   memset(this, 0, sizeof (realdec_decoder_t));
 
-  this->video_decoder.decode_data         = realdec_decode_data;
-  this->video_decoder.flush               = realdec_flush;
-  this->video_decoder.reset               = realdec_reset;
-  this->video_decoder.discontinuity       = realdec_discontinuity;
-  this->video_decoder.dispose             = realdec_dispose;
+  this->audio_decoder.decode_data         = realdec_decode_data;
+  this->audio_decoder.reset               = realdec_reset;
+  this->audio_decoder.discontinuity       = realdec_discontinuity;
+  this->audio_decoder.dispose             = realdec_dispose;
   this->stream                            = stream;
   this->cls                               = cls;
 
-  return &this->video_decoder;
+  this->output_open = 0;
+
+  return &this->audio_decoder;
 }
 
 /*
  * real plugin class
  */
 
-static char *get_identifier (video_decoder_class_t *this) {
+static char *get_identifier (audio_decoder_class_t *this) {
   return "realadec";
 }
 
-static char *get_description (video_decoder_class_t *this) {
+static char *get_description (audio_decoder_class_t *this) {
   return "real binary-only codec based audio decoder plugin";
 }
 
-static void dispose_class (video_decoder_class_t *this) {
+static void dispose_class (audio_decoder_class_t *this) {
   free (this);
 }
 
@@ -699,9 +730,15 @@ static void *init_class (xine_t *xine, void *data) {
     if (!stat ("/usr/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
       config->update_string (config, "codec.real_codecs_path", 
 			     "/usr/RealPlayer8/Codecs");
+    if (!stat ("/usr/lib/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
+      config->update_string (config, "codec.real_codecs_path", 
+			     "/usr/lib/RealPlayer8/Codecs");
     if (!stat ("/opt/RealPlayer8/Codecs/drv3.so.6.0", &s)) 
       config->update_string (config, "codec.real_codecs_path", 
 			     "/opt/RealPlayer8/Codecs");
+    if (!stat ("/usr/lib/RealPlayer9/users/Real/Codecs/drv3.so.6.0", &s)) 
+      config->update_string (config, "codec.real_codecs_path", 
+			     "/usr/lib/RealPlayer9/users/Real/Codecs");
   }
 
 #ifdef LOG
@@ -726,6 +763,6 @@ static decoder_info_t dec_info_audio = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_AUDIO_DECODER, 12, "realadec", XINE_VERSION_CODE, &dec_info_audio, init_class },
+  { PLUGIN_AUDIO_DECODER | PLUGIN_MUST_PRELOAD, 13, "realadec", XINE_VERSION_CODE, &dec_info_audio, init_class },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
