@@ -23,7 +23,7 @@
  * avoid when implementing a FLI decoder, visit:
  *   http://www.pcisys.net/~melanson/codecs/
  * 
- * $Id: fli.c,v 1.11 2002/12/06 01:44:06 miguelfreitas Exp $
+ * $Id: fli.c,v 1.17 2003/01/08 01:02:31 miguelfreitas Exp $
  */
 
 #include <stdio.h>
@@ -104,7 +104,7 @@ void decode_fli_frame(fli_decoder_t *this) {
   int color_packets;
   int color_changes;
   int color_shift;
-  unsigned r, g, b;
+  unsigned char r, g, b;
 
   int lines, x;
   int compressed_lines;
@@ -116,6 +116,7 @@ void decode_fli_frame(fli_decoder_t *this) {
   int update_whole_frame = 0;   /* palette change flag */
   int ghost_pixel_ptr;
   int ghost_y_ptr;
+  int pixel_countdown;
 
   frame_size = LE_32(&this->buf[stream_ptr]);
   stream_ptr += 6;  /* skip the magic number */
@@ -145,22 +146,27 @@ void decode_fli_frame(fli_decoder_t *this) {
       for (i = 0; i < color_packets; i++) {
         /* first byte is how many colors to skip */
         palette_ptr1 += (this->buf[stream_ptr++] * 4);
-        /* wrap around, for good measure */
-        if (palette_ptr1 >= PALETTE_SIZE)
-          palette_ptr1 = 0;
+
         /* next byte indicates how many entries to change */
         color_changes = this->buf[stream_ptr++];
+
         /* if there are 0 color changes, there are actually 256 */
         if (color_changes == 0)
           color_changes = 256;
+
         for (j = 0; j < color_changes; j++) {
+
+          /* wrap around, for good measure */
+          if (palette_ptr1 >= PALETTE_SIZE)
+            palette_ptr1 = 0;
+
           r = this->buf[stream_ptr + 0] << color_shift;
           g = this->buf[stream_ptr + 1] << color_shift;
           b = this->buf[stream_ptr + 2] << color_shift;
 
-          this->yuv_palette[palette_ptr1++] = COMPUTE_Y(r,g, b);
-          this->yuv_palette[palette_ptr1++] = COMPUTE_U(r,g, b);
-          this->yuv_palette[palette_ptr1++] = COMPUTE_V(r,g, b);
+          this->yuv_palette[palette_ptr1++] = COMPUTE_Y(r, g, b);
+          this->yuv_palette[palette_ptr1++] = COMPUTE_U(r, g, b);
+          this->yuv_palette[palette_ptr1++] = COMPUTE_V(r, g, b);
 
           palette_ptr1++;
           stream_ptr += 3;
@@ -306,44 +312,42 @@ void decode_fli_frame(fli_decoder_t *this) {
       break;
 
     case FLI_BRUN:
-      /* byte run compression */
-      y_ptr = 0;
+      /* Byte run compression: This chunk type only occurs in the first
+       * FLI frame and it will update the entire frame. Take a lazy
+       * approach and update only the ghost image. Then let the ghost
+       * image updater at the end of the decoder handle the rest. */
+      update_whole_frame = 1;
       ghost_y_ptr = 0;
       for (lines = 0; lines < this->height; lines++) {
-        pixel_ptr = y_ptr;
         ghost_pixel_ptr = ghost_y_ptr;
-        line_packets = this->buf[stream_ptr++];
-        for (i = 0; i < line_packets; i++) {
+        /* disregard the line packets; instead, iterate through all
+         * pixels on a row */
+        stream_ptr++;
+        pixel_countdown = this->width;
+        while (pixel_countdown > 0) {
           byte_run = this->buf[stream_ptr++];
           if (byte_run > 0) {
             palette_ptr1 = (palette_idx1 = this->buf[stream_ptr++]) * 4;
             for (j = 0; j < byte_run; j++) {
               this->ghost_image[ghost_pixel_ptr++] = palette_idx1;
-              this->yuv_planes.y[pixel_ptr] = 
-                this->yuv_palette[palette_ptr1 + 0];
-              this->yuv_planes.u[pixel_ptr] = 
-                this->yuv_palette[palette_ptr1 + 1];
-              this->yuv_planes.v[pixel_ptr] = 
-                this->yuv_palette[palette_ptr1 + 2];
-              pixel_ptr++;
+              pixel_countdown--;
+              if (pixel_countdown < 0)
+                printf ("fli warning: pixel_countdown < 0 (%d)\n", 
+                  pixel_countdown);
             }
           } else {  /* copy bytes if byte_run < 0 */
             byte_run = -byte_run;
             for (j = 0; j < byte_run; j++) {
               palette_ptr1 = (palette_idx1 = this->buf[stream_ptr++]) * 4;
               this->ghost_image[ghost_pixel_ptr++] = palette_idx1;
-              this->yuv_planes.y[pixel_ptr] = 
-                this->yuv_palette[palette_ptr1 + 0];
-              this->yuv_planes.u[pixel_ptr] = 
-                this->yuv_palette[palette_ptr1 + 1];
-              this->yuv_planes.v[pixel_ptr] = 
-                this->yuv_palette[palette_ptr1 + 2];
-              pixel_ptr++;
+              pixel_countdown--;
+              if (pixel_countdown < 0)
+                printf ("fli warning: pixel_countdown < 0 (%d)\n", 
+                  pixel_countdown);
             }
           }
         }
 
-        y_ptr += this->yuv_planes.row_width;
         ghost_y_ptr += this->width;
       }
       break;
@@ -388,8 +392,6 @@ void decode_fli_frame(fli_decoder_t *this) {
         this->yuv_planes.v[pixel_ptr] = this->yuv_palette[palette_ptr1 + 2];
         pixel_ptr++;
       }
-
-      pixel_ptr += 2;
     }
   }
 
@@ -426,8 +428,9 @@ static void fli_decode_data (video_decoder_t *this_gen,
     if(this->buf)
       free(this->buf);
 
-    this->width = (LE_16(&buf->content[8]) + 3) & ~0x03;
-    this->height = (LE_16(&buf->content[10]) + 3) & ~0x03;
+    /* RGB -> YUV converter requires even width */
+    this->width = (LE_16(&buf->content[8]) + 1) & ~0x1;
+    this->height = LE_16(&buf->content[10]);
     this->video_step = buf->decoder_info[1];
 
     this->ghost_image = xine_xmalloc(this->width * this->height);
@@ -442,6 +445,9 @@ static void fli_decode_data (video_decoder_t *this_gen,
     this->stream->video_out->open (this->stream->video_out, this->stream);
     this->decoder_ok = 1;
 
+    /* load the stream/meta info */
+    this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] = strdup("FLI/FLC Video");
+
     return;
   } else if (this->decoder_ok) {
 
@@ -584,7 +590,7 @@ static decoder_info_t dec_info_video = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_DECODER, 13, "fli", XINE_VERSION_CODE, &dec_info_video, init_plugin },
+  { PLUGIN_VIDEO_DECODER, 14, "fli", XINE_VERSION_CODE, &dec_info_video, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
 
