@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: svq1.c,v 1.15 2002/12/06 01:44:06 miguelfreitas Exp $
+ * $Id: svq1.c,v 1.23 2003/01/08 01:02:32 miguelfreitas Exp $
  */
 
 #include <stdio.h>
@@ -50,7 +50,8 @@
 /* memory bit stream */
 typedef struct bit_buffer_s {
   uint8_t	*buffer;
-  uint32_t	 bitpos;
+  unsigned int	 bitpos;
+  unsigned int	 length;
 } bit_buffer_t;
 
 /* variable length (bit) code */
@@ -579,11 +580,23 @@ static vlc_code_t inter_mean_table_5[292] = {
 static uint32_t get_bits (bit_buffer_t *bitbuf, int count) {
   uint32_t result;
 
-  /* load 32 bits of data (byte-aligned) */
-  result   = BE_32 (&bitbuf->buffer[bitbuf->bitpos >> 3]);
+  /* avoid buffer overflow */
+  if ((bitbuf->bitpos + 24) >= bitbuf->length) {
+    int i;
 
-  /* compensate for sub-byte offset */
-  result <<= (bitbuf->bitpos & 0x7);
+    /* load upto 24 bits of data on sub-byte offset */
+    result = 0;
+
+    for (i=(bitbuf->bitpos & ~0x7); i < bitbuf->length; i+=8) {
+      result |= bitbuf->buffer[i >> 3] << (24 + (bitbuf->bitpos - i));
+    }
+  } else {
+    /* load 32 bits of data (byte-aligned) */
+    result   = BE_32 (&bitbuf->buffer[bitbuf->bitpos >> 3]);
+
+    /* compensate for sub-byte offset */
+    result <<= (bitbuf->bitpos & 0x7);
+  }
 
   /* flush num bits */
   bitbuf->bitpos += count;
@@ -598,11 +611,23 @@ static uint32_t get_bits (bit_buffer_t *bitbuf, int count) {
 static uint32_t get_bit_cache(bit_buffer_t *bitbuf) {
   uint32_t result;
 
-  /* load 32 bits of data (byte-aligned) */
-  result   = BE_32 (&bitbuf->buffer[bitbuf->bitpos >> 3]);
+  /* avoid buffer overflow */
+  if ((bitbuf->bitpos + 24) >= bitbuf->length) {
+    int i;
 
-  /* compensate for sub-byte offset */
-  result <<= (bitbuf->bitpos & 0x7);
+    /* load upto 24 bits of data on sub-byte offset */
+    result = 0;
+
+    for (i=(bitbuf->bitpos & ~0x7); i < bitbuf->length; i+=8) {
+      result |= bitbuf->buffer[i >> 3] << (24 + (bitbuf->bitpos - i));
+    }
+  } else {
+    /* load 32 bits of data (byte-aligned) */
+    result   = BE_32 (&bitbuf->buffer[bitbuf->bitpos >> 3]);
+
+    /* compensate for sub-byte offset */
+    result <<= (bitbuf->bitpos & 0x7);
+  }
 
   return result;
 }
@@ -812,6 +837,19 @@ static int decode_motion_vector (bit_buffer_t *bitbuf, svq1_pmv_t *mv, svq1_pmv_
   return 0;
 }
 
+static void clip_motion_vector (int x, int y, int width, int height, svq1_pmv_t *mv) {
+
+  if (mv->x < -2*x)
+    mv->x = -2*x;
+  else if (mv->x > 2*(width - x))
+    mv->x = 2*(width - x);
+
+  if (mv->y < -2*y)
+    mv->y = -2*y;
+  else if (mv->y > 2*(height - y))
+    mv->y = 2*(height - y);
+}
+
 static void skip_block (uint8_t *current, uint8_t *previous, int pitch, int x, int y) {
   uint8_t *src;
   uint8_t *dst;
@@ -829,7 +867,9 @@ static void skip_block (uint8_t *current, uint8_t *previous, int pitch, int x, i
 
 static int motion_inter_block (bit_buffer_t *bitbuf,
 			       uint8_t *current, uint8_t *previous, int pitch,
-			       svq1_pmv_t *motion, int x, int y) {
+			       svq1_pmv_t *motion,
+			       unsigned int x, unsigned int y,
+			       unsigned int width, unsigned int height) {
   uint8_t    *src;
   uint8_t    *dst;
   svq1_pmv_t  mv;
@@ -859,6 +899,9 @@ static int motion_inter_block (bit_buffer_t *bitbuf,
   motion[(x / 8) + 3].x	= mv.x;
   motion[(x / 8) + 3].y	= mv.y;
 
+  /* clip motion vector to frame border */
+  clip_motion_vector (x, y, (width - 16), (height - 16), &mv);
+
   src = &previous[(x + (mv.x >> 1)) + (y + (mv.y >> 1))*pitch];
   dst = current;
 
@@ -904,7 +947,9 @@ static int motion_inter_block (bit_buffer_t *bitbuf,
 
 static int motion_inter_4v_block (bit_buffer_t *bitbuf,
 				  uint8_t *current, uint8_t *previous, int pitch,
-				  svq1_pmv_t *motion,int x, int y) {
+				  svq1_pmv_t *motion,
+				  unsigned int x, unsigned int y,
+				  unsigned int width, unsigned int height) {
   uint8_t    *src;
   uint8_t    *dst;
   svq1_pmv_t  mv;
@@ -959,6 +1004,12 @@ static int motion_inter_4v_block (bit_buffer_t *bitbuf,
   if (result != 0)
     return result;
 
+  /* clip motion vectors to frame border */
+  clip_motion_vector (x, y, (width - 8), (height - 8), pmv[0]);
+  clip_motion_vector ((x + 8), y, (width - 8), (height - 8), pmv[1]);
+  clip_motion_vector (x, (y + 8), (width - 8), (height - 8), pmv[2]);
+  clip_motion_vector ((x + 8), (y + 8), (width - 8), (height - 8), pmv[3]);
+
   /* form predictions */
   for (i=0; i < 4; i++) {
     src = &previous[(x + (pmv[i]->x >> 1)) + (y + (pmv[i]->y >> 1))*pitch];
@@ -1015,7 +1066,9 @@ static int motion_inter_4v_block (bit_buffer_t *bitbuf,
 
 static int decode_delta_block (bit_buffer_t *bitbuf,
 			uint8_t *current, uint8_t *previous, int pitch,
-			svq1_pmv_t *motion, int x, int y) {
+			svq1_pmv_t *motion,
+			unsigned int x, unsigned int y,
+			unsigned int width, unsigned int height) {
   uint32_t bit_cache;
   uint32_t block_type;
   int	   result = 0;
@@ -1043,7 +1096,7 @@ static int decode_delta_block (bit_buffer_t *bitbuf,
     break;
 
   case SVQ1_BLOCK_INTER:
-    result = motion_inter_block (bitbuf, current, previous, pitch, motion, x, y);
+    result = motion_inter_block (bitbuf, current, previous, pitch, motion, x, y, width, height);
 
     if (result != 0)
       break;
@@ -1052,7 +1105,7 @@ static int decode_delta_block (bit_buffer_t *bitbuf,
     break;
 
   case SVQ1_BLOCK_INTER_4V:
-    result = motion_inter_4v_block (bitbuf, current, previous, pitch, motion, x, y);
+    result = motion_inter_4v_block (bitbuf, current, previous, pitch, motion, x, y, width, height);
 
     if (result != 0)
       break;
@@ -1135,7 +1188,7 @@ static int decode_frame_header (bit_buffer_t *bitbuf, svq1_t *svq1) {
   return 0;
 }
 
-static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer) {
+static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer, int length) {
   bit_buffer_t	bitbuf;
   uint8_t      *current, *previous;
   int		result, i, x, y, width, height;
@@ -1144,6 +1197,7 @@ static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer) {
   /* initialize bit buffer */
   bitbuf.buffer	= buffer;
   bitbuf.bitpos	= 0;
+  bitbuf.length	= 8*length;
 
   /* decode frame header */
   svq1->frame_code = get_bits (&bitbuf, 22);
@@ -1162,7 +1216,7 @@ static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer) {
 
   result = decode_frame_header (&bitbuf, svq1);
 
-  if (result != 0)
+  if (result != 0 || bitbuf.bitpos > bitbuf.length)
     return result;
 
   /* check frame size (changed?) */
@@ -1222,7 +1276,7 @@ static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer) {
 	for (x=0; x < width; x+=16) {
 	  result = decode_svq1_block (&bitbuf, &current[x], width, 1);
 
-	  if (result != 0)
+	  if (result != 0 || bitbuf.bitpos > bitbuf.length)
 	    return result;
 	}
 
@@ -1235,9 +1289,9 @@ static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer) {
       for (y=0; y < height; y+=16) {
 	for (x=0; x < width; x+=16) {
 	  result = decode_delta_block (&bitbuf, &current[x], previous,
-				       width, svq1->motion, x, y);
+				       width, svq1->motion, x, y, width, height);
 
-	  if (result != 0)
+	  if (result != 0 || bitbuf.bitpos > bitbuf.length)
 	    return result;
 	}
 
@@ -1265,113 +1319,6 @@ static int svq1_decode_frame (svq1_t *svq1, uint8_t *buffer) {
   return 0;
 }
 
-static void hscale_chroma_line (uint8_t *dst, uint8_t *src, int width) {
-  uint32_t  n1, n2;
-  int	    x;
-
-  n1	   = *src;
-  *(dst++) = n1;
-
-  for (x=0; x < (width - 1); x++) {
-    n2	     = *(++src);
-    *(dst++) = (3*n1 + n2 + 2) >> 2;
-    *(dst++) = (n1 + 3*n2 + 2) >> 2;
-    n1	     = n2;
-  }
-
-  *dst = n1;
-}
-
-static void vscale_chroma_line (uint8_t *dst, int pitch, uint8_t *src1, uint8_t *src2, int width) {
-  uint32_t  t1, t2;
-  uint32_t  n1, n2, n3, n4;
-  uint32_t *dst1, *dst2;
-  int	    x;
-
-  dst1 = (uint32_t *) dst;
-  dst2 = (uint32_t *) (dst + pitch);
-
-  /* process blocks of 4 pixels */
-  for (x=0; x < (width / 4); x++) {
-    n1  = *(((uint32_t *) src1)++);
-    n2  = *(((uint32_t *) src2)++);
-    n3  = (n1 & 0xFF00FF00) >> 8;
-    n4  = (n2 & 0xFF00FF00) >> 8;
-    n1 &= 0x00FF00FF;
-    n2 &= 0x00FF00FF;
-
-    t1 = (2*n1 + 2*n2 + 0x20002);
-    t2 = (n1 - n2);
-    n1 = (t1 + t2);
-    n2 = (t1 - t2);
-    t1 = (2*n3 + 2*n4 + 0x20002);
-    t2 = (n3 - n4);
-    n3 = (t1 + t2);
-    n4 = (t1 - t2);
-
-    *(dst1++) = ((n1 >> 2) & 0x00FF00FF) | ((n3 << 6) & 0xFF00FF00);
-    *(dst2++) = ((n2 >> 2) & 0x00FF00FF) | ((n4 << 6) & 0xFF00FF00);
-  }
-
-  /* process remaining pixels */
-  for (x=(width & ~0x3); x < width; x++) {
-    n1 = src1[x];
-    n2 = src2[x];
-
-    dst[x]	 = (3*n1 + n2 + 2) >> 2;
-    dst[x+pitch] = (n1 + 3*n2 + 2) >> 2;
-  }
-}
-
-static void svq1_copy_frame (svq1_t *svq1, uint8_t *base[3], int pitches[3]) {
-  uint8_t *src;
-  uint8_t *dst;
-  uint8_t *cr1, *cr2, *tmp;
-  int	   y, i;
-
-  src = svq1->base[0];
-  dst = base[0];
-
-  for (y=0; y < svq1->height; y++) {
-    memcpy (dst, src, svq1->width);
-    src += svq1->luma_width;
-    dst += pitches[0];
-  }
-
-  for (i=1; i < 3; i++) {
-    src = svq1->base[i];
-    dst = base[i];
-    cr1 = &dst[pitches[i] * ((svq1->height / 2) - 2)];
-    cr2 = &dst[pitches[i] * ((svq1->height / 2) - 3)];
-
-    /* horizontally upscale first line */
-    hscale_chroma_line (cr1, src, (svq1->width / 4));
-    src += svq1->chroma_width;
-
-    /* store first line */
-    memcpy (dst, cr1, (svq1->width / 2));
-    dst += pitches[i];
-
-    for (y=0; y < (svq1->height / 4) - 1; y++) {
-      hscale_chroma_line (cr2, src, (svq1->width / 4));
-      src += svq1->chroma_width;
-
-      /* interpolate and store two lines */
-      vscale_chroma_line (dst, pitches[i], cr1, cr2, (svq1->width / 2));
-      dst += 2*pitches[i];
-
-      /* swap buffers */
-      tmp = cr2;
-      cr2 = cr1;
-      cr1 = tmp;
-    }
-
-    /* horizontally upscale and store last line */
-    src -= svq1->chroma_width;
-    hscale_chroma_line (dst, src, (svq1->width / 4));
-  }
-}
-
 static void svq1dec_decode_data (video_decoder_t *this_gen, buf_element_t *buf) {
   svq1dec_decoder_t *this = (svq1dec_decoder_t *) this_gen;
 
@@ -1392,7 +1339,6 @@ static void svq1dec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
 
     /* load the stream/meta info */
     this->stream->meta_info[XINE_META_INFO_VIDEOCODEC] = strdup("Sorenson Video 1");
-    this->stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = 1;
 
   } else if (this->decoder_ok) {
     
@@ -1413,21 +1359,29 @@ static void svq1dec_decode_data (video_decoder_t *this_gen, buf_element_t *buf)
       vo_frame_t *img;
       int	  result;
 
-      result = svq1_decode_frame (this->svq1, this->buf);
+      result = svq1_decode_frame (this->svq1, this->buf, this->size);
 
       if (this->svq1->width > 0 && this->svq1->height > 0) {
 
 	img = this->stream->video_out->get_frame (this->stream->video_out,
-					  this->svq1->width,
-					  this->svq1->height,
-					  XINE_VO_ASPECT_DONT_TOUCH, XINE_IMGFMT_YV12, VO_BOTH_FIELDS);
+						  this->svq1->width,
+						  this->svq1->height,
+						  XINE_VO_ASPECT_DONT_TOUCH,
+						  XINE_IMGFMT_YV12,
+						  VO_BOTH_FIELDS);
 
 	img->duration	= this->video_step;
 	img->pts	= buf->pts;
 	img->bad_frame	= (result != 0);
 
 	if (result == 0) {
-	  svq1_copy_frame (this->svq1, img->base, img->pitches);
+	  yuv9_to_yv12 (this->svq1->base[0], this->svq1->luma_width,
+			img->base[0], img->pitches[0],
+			this->svq1->base[1], this->svq1->chroma_width,
+			img->base[1], img->pitches[1],
+			this->svq1->base[2], this->svq1->chroma_width,
+			img->base[2], img->pitches[2],
+			this->svq1->width, this->svq1->height);
 	}
 
 	img->draw(img, this->stream);
@@ -1546,6 +1500,6 @@ static decoder_info_t dec_info_video = {
 
 plugin_info_t xine_plugin_info[] = {
   /* type, API, "name", version, special_info, init_function */  
-  { PLUGIN_VIDEO_DECODER, 13, "svq1", XINE_VERSION_CODE, &dec_info_video, init_plugin },
+  { PLUGIN_VIDEO_DECODER, 14, "svq1", XINE_VERSION_CODE, &dec_info_video, init_plugin },
   { PLUGIN_NONE, 0, "", 0, NULL, NULL }
 };
