@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  *
- * $Id: video_out_pgx64.c,v 1.17 2002/12/07 23:00:07 f1rmb Exp $
+ * $Id: video_out_pgx64.c,v 1.23 2002/12/22 00:35:05 komadori Exp $
  *
  * video_out_pgx64.c, Sun PGX64/PGX24 output plugin for xine
  *
@@ -53,13 +53,6 @@
 #define BUS_EXT_REG_EN 0x08000000
 #define FIFO_STAT 0x1C4
 
-#define OVERLAY_SCALE_INC 0x008
-#define SCALER_HEIGHT_WIDTH 0x00A
-#define OVERLAY_GRAPHICS_KEY_CLR 0x004
-#define OVERLAY_GRAPHICS_KEY_MSK 0x005
-#define OVERLAY_KEY_CNTL 0x006
-#define SCALER_COLOUR_CNTL 0x054
-
 #define SCALER_H_COEFF0 0x055
 #define SCALER_H_COEFF0_DEFAULT 0x00002000
 #define SCALER_H_COEFF1 0x056
@@ -71,6 +64,11 @@
 #define SCALER_H_COEFF4 0x059
 #define SCALER_H_COEFF4_DEFAULT 0x0C14140C
 
+#define OVERLAY_GRAPHICS_KEY_CLR 0x004
+#define OVERLAY_GRAPHICS_KEY_MSK 0x005
+#define OVERLAY_KEY_CNTL 0x006
+#define SCALER_COLOUR_CNTL 0x054
+
 #define SCALER_BUF0_OFFSET 0x00D
 #define SCALER_BUF0_OFFSET_U 0x075
 #define SCALER_BUF0_OFFSET_V 0x076
@@ -83,6 +81,11 @@
 #define OVERLAY_X_Y_START 0x000
 #define OVERLAY_X_Y_END 0x001
 #define OVERLAY_X_Y_LOCK 0x80000000
+#define OVERLAY_SCALE_INC 0x008
+#define SCALER_HEIGHT_WIDTH 0x00A
+#define OVERLAY_EXCLUSIVE_HORZ 0x016
+#define OVERLAY_EXCLUSIVE_VERT 0x017
+#define OVERLAY_EXCLUSIVE_EN 0x80000000
 #define OVERLAY_SCALE_CNTL 0x009
 #define OVERLAY_EN 0xC0000000
 
@@ -128,12 +131,12 @@ typedef struct {
   GC gc;
 
   int fbfd;
-  void *fbbase;
+  uint8_t *fbbase;
   volatile uint32_t *fbregs;
   uint32_t top, fb_width, fb_height;
 
   int colour_key, brightness, saturation;
-  int deinterlace, deinterlace_method;
+  int deinterlace, deinterlace_method, use_exclusive;
 } pgx64_driver_t;
 
 /*
@@ -308,8 +311,8 @@ static void pgx64_update_frame_format(pgx64_driver_t *this, pgx64_frame_t *frame
         frame->vo_frame.pitches[1] = frame->pitch / 2;
         frame->vo_frame.pitches[2] = frame->pitch / 2;
         frame->lengths[0] = frame->vo_frame.pitches[0] * height;
-        frame->lengths[1] = frame->vo_frame.pitches[1] * height;
-        frame->lengths[2] = frame->vo_frame.pitches[2] * height;
+        frame->lengths[1] = frame->vo_frame.pitches[1] * (height / 2);
+        frame->lengths[2] = frame->vo_frame.pitches[2] * (height / 2);
         frame->vo_frame.base[0] = (void*)memalign(8, frame->lengths[0]);
         frame->vo_frame.base[1] = (void*)memalign(8, frame->lengths[1]);
         frame->vo_frame.base[2] = (void*)memalign(8, frame->lengths[2]);
@@ -342,48 +345,83 @@ static void pgx64_display_frame(pgx64_driver_t *this, pgx64_frame_t *frame)
     write_reg(this, SCALER_BUF0_OFFSET, frame->buf_y);
     write_reg(this, SCALER_BUF0_OFFSET_U, frame->buf_u);
     write_reg(this, SCALER_BUF0_OFFSET_V, frame->buf_v);
-    write_reg(this, SCALER_BUF_PITCH, this->deinterlace && (this->deinterlace_method == DEINTERLACE_ONEFIELD) ? frame->pitch * 2 : frame->pitch);
+    write_reg(this, SCALER_BUF_PITCH, frame->pitch);
     write_reg(this, OVERLAY_X_Y_START, ((this->vo_scale.gui_win_x + this->vo_scale.output_xoffset) << 16) | (this->vo_scale.gui_win_y + this->vo_scale.output_yoffset) | OVERLAY_X_Y_LOCK);
-    write_reg(this, OVERLAY_X_Y_END, ((this->vo_scale.gui_win_x + this->vo_scale.output_xoffset + this->vo_scale.output_width) << 16) | (this->vo_scale.gui_win_y + this->vo_scale.output_yoffset + this->vo_scale.output_height));
+    write_reg(this, OVERLAY_X_Y_END, ((this->vo_scale.gui_win_x + this->vo_scale.output_xoffset + this->vo_scale.output_width) << 16) | (this->vo_scale.gui_win_y + this->vo_scale.output_yoffset + this->vo_scale.output_height - 1));
     write_reg(this, OVERLAY_SCALE_INC, (((frame->width << 12) / this->vo_scale.output_width) << 16) | (((this->deinterlace && (this->deinterlace_method == DEINTERLACE_ONEFIELD) ? frame->height/2 : frame->height) << 12) / this->vo_scale.output_height));
     write_reg(this, SCALER_HEIGHT_WIDTH, (frame->width << 16) | (this->deinterlace && (this->deinterlace_method == DEINTERLACE_ONEFIELD) ? frame->height/2 : frame->height));
+
+    if (this->use_exclusive) {
+      int horz_start = (this->vo_scale.gui_win_x + this->vo_scale.output_xoffset + 7) / 8;
+      int horz_end = (this->vo_scale.gui_win_x + this->vo_scale.output_xoffset + this->vo_scale.output_width) / 8;
+
+      write_reg(this, OVERLAY_EXCLUSIVE_VERT, (this->vo_scale.gui_win_y + this->vo_scale.output_yoffset) | ((this->vo_scale.gui_win_y + this->vo_scale.output_yoffset + this->vo_scale.output_height - 1) << 16));
+      write_reg(this, OVERLAY_EXCLUSIVE_HORZ,  horz_start | (horz_end << 8) | (((this->fb_width/8) - horz_end) << 16) | OVERLAY_EXCLUSIVE_EN);
+    }
+    else {
+      write_reg(this, OVERLAY_EXCLUSIVE_HORZ, 0);
+    }
+
     set_reg_bits(this, OVERLAY_SCALE_CNTL, OVERLAY_EN);
   }
 
   if (frame->format == XINE_IMGFMT_YV12) {
     switch (this->deinterlace ? this->deinterlace_method : ~0) {
+      case DEINTERLACE_ONEFIELD: {
+        register uint8_t *y = frame->vo_frame.base[0];
+        register uint8_t *ydest = this->fbbase+frame->buf_y;
+        register uint8_t *u = frame->vo_frame.base[1];
+        register uint8_t *v = frame->vo_frame.base[2];
+        register uint8_t *udest = this->fbbase+frame->buf_u;
+        register uint8_t *vdest = this->fbbase+frame->buf_v;
+        int i = 0;
+
+        for (i = 0; i < frame->height/2; i++, y += 2*frame->vo_frame.pitches[0], ydest += frame->vo_frame.pitches[0]) {
+          memcpy(ydest, y, frame->vo_frame.pitches[0]);
+        }
+
+        for (i = 0; i < frame->height/4; i++, u += 2*frame->vo_frame.pitches[1], udest += frame->vo_frame.pitches[1], v += 2*frame->vo_frame.pitches[2], vdest += frame->vo_frame.pitches[2]) {
+          memcpy(udest, u, frame->vo_frame.pitches[1]);
+          memcpy(vdest, v, frame->vo_frame.pitches[2]);
+        }
+      }
+      break;
+
       case DEINTERLACE_LINEARBLEND: {
         register uint8_t *first = frame->vo_frame.base[0];
-        register uint8_t *second = frame->vo_frame.base[0]+frame->width;
-        register uint8_t *third = frame->vo_frame.base[0]+(2*frame->width);
-        register uint8_t *last = frame->vo_frame.base[0]+(frame->width*frame->height);
+        register uint8_t *second = frame->vo_frame.base[0]+frame->vo_frame.pitches[0];
+        register uint8_t *third = frame->vo_frame.base[0]+(2*frame->vo_frame.pitches[0]);
+        register uint8_t *last = frame->vo_frame.base[0]+(frame->vo_frame.pitches[0]*frame->height);
         register uint8_t *dest = this->fbbase+frame->buf_y;
 
-        memcpy(dest, first, frame->width);
-        dest += frame->width;
+        memcpy(dest, first, frame->vo_frame.pitches[0]);
+        dest += frame->vo_frame.pitches[0];
 
-        for (;third != last;first++,second++,third++,dest++) {
+        for (; third != last; first++, second++, third++, dest++) {
           *dest = (*first + (*second << 1) + *third) >> 2;
         }
 
-        memcpy(dest, second, frame->width);
+        memcpy(dest, second, frame->vo_frame.pitches[0]);
+
+        memcpy(this->fbbase+frame->buf_u, frame->vo_frame.base[1], frame->lengths[1]);
+        memcpy(this->fbbase+frame->buf_v, frame->vo_frame.base[2], frame->lengths[2]);
       }
       break;
 
 #ifdef ENABLE_VIS
       case DEINTERLACE_LINEARBLEND_VIS: {
         register uint32_t *first = (uint32_t *)(frame->vo_frame.base[0]);
-        register uint32_t *second = (uint32_t *)(frame->vo_frame.base[0]+frame->width);
-        register uint32_t *third = (uint32_t *)(frame->vo_frame.base[0]+(2*frame->width));
-        register uint32_t *last = (uint32_t *)(frame->vo_frame.base[0]+(frame->width*frame->height));
+        register uint32_t *second = (uint32_t *)(frame->vo_frame.base[0]+frame->vo_frame.pitches[0]);
+        register uint32_t *third = (uint32_t *)(frame->vo_frame.base[0]+(2*frame->vo_frame.pitches[0]));
+        register uint32_t *last = (uint32_t *)(frame->vo_frame.base[0]+(frame->vo_frame.pitches[0]*frame->height));
         register uint32_t *dest = (uint32_t *)(this->fbbase+frame->buf_y);
 
         write_gsr((read_gsr() & 0xffffff07) | 0x000000008);
 
-        memcpy(dest, first, frame->width);
-        dest += frame->width/4;
+        memcpy(dest, first, frame->vo_frame.pitches[0]);
+        dest += frame->vo_frame.pitches[0]/4;
 
-        for (;third != last;first++,second++,third++,dest++) {
+        for (; third != last; first++, second++, third++, dest++) {
           asm volatile("ld	[%0], %%f0\n\t"
                        "fexpand	%%f0, %%f2\n\t"
                        "ld	[%1], %%f4\n\t"
@@ -400,19 +438,22 @@ static void pgx64_display_frame(pgx64_driver_t *this, pgx64_frame_t *frame)
                          "%f6", "%f7", "%f8", "%f9", "%f10", "%f11");
         }
 
-        memcpy(dest, second, frame->width);
+        memcpy(dest, second, frame->vo_frame.pitches[0]);
+
+        memcpy(this->fbbase+frame->buf_u, frame->vo_frame.base[1], frame->lengths[1]);
+        memcpy(this->fbbase+frame->buf_v, frame->vo_frame.base[2], frame->lengths[2]);
       }
       break;
 #endif
 
       default: {
         memcpy(this->fbbase+frame->buf_y, frame->vo_frame.base[0], frame->lengths[0]);
+
+        memcpy(this->fbbase+frame->buf_u, frame->vo_frame.base[1], frame->lengths[1]);
+        memcpy(this->fbbase+frame->buf_v, frame->vo_frame.base[2], frame->lengths[2]);
       }
       break;
     }
-
-    memcpy(this->fbbase+frame->buf_u, frame->vo_frame.base[1], frame->lengths[1]);
-    memcpy(this->fbbase+frame->buf_v, frame->vo_frame.base[2], frame->lengths[2]);
   }
   else {
     memcpy(this->fbbase+frame->buf_y, frame->vo_frame.base[0], frame->lengths[0]);
@@ -585,6 +626,7 @@ static int pgx64_redraw_needed(pgx64_driver_t *this)
 
 static void pgx64_dispose(pgx64_driver_t *this)
 {
+  write_reg(this, OVERLAY_EXCLUSIVE_HORZ, 0);
   write_reg(this, OVERLAY_SCALE_CNTL, 0);
   clear_reg_bits(this, BUS_CNTL, BUS_EXT_REG_EN);
   munmap(this->fbbase, ADDRSPACE);
@@ -612,6 +654,10 @@ static void pgx64_config_changed(pgx64_driver_t *this, xine_cfg_entry_t *entry)
     this->deinterlace_method = entry->num_value;
     this->vo_scale.force_redraw = 1;
   }
+  else if (strcmp(entry->key, "video.pgx64_use_exclusive") == 0) {
+    this->use_exclusive = entry->num_value;
+    this->vo_scale.force_redraw = 1;
+  }
 }
 
 /*
@@ -623,7 +669,7 @@ static pgx64_driver_t* init_driver(pgx64_driver_class_t *class)
   pgx64_driver_t *this;
   char *devname;
   int fbfd;
-  void *baseaddr;
+  uint8_t *baseaddr;
   struct fbgattr attr;
 
   printf("video_out_pgx64: PGX64 video output plugin - By Robin Kay\n");
@@ -682,6 +728,7 @@ static pgx64_driver_t* init_driver(pgx64_driver_class_t *class)
   this->brightness = this->class->config->register_range(this->class->config, "video.pgx64_brightness", 0, -64, 63, "video overlay brightness", NULL, 10, (void*)pgx64_config_changed, this);
   this->saturation = this->class->config->register_range(this->class->config, "video.pgx64_saturation", 16, 0, 31, "video overlay saturation", NULL, 10, (void*)pgx64_config_changed, this);
   this->deinterlace_method = this->class->config->register_enum(this->class->config, "video.pgx64_deinterlace_method", 0, deinterlace_methods, "video deinterlacing method", NULL, 10, (void*)pgx64_config_changed, this);
+  this->use_exclusive = this->class->config->register_bool(this->class->config, "video.pgx64_use_exclusive", 0, "use exclusive video overlays", NULL, 10, (void*)pgx64_config_changed, this);
 
   this->fbfd = fbfd;
   this->top = attr.sattr.dev_specific[0];
@@ -834,8 +881,8 @@ static pgx64_driver_class_t* pgx64fb_init_class(xine_t *xine, void *visual_gen)
 
 plugin_info_t xine_plugin_info[] = {
 #ifdef HAVE_X11
-  {PLUGIN_VIDEO_OUT, 13, "pgx64", XINE_VERSION_CODE, &vo_info_pgx64, (void*)pgx64_init_class},
+  {PLUGIN_VIDEO_OUT, 14, "pgx64", XINE_VERSION_CODE, &vo_info_pgx64, (void*)pgx64_init_class},
 #endif
-  {PLUGIN_VIDEO_OUT, 13, "pgx64fb", XINE_VERSION_CODE, &vo_info_pgx64fb, (void*)pgx64fb_init_class},
+  {PLUGIN_VIDEO_OUT, 14, "pgx64fb", XINE_VERSION_CODE, &vo_info_pgx64fb, (void*)pgx64fb_init_class},
   {PLUGIN_NONE, 0, "", 0, NULL, NULL}
 };
