@@ -1,5 +1,5 @@
 /* 
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with self program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: audio_out.c,v 1.87 2002/12/06 01:13:17 miguelfreitas Exp $
+ * $Id: audio_out.c,v 1.100 2003/01/13 23:02:51 miguelfreitas Exp $
  * 
  * 22-8-2001 James imported some useful AC3 sections from the previous alsa driver.
  *   (c) 2001 Andy Lo A Foe <andy@alsaplayer.org>
@@ -65,6 +65,8 @@
 #include <unistd.h>
 #include <inttypes.h>
 
+#define XINE_ENABLE_EXPERIMENTAL_FEATURES
+
 #include "xine_internal.h"
 #include "xineutils.h"
 #include "audio_out.h"
@@ -91,7 +93,7 @@
  *
  * Unfortunately audio fifo adds a large delay to our closed loop.
  *
- * These are designed to avoid updating the metronom too fast.
+ * The defines below are designed to avoid updating the metronom too fast.
  * - it will only be updated 1 time per second (so it has a chance of
  *   distributing the error for several frames).
  * - it will only be updated 2 times for the whole audio fifo size
@@ -106,6 +108,51 @@
 #define SYNC_BUF_INTERVAL   NUM_AUDIO_BUFFERS / 2
 #define SYNC_GAP_RATE       4
 
+
+typedef struct {
+ 
+  xine_audio_port_t    ao; /* public part */
+
+  /* private stuff */
+  ao_driver_t         *driver;
+  pthread_mutex_t      driver_lock;
+  metronom_clock_t    *clock;
+  xine_t              *xine;
+  xine_list_t         *streams;
+  pthread_mutex_t      streams_lock;
+
+  int             audio_loop_running;
+  int             grab_only; /* => do not start thread, frontend will consume samples */
+  int             audio_paused;
+  pthread_t       audio_thread;
+
+  int             audio_step;           /* pts per 32 768 samples (sample = #bytes/2) */
+  int32_t         frames_per_kpts;      /* frames per 1024/90000 sec                  */
+  
+  ao_format_t     input, output;        /* format conversion done at audio_out.c */
+  double          frame_rate_factor;
+  
+  int             resample_conf;
+  int             force_rate;           /* force audio output rate to this value if non-zero */
+  int             do_resample;
+  int             gap_tolerance;
+  audio_fifo_t   *free_fifo;
+  audio_fifo_t   *out_fifo;
+  int64_t         last_audio_vpts;
+  
+  audio_buffer_t *frame_buf[2];         /* two buffers for "stackable" conversions */
+  int16_t        *zero_space;
+
+  int64_t         passthrough_offset;
+  int             flush_audio_driver;
+  int             discard_buffers;
+
+  int             do_compress;
+  double          compression_factor;   /* current compression */
+  double          compression_factor_max; /* user limit on compression */
+
+} aos_t;
+
 struct audio_fifo_s {
   audio_buffer_t    *first;
   audio_buffer_t    *last;
@@ -183,7 +230,6 @@ static audio_buffer_t *fifo_remove_int (audio_fifo_t *fifo) {
 
       fifo->last = NULL;
       fifo->num_buffers = 0;
-      pthread_cond_init  (&fifo->not_empty, NULL);
 
     } else 
       fifo->num_buffers--;
@@ -207,8 +253,8 @@ static audio_buffer_t *fifo_remove (audio_fifo_t *fifo) {
 }
 
 
-void write_pause_burst(xine_audio_port_t *this, uint32_t num_frames) { 
- 
+static void write_pause_burst(aos_t *this, uint32_t num_frames) { 
+  
   int error = 0;
   unsigned char buf[8192];
   unsigned short *sbuf = (unsigned short *)&buf[0];
@@ -245,7 +291,7 @@ void write_pause_burst(xine_audio_port_t *this, uint32_t num_frames) {
 }
 
 
-static void ao_fill_gap (xine_audio_port_t *this, int64_t pts_len) {
+static void ao_fill_gap (aos_t *this, int64_t pts_len) {
 
   int num_frames ;
 
@@ -259,7 +305,7 @@ static void ao_fill_gap (xine_audio_port_t *this, int64_t pts_len) {
     return; 
   }
 
-  while (num_frames > 0) {
+  while (num_frames > 0 && !this->discard_buffers) {
     if (num_frames > ZERO_BUF_SIZE) {
       pthread_mutex_lock( &this->driver_lock );
       this->driver->write(this->driver, this->zero_space, ZERO_BUF_SIZE);
@@ -286,7 +332,7 @@ static void ensure_buffer_size (audio_buffer_t *buf, int bytes_per_frame,
   buf->num_frames = frames;
 }
 
-static audio_buffer_t * swap_frame_buffers ( xine_audio_port_t *this ) {
+static audio_buffer_t * swap_frame_buffers ( aos_t *this ) {
   audio_buffer_t *tmp;
 
   tmp = this->frame_buf[1];
@@ -311,7 +357,7 @@ static int mode_channels( int mode ) {
   return 0;
 } 
 
-static void audio_filter_compress (xine_audio_port_t *this, int16_t *mem, int num_frames) {
+static void audio_filter_compress (aos_t *this, int16_t *mem, int num_frames) {
 
   int    i, maxs;
   double f_max;
@@ -359,7 +405,7 @@ static void audio_filter_compress (xine_audio_port_t *this, int16_t *mem, int nu
   }
 }
 
-static audio_buffer_t* prepare_samples( xine_audio_port_t  *this, audio_buffer_t *buf) {
+static audio_buffer_t* prepare_samples( aos_t *this, audio_buffer_t *buf) {
   double          acc_output_frames, output_frame_excess = 0;
   int             num_output_frames ;
 
@@ -493,7 +539,7 @@ static audio_buffer_t* prepare_samples( xine_audio_port_t  *this, audio_buffer_t
  */
 static void *ao_loop (void *this_gen) {
 
-  xine_audio_port_t *this = (xine_audio_port_t *) this_gen;
+  aos_t *this = (aos_t *) this_gen;
   int64_t         hw_vpts;
   audio_buffer_t *in_buf, *out_buf;
   int64_t         gap;
@@ -524,12 +570,18 @@ static void *ao_loop (void *this_gen) {
 #endif
     }
 
+    if (this->discard_buffers) {
+      fifo_append (this->free_fifo, in_buf);
+      in_buf = NULL;
+        
+      if( !this->flush_audio_driver )
+        continue;
+    }
+    
     if (this->flush_audio_driver) {
-#ifdef LOG
-      printf ("audio_out: flush audio driver\n");
-#endif
-      this->control(this, AO_CTRL_FLUSH_BUFFERS);
+      this->ao.control(&this->ao, AO_CTRL_FLUSH_BUFFERS);
       this->flush_audio_driver = 0;
+      continue;
     }
 
     /* 
@@ -567,11 +619,20 @@ static void *ao_loop (void *this_gen) {
     delay = this->driver->delay(this->driver);
     while (delay < 0 && this->audio_loop_running) {
       /* Get the audio card into RUNNING state. */
+      pthread_mutex_unlock( &this->driver_lock ); 
       ao_fill_gap (this, 10000); /* FIXME, this PTS of 1000 should == period size */
+      pthread_mutex_lock( &this->driver_lock ); 
       delay = this->driver->delay(this->driver);
     }
     pthread_mutex_unlock( &this->driver_lock ); 
 
+    
+    if( in_buf && in_buf->stream && !in_buf->stream->video_decoder_plugin ) {
+      pthread_mutex_lock( &in_buf->stream->current_extra_info_lock );
+      extra_info_merge( in_buf->stream->current_extra_info, in_buf->extra_info );
+      pthread_mutex_unlock( &in_buf->stream->current_extra_info_lock );
+    }
+    
     /*
      * where, in the timeline is the "end" of the 
      * hardware audio buffer at the moment?
@@ -685,13 +746,79 @@ static void *ao_loop (void *this_gen) {
   return NULL;
 }
 
+/*
+ * public a/v processing interface
+ */
+
+int xine_get_next_audio_frame (xine_audio_port_t *this_gen,
+			       xine_audio_frame_t *frame) {
+
+  aos_t          *this = (aos_t *) this_gen;
+  audio_buffer_t *buf;
+  xine_stream_t  *stream;
+
+  do {
+    stream = xine_list_first_content(this->streams);
+    if (!stream)
+      xine_usec_sleep (1000);
+  } while( !stream );
+  
+  pthread_mutex_lock (&this->out_fifo->mutex);
+
+  buf = this->out_fifo->first;
+
+  /* FIXME: ugly, use conditions and locks instead */
+
+  while (!buf
+	 && (stream->demux_plugin->get_status (stream->demux_plugin)==DEMUX_OK)) {
+
+    pthread_mutex_unlock(&this->out_fifo->mutex);
+    xine_usec_sleep (1000);
+    pthread_mutex_lock(&this->out_fifo->mutex);
+
+    buf = this->out_fifo->first;
+  }
+
+  if (!buf) {
+    pthread_mutex_unlock(&this->out_fifo->mutex);
+    return 0;
+  }
+
+  buf = fifo_remove_int (this->out_fifo);
+  pthread_mutex_unlock(&this->out_fifo->mutex);
+
+  frame->vpts            = buf->vpts;
+  frame->num_samples     = buf->num_frames;
+  frame->sample_rate     = this->input.rate;
+  frame->num_channels    = mode_channels (this->input.mode);
+  frame->bits_per_sample = this->input.bits;
+  frame->pos_stream      = buf->extra_info->input_pos;
+  frame->pos_time        = buf->extra_info->input_time;
+  frame->data            = (uint8_t *) buf->mem;
+  frame->xine_frame      = buf;
+
+  return 1;
+}
+
+void xine_free_audio_frame (xine_audio_port_t *this_gen, xine_audio_frame_t *frame) {
+
+  aos_t          *this = (aos_t *) this_gen;
+  audio_buffer_t *buf;
+
+  buf = (audio_buffer_t *) frame->xine_frame;
+
+  fifo_append (this->free_fifo, buf);
+}
+
+
 /*
  * open the audio device for writing to, start audio output thread
  */
 
-static int ao_open(xine_audio_port_t *this, xine_stream_t *stream,
+static int ao_open(xine_audio_port_t *this_gen, xine_stream_t *stream,
 		   uint32_t bits, uint32_t rate, int mode) {
  
+  aos_t *this = (aos_t *) this_gen;
   int output_sample_rate, err;
   pthread_attr_t pth_attrs;
 
@@ -732,30 +859,33 @@ static int ao_open(xine_audio_port_t *this, xine_stream_t *stream,
   this->input.mode            = mode;
   this->input.rate            = rate;
   this->input.bits            = bits;
-  
-  /* not all drivers/cards support 8 bits */
-  if( this->input.bits == 8 && 
-      !(this->driver->get_capabilities(this->driver) & AO_CAP_8BITS) ) {
-    bits = 16;      
-    printf("audio_out: 8 bits not supported by driver, converting to 16 bits.\n");
-  }
+
+  if (!this->grab_only) {
+    /* not all drivers/cards support 8 bits */
+    if( this->input.bits == 8 && 
+	!(this->driver->get_capabilities(this->driver) & AO_CAP_8BITS) ) {
+      bits = 16;      
+      printf("audio_out: 8 bits not supported by driver, converting to 16 bits.\n");
+    }
     
-  /* provide mono->stereo and stereo->mono conversions */
-  if( this->input.mode == AO_CAP_MODE_MONO && 
-      !(this->driver->get_capabilities(this->driver) & AO_CAP_MODE_MONO) ) {
-    mode = AO_CAP_MODE_STEREO;
-    printf("audio_out: mono not supported by driver, converting to stereo.\n");
-  }
-  if( this->input.mode == AO_CAP_MODE_STEREO && 
-      !(this->driver->get_capabilities(this->driver) & AO_CAP_MODE_STEREO) ) {
-    mode = AO_CAP_MODE_MONO;
-    printf("audio_out: stereo not supported by driver, converting to mono.\n");
-  }
+    /* provide mono->stereo and stereo->mono conversions */
+    if( this->input.mode == AO_CAP_MODE_MONO && 
+	!(this->driver->get_capabilities(this->driver) & AO_CAP_MODE_MONO) ) {
+      mode = AO_CAP_MODE_STEREO;
+      printf("audio_out: mono not supported by driver, converting to stereo.\n");
+    }
+    if( this->input.mode == AO_CAP_MODE_STEREO && 
+	!(this->driver->get_capabilities(this->driver) & AO_CAP_MODE_STEREO) ) {
+      mode = AO_CAP_MODE_MONO;
+      printf("audio_out: stereo not supported by driver, converting to mono.\n");
+    }
  
-  pthread_mutex_lock( &this->driver_lock );
-  output_sample_rate=this->driver->open(this->driver,bits,(this->force_rate ? this->force_rate : rate),mode);
-  pthread_mutex_unlock( &this->driver_lock );
-  
+    pthread_mutex_lock( &this->driver_lock );
+    output_sample_rate=this->driver->open(this->driver,bits,(this->force_rate ? this->force_rate : rate),mode);
+    pthread_mutex_unlock( &this->driver_lock );
+  } else
+    output_sample_rate = this->input.rate;
+
   if ( output_sample_rate == 0) {
     printf("audio_out: open failed!\n");
     return 0;
@@ -797,40 +927,51 @@ static int ao_open(xine_audio_port_t *this, xine_stream_t *stream,
     stream->metronom->set_audio_rate(stream->metronom, this->audio_step);
   pthread_mutex_unlock(&this->streams_lock);
 
-  /*
-   * start output thread
-   */
+  if (!this->grab_only) {
+    /*
+     * start output thread
+     */
 
-  if( this->audio_thread ) {
-    printf("audio_out: pthread already running!\n");
+    if( this->audio_thread ) {
+      printf("audio_out: pthread already running!\n");
+    }
+    
+    this->audio_loop_running = 1;  
+    
+    pthread_attr_init(&pth_attrs);
+    pthread_attr_setscope(&pth_attrs, PTHREAD_SCOPE_SYSTEM);
+    
+    if ((err = pthread_create (&this->audio_thread,
+			       &pth_attrs, ao_loop, this)) != 0) {
+      
+      /* FIXME: how does this happen ? */
+      
+      printf ("audio_out: can't create thread (%s)\n", strerror(err));
+      printf ("audio_out: sorry, this should not happen. please restart xine.\n");
+      abort();
+      
+    } else
+      printf ("audio_out: thread created\n");
   }
-  
-  this->audio_loop_running = 1;  
-
-  pthread_attr_init(&pth_attrs);
-  pthread_attr_setscope(&pth_attrs, PTHREAD_SCOPE_SYSTEM);
-
-  if ((err = pthread_create (&this->audio_thread,
-			     &pth_attrs, ao_loop, this)) != 0) {
-
-    /* FIXME: how does this happen ? */
-
-    printf ("audio_out: can't create thread (%s)\n", strerror(err));
-    printf ("audio_out: sorry, this should not happen. please restart xine.\n");
-    abort();
-
-  } else
-    printf ("audio_out: thread created\n");
 
   return this->output.rate;
 }
 
-static audio_buffer_t *ao_get_buffer (xine_audio_port_t *this) {
-  return fifo_remove (this->free_fifo);
+static audio_buffer_t *ao_get_buffer (xine_audio_port_t *this_gen) {
+
+  aos_t *this = (aos_t *) this_gen;
+  audio_buffer_t *buf;
+   
+  buf = fifo_remove (this->free_fifo);
+  extra_info_reset( buf->extra_info );
+  
+  return buf;
 }
 
-static void ao_put_buffer (xine_audio_port_t *this, audio_buffer_t *buf, xine_stream_t *stream) {
+static void ao_put_buffer (xine_audio_port_t *this_gen, 
+                           audio_buffer_t *buf, xine_stream_t *stream) {
 
+  aos_t *this = (aos_t *) this_gen;
   int64_t pts;
 
   if (buf->num_frames == 0) {
@@ -838,38 +979,31 @@ static void ao_put_buffer (xine_audio_port_t *this, audio_buffer_t *buf, xine_st
     return;
   }
 
+  buf->stream = stream;
+  extra_info_merge( buf->extra_info, stream->audio_decoder_extra_info );
+  
   pts = buf->vpts;
 
   buf->vpts = stream->metronom->got_audio_samples (stream->metronom, pts, 
 						   buf->num_frames);
-
+  buf->extra_info->vpts = buf->vpts;
+         
 #ifdef LOG
   printf ("audio_out: ao_put_buffer, pts=%lld, vpts=%lld\n",
 	  pts, buf->vpts);
 #endif
 
-  if ( buf->vpts + AO_MAX_GAP < this->last_audio_vpts) {
-
-    /* reject buffer */
-    printf ("audio_out: rejected buffer vpts=%lld, last_audio_vpts=%lld\n", 
-	    buf->vpts, this->last_audio_vpts);
-
-    fifo_append (this->free_fifo, buf);
-
-  } else {
-
-    fifo_append (this->out_fifo, buf);
-    this->last_audio_vpts = buf->vpts;
-
-  }
+  fifo_append (this->out_fifo, buf);
+  this->last_audio_vpts = buf->vpts;
 
 #ifdef LOG
   printf ("audio_out: ao_put_buffer done\n");
 #endif
 }
 
-static void ao_close(xine_audio_port_t *this, xine_stream_t *stream) {
+static void ao_close(xine_audio_port_t *this_gen, xine_stream_t *stream) {
 
+  aos_t *this = (aos_t *) this_gen;
   audio_buffer_t *audio_buffer;
   xine_stream_t *cur;
 
@@ -897,36 +1031,43 @@ static void ao_close(xine_audio_port_t *this, xine_stream_t *stream) {
     }
   pthread_mutex_unlock(&this->streams_lock);
 
-  pthread_mutex_lock( &this->driver_lock );
-  this->driver->close(this->driver);  
-  pthread_mutex_unlock( &this->driver_lock );
+  if (!this->grab_only) {
+    pthread_mutex_lock( &this->driver_lock );
+    this->driver->close(this->driver);  
+    pthread_mutex_unlock( &this->driver_lock );
+  }
 }
 
-static void ao_exit(xine_audio_port_t *this) {
+static void ao_exit(xine_audio_port_t *this_gen) {
+  aos_t *this = (aos_t *) this_gen;
   int vol;
   int prop = 0;
   
   audio_buffer_t *buf, *next;
   
-  pthread_mutex_lock( &this->driver_lock );
-  
-  if((this->driver->get_capabilities(this->driver)) & AO_CAP_MIXER_VOL)
-    prop = AO_PROP_MIXER_VOL;
-  else if((this->driver->get_capabilities(this->driver)) & AO_CAP_PCM_VOL)
-    prop = AO_PROP_PCM_VOL;
-  
-  vol = this->driver->get_property(this->driver, prop);
-  this->xine->config->update_num(this->xine->config, "audio.mixer_volume", vol);
-  this->driver->exit(this->driver);
-  pthread_mutex_unlock( &this->driver_lock );
-  
+  if (!this->grab_only) {
+    pthread_mutex_lock( &this->driver_lock );
+    
+    if((this->driver->get_capabilities(this->driver)) & AO_CAP_MIXER_VOL)
+      prop = AO_PROP_MIXER_VOL;
+    else if((this->driver->get_capabilities(this->driver)) & AO_CAP_PCM_VOL)
+      prop = AO_PROP_PCM_VOL;
+    
+    vol = this->driver->get_property(this->driver, prop);
+    this->xine->config->update_num(this->xine->config, "audio.mixer_volume", vol);
+    this->driver->exit(this->driver);
+    pthread_mutex_unlock( &this->driver_lock );
+  }
+
   pthread_mutex_destroy(&this->driver_lock);
   pthread_mutex_destroy(&this->streams_lock);
   xine_list_free(this->streams);
 
   free (this->frame_buf[0]->mem);
+  free (this->frame_buf[0]->extra_info);
   free (this->frame_buf[0]);
   free (this->frame_buf[1]->mem);
+  free (this->frame_buf[1]->extra_info);
   free (this->frame_buf[1]);
   free (this->zero_space);
 
@@ -937,6 +1078,7 @@ static void ao_exit(xine_audio_port_t *this) {
     next = buf->next;
 
     free (buf->mem);
+    free (buf->extra_info);
     free (buf);
 
     buf = next;
@@ -949,6 +1091,7 @@ static void ao_exit(xine_audio_port_t *this) {
     next = buf->next;
 
     free (buf->mem);
+    free (buf->extra_info);
     free (buf);
 
     buf = next;
@@ -959,23 +1102,41 @@ static void ao_exit(xine_audio_port_t *this) {
   free (this);
 }
 
-static uint32_t ao_get_capabilities (xine_audio_port_t *this) {
+static uint32_t ao_get_capabilities (xine_audio_port_t *this_gen) {
+  aos_t *this = (aos_t *) this_gen;
   uint32_t result;
   
-  pthread_mutex_lock( &this->driver_lock );
-  result=this->driver->get_capabilities(this->driver);  
-  pthread_mutex_unlock( &this->driver_lock );
-  
+  if (this->grab_only) {
+
+    return AO_CAP_MODE_MONO | AO_CAP_MODE_STEREO ;
+    /* FIXME: make configurable
+      | AO_CAP_MODE_4CHANNEL | AO_CAP_MODE_5CHANNEL
+      | AO_CAP_MODE_5_1CHANNEL | AO_CAP_8BITS;
+    */
+  } else {
+    pthread_mutex_lock( &this->driver_lock );
+    result=this->driver->get_capabilities(this->driver);  
+    pthread_mutex_unlock( &this->driver_lock );
+  }
   return result;
 }
 
-static int ao_get_property (xine_audio_port_t *this, int property) {
+static int ao_get_property (xine_audio_port_t *this_gen, int property) {
+  aos_t *this = (aos_t *) this_gen;
   int ret;
 
   switch (property) {
   case AO_PROP_COMPRESSOR:
     ret = this->compression_factor_max*100;
     break;
+  
+  case AO_PROP_DISCARD_BUFFERS:
+    ret = this->discard_buffers;
+    break;
+
+  case AO_PROP_PAUSED:
+    ret = this->audio_paused;
+    break;
 
   default:
     pthread_mutex_lock( &this->driver_lock );
@@ -985,7 +1146,8 @@ static int ao_get_property (xine_audio_port_t *this, int property) {
   return ret;
 }
 
-static int ao_set_property (xine_audio_port_t *this, int property, int value) {
+static int ao_set_property (xine_audio_port_t *this_gen, int property, int value) {
+  aos_t *this = (aos_t *) this_gen;
   int ret;
 
   switch (property) {
@@ -996,6 +1158,16 @@ static int ao_set_property (xine_audio_port_t *this, int property, int value) {
 
     ret = this->compression_factor_max*100;
     break;
+  
+  case AO_PROP_DISCARD_BUFFERS:
+    this->discard_buffers = value;
+    ret = this->discard_buffers;
+    break;
+
+  case AO_PROP_PAUSED:
+    this->audio_paused = value;
+    ret = this->audio_paused;
+    break;
 
   default:
     pthread_mutex_lock( &this->driver_lock );
@@ -1006,59 +1178,72 @@ static int ao_set_property (xine_audio_port_t *this, int property, int value) {
   return ret;
 }
 
-static int ao_control (xine_audio_port_t *this, int cmd, ...) {
+static int ao_control (xine_audio_port_t *this_gen, int cmd, ...) {
 
+  aos_t *this = (aos_t *) this_gen;
   va_list args;
   void *arg;
   int rval;
 
+  pthread_mutex_lock( &this->driver_lock );
   va_start(args, cmd);
   arg = va_arg(args, void*);
-  pthread_mutex_lock( &this->driver_lock );
   rval = this->driver->control(this->driver, cmd, arg);
-  pthread_mutex_unlock( &this->driver_lock );
   va_end(args);
+  pthread_mutex_unlock( &this->driver_lock );
 
   return rval;
 }
 
-static void ao_flush (xine_audio_port_t *this) {
+static void ao_flush (xine_audio_port_t *this_gen) {
+  aos_t *this = (aos_t *) this_gen;
   audio_buffer_t *buf;
-  int            i, num_buffers;
 
-  pthread_mutex_lock (&this->out_fifo->mutex);
-  pthread_mutex_lock (&this->free_fifo->mutex);
-
-  num_buffers = this->out_fifo->num_buffers;
-
-  printf ("audio_out: flush fifo (%d buffers)\n", num_buffers);
-
-  for (i = 0; i < num_buffers; i++) {
-    buf = fifo_remove_int (this->out_fifo);
-    fifo_append_int (this->free_fifo, buf);
+  if( this->audio_loop_running ) {
+    this->discard_buffers    = 1;
+    this->flush_audio_driver = 1;
+    
+    buf = fifo_remove (this->free_fifo);
+    buf->num_frames = 0;
+    fifo_append (this->out_fifo, buf);
+  
+    /* do not try this in paused mode */
+    while( this->flush_audio_driver )
+      xine_usec_sleep (20000); /* pthread_cond_t could be used here */
+    this->discard_buffers    = 0;
   }
-
-  /* 
-   * make sure ao_loop can savely quit
-   */
-
-  buf = fifo_remove_int (this->free_fifo);
-  buf->num_frames = 0;
-  fifo_append_int (this->out_fifo, buf);
-
-  this->flush_audio_driver = 1;
-  pthread_mutex_unlock (&this->free_fifo->mutex);
-  pthread_mutex_unlock (&this->out_fifo->mutex);
 }
 
-xine_audio_port_t *ao_new_port (xine_t *xine, ao_driver_t *driver) {
+static int ao_status (xine_audio_port_t *this_gen, xine_stream_t *stream,
+	       uint32_t *bits, uint32_t *rate, int *mode) {
+  aos_t *this = (aos_t *) this_gen;
+  xine_stream_t *cur;
+  int ret = 0;
+            
+  pthread_mutex_lock(&this->streams_lock);
+  for (cur = xine_list_first_content(this->streams); cur;
+       cur = xine_list_next_content(this->streams))
+    if (cur == stream || !stream) {
+      *bits = this->input.bits;
+      *rate = this->input.rate;
+      *mode = this->input.mode;
+      ret = 1;
+      break;
+    }
+  pthread_mutex_unlock(&this->streams_lock);
+  
+  return ret;        
+}
+
+xine_audio_port_t *ao_new_port (xine_t *xine, ao_driver_t *driver,
+				int grab_only) {
  
   config_values_t *config = xine->config;
-  xine_audio_port_t *this;
+  aos_t           *this;
   int              i;
   static     char *resample_modes[] = {"auto", "off", "on", NULL};
 
-  this = xine_xmalloc (sizeof (xine_audio_port_t)) ;
+  this = xine_xmalloc (sizeof (aos_t)) ;
 
   this->driver                = driver;
   this->xine                  = xine;
@@ -1068,21 +1253,26 @@ xine_audio_port_t *ao_new_port (xine_t *xine, ao_driver_t *driver) {
   pthread_mutex_init( &this->streams_lock, NULL );
   pthread_mutex_init( &this->driver_lock, NULL );
 
-  this->open                   = ao_open;
-  this->get_buffer             = ao_get_buffer;
-  this->put_buffer             = ao_put_buffer;
-  this->close                  = ao_close;
-  this->exit                   = ao_exit;
-  this->get_capabilities       = ao_get_capabilities;
-  this->get_property           = ao_get_property;
-  this->set_property           = ao_set_property;
-  this->control                = ao_control;
-  this->flush                  = ao_flush;
+  this->ao.open                   = ao_open;
+  this->ao.get_buffer             = ao_get_buffer;
+  this->ao.put_buffer             = ao_put_buffer;
+  this->ao.close                  = ao_close;
+  this->ao.exit                   = ao_exit;
+  this->ao.get_capabilities       = ao_get_capabilities;
+  this->ao.get_property           = ao_get_property;
+  this->ao.set_property           = ao_set_property;
+  this->ao.control                = ao_control;
+  this->ao.flush                  = ao_flush;
+  this->ao.status                 = ao_status;
+  
   this->audio_loop_running     = 0;
+  this->grab_only              = grab_only;
   this->audio_paused           = 0;
   this->flush_audio_driver     = 0;
+  this->discard_buffers        = 0;
   this->zero_space             = xine_xmalloc (ZERO_BUF_SIZE * 2 * 6);
-  this->gap_tolerance          = driver->get_gap_tolerance (this->driver);
+  if (!grab_only)
+    this->gap_tolerance          = driver->get_gap_tolerance (this->driver);
 
   this->resample_conf = config->register_enum (config, "audio.resample_mode", 0,
 					       resample_modes,
@@ -1116,7 +1306,8 @@ xine_audio_port_t *ao_new_port (xine_t *xine, ao_driver_t *driver) {
     buf = (audio_buffer_t *) xine_xmalloc (sizeof (audio_buffer_t));
     buf->mem = xine_xmalloc (AUDIO_BUF_SIZE);
     buf->mem_size = AUDIO_BUF_SIZE;
-
+    buf->extra_info = malloc(sizeof(extra_info_t));
+    
     fifo_append (this->free_fifo, buf);
   }
   
@@ -1128,6 +1319,7 @@ xine_audio_port_t *ao_new_port (xine_t *xine, ao_driver_t *driver) {
     buf = (audio_buffer_t *) xine_xmalloc (sizeof (audio_buffer_t));
     buf->mem = xine_xmalloc (4*AUDIO_BUF_SIZE);
     buf->mem_size = 4*AUDIO_BUF_SIZE;
+    buf->extra_info = malloc(sizeof(extra_info_t));
 
     this->frame_buf[i] = buf;
   }
@@ -1148,14 +1340,14 @@ xine_audio_port_t *ao_new_port (xine_t *xine, ao_driver_t *driver) {
 			      0, NULL, NULL)) {
       int prop = 0;
 
-      if((ao_get_capabilities(this)) & AO_CAP_MIXER_VOL)
+      if((ao_get_capabilities(&this->ao)) & AO_CAP_MIXER_VOL)
 	prop = AO_PROP_MIXER_VOL;
-      else if((ao_get_capabilities(this)) & AO_CAP_PCM_VOL)
+      else if((ao_get_capabilities(&this->ao)) & AO_CAP_PCM_VOL)
 	prop = AO_PROP_PCM_VOL;
       
-      ao_set_property(this, prop, vol);
+      ao_set_property(&this->ao, prop, vol);
     }
   }
 
-  return this;
+  return &this->ao;
 }
