@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer.c,v 1.18 2002/07/17 18:36:44 miguelfreitas Exp $
+ * $Id: buffer.c,v 1.22 2003/01/26 23:36:46 f1rmb Exp $
  *
  *
  * contents:
@@ -39,6 +39,7 @@
 #include <stdlib.h>
 #include "buffer.h"
 #include "xineutils.h"
+#include "xine_internal.h"
 
 /*
  * put a previously allocated buffer element back into the buffer pool
@@ -108,12 +109,48 @@ static buf_element_t *buffer_pool_alloc (fifo_buffer_t *this) {
   buf->content = buf->mem; /* 99% of demuxers will want this */
   buf->pts = 0;
   buf->size = 0;
-  buf->input_pos = buf->input_length = buf->input_time = 0;
   buf->decoder_flags = 0;
+  extra_info_reset( buf->extra_info );
 
   return buf;
 }
 
+/*
+ * allocate a buffer from buffer pool - may fail if none is available
+ */
+
+static buf_element_t *buffer_pool_try_alloc (fifo_buffer_t *this) {
+  
+  buf_element_t *buf;
+
+  pthread_mutex_lock (&this->buffer_pool_mutex);
+
+  if (this->buffer_pool_top) {
+
+    buf = this->buffer_pool_top;
+    this->buffer_pool_top = this->buffer_pool_top->next;
+    this->buffer_pool_num_free--;
+  
+  } else {
+    
+    buf = NULL;
+    
+  }
+
+  pthread_mutex_unlock (&this->buffer_pool_mutex);
+
+  /* set sane values to the newly allocated buffer */
+  if( buf ) {
+    buf->content = buf->mem; /* 99% of demuxers will want this */
+    buf->pts = 0;
+    buf->size = 0;
+    buf->decoder_flags = 0;
+    extra_info_reset( buf->extra_info );
+  }
+  return buf;
+}
+
+
 /*
  * append buffer element to fifo buffer
  */
@@ -135,6 +172,27 @@ static void fifo_buffer_put (fifo_buffer_t *fifo, buf_element_t *element) {
   pthread_mutex_unlock (&fifo->mutex);
 }
 
+/*
+ * insert buffer element to fifo buffer (demuxers MUST NOT call this one)
+ */
+static void fifo_buffer_insert (fifo_buffer_t *fifo, buf_element_t *element) {
+  
+  pthread_mutex_lock (&fifo->mutex);
+
+  element->next = fifo->first;
+  fifo->first = element;
+  
+  if( !fifo->last )
+    fifo->last = element;
+    
+  fifo->fifo_size++;
+
+  pthread_cond_signal (&fifo->not_empty);
+
+  pthread_mutex_unlock (&fifo->mutex);
+}
+
+
 /*
  * get element from fifo buffer
  */
@@ -196,18 +254,6 @@ static void fifo_buffer_clear (fifo_buffer_t *fifo) {
     
     buf = next;
   }
-  /*
-  while (fifo->first != NULL) {
-
-    buf = fifo->first;
-
-    fifo->first = fifo->first->next;
-    if (fifo->first==NULL)
-      fifo->last = NULL;
-
-    buf->free_buffer(buf);
-  }
-  */
 
   /*printf("Free buffers after clear: %d\n", fifo->buffer_pool_num_free);*/
   pthread_mutex_unlock (&fifo->mutex);
@@ -241,6 +287,7 @@ static void fifo_buffer_dispose (fifo_buffer_t *this) {
 
     next = buf->next;
 
+    free (buf->extra_info);
     free (buf);
     received++;
 
@@ -251,6 +298,7 @@ static void fifo_buffer_dispose (fifo_buffer_t *this) {
   
     buf = this->get(this);
     
+    free(buf->extra_info);
     free(buf);
     received++;
   }
@@ -279,6 +327,7 @@ fifo_buffer_t *fifo_buffer_new (int num_buffers, uint32_t buf_size) {
   this->last            = NULL;
   this->fifo_size       = 0;
   this->put             = fifo_buffer_put;
+  this->insert          = fifo_buffer_insert;
   this->get             = fifo_buffer_get;
   this->clear           = fifo_buffer_clear;
   this->size		= fifo_buffer_size;
@@ -311,20 +360,22 @@ fifo_buffer_t *fifo_buffer_new (int num_buffers, uint32_t buf_size) {
     buf_element_t *buf;
 
     buf = xine_xmalloc (sizeof (buf_element_t));
-
+    
     buf->mem = multi_buffer;
     multi_buffer += buf_size;
 
     buf->max_size    = buf_size;
     buf->free_buffer = buffer_pool_free;
     buf->source      = this;
+    buf->extra_info  = malloc(sizeof(extra_info_t));
     
     buffer_pool_free (buf);
   }
-  this->buffer_pool_num_free = num_buffers;
-  this->buffer_pool_capacity = num_buffers;
-  this->buffer_pool_buf_size = buf_size;
-  this->buffer_pool_alloc    = buffer_pool_alloc;
+  this->buffer_pool_num_free  = num_buffers;
+  this->buffer_pool_capacity  = num_buffers;
+  this->buffer_pool_buf_size  = buf_size;
+  this->buffer_pool_alloc     = buffer_pool_alloc;
+  this->buffer_pool_try_alloc = buffer_pool_try_alloc;
 
   return this;
 }
