@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: buffer_types.c,v 1.48 2002/12/01 17:04:30 tmmm Exp $
+ * $Id: buffer_types.c,v 1.52 2003/01/23 16:12:19 miguelfreitas Exp $
  *
  *
  * contents:
@@ -262,13 +262,6 @@ static video_db_t video_db[] = {
   BUF_VIDEO_I263,
   "I263 format"
 },
-{
-  {
-    0
-  },
-  BUF_VIDEO_RV10,
-  ""
-},
 {
   {
     meFOURCC('D','I','B',' '),  /* device-independent bitmap */
@@ -315,6 +308,14 @@ static video_db_t video_db[] = {
   BUF_VIDEO_WMV8,
   "Windows Media Video 8"
 },
+{
+  {
+    meFOURCC('W','M','V','3'),
+    0
+  },
+  BUF_VIDEO_WMV9,
+  "Windows Media Video 9"
+},
 {
   {
     meFOURCC('c','r','a','m'),
@@ -493,6 +494,38 @@ static video_db_t video_db[] = {
   BUF_VIDEO_UCOD,
   "ClearVideo"
 },
+{
+  {
+    meFOURCC('R', 'V', '1', '0'),
+    0
+  },
+  BUF_VIDEO_RV10,
+  "Real Video 1.0"
+},
+{
+  {
+    meFOURCC('R', 'V', '2', '0'),
+    0
+  },
+  BUF_VIDEO_RV20,
+  "Real Video 2.0"
+},
+{
+  {
+    meFOURCC('R', 'V', '3', '0'),
+    0
+  },
+  BUF_VIDEO_RV30,
+  "Real Video 3.0"
+},
+{
+  {
+    meFOURCC('R', 'V', '4', '0'),
+    0
+  },
+  BUF_VIDEO_RV40,
+  "Real Video 4.0"
+},
 { { 0 }, 0, "last entry" }
 };
 
@@ -547,6 +580,13 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_WMAV2,
   "Windows Media Audio v2"
 },
+{
+  {
+    0x162, 0
+  },
+  BUF_AUDIO_WMAV3,
+  "Windows Media Audio v3"
+},
 {
   {
     0
@@ -697,6 +737,30 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_DNET,
   "RealAudio DNET"
 },
+{
+  {
+    meFOURCC('s', 'i', 'p', 'r'),
+    0
+  },
+  BUF_AUDIO_SIPRO,
+  "RealAudio SIPRO"
+},
+{
+  {
+    meFOURCC('c', 'o', 'o', 'k'),
+    0
+  },
+  BUF_AUDIO_COOK,
+  "RealAudio COOK"
+},
+{
+  {
+    meFOURCC('a', 't', 'r', 'c'),
+    0
+  },
+  BUF_AUDIO_ATRK,
+  "RealAudio ATRK"
+},
 {
   {
     meFOURCC('Q', 'c', 'l', 'p'),
@@ -731,6 +795,13 @@ static audio_db_t audio_db[] = {
   BUF_AUDIO_GSM610,
   "GSM 6.10"
 },
+{
+  {
+    0
+  },
+  BUF_AUDIO_FLAC,
+  "Free Lossless Audio Codec (FLAC)"
+},
 { { 0 }, 0, "last entry" }
 };
 
