@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: load_plugins.c,v 1.119 2002/12/08 20:53:02 miguelfreitas Exp $
+ * $Id: load_plugins.c,v 1.139 2003/01/29 02:33:36 miguelfreitas Exp $
  *
  *
  * Load input/demux/audio_out/video_out/codec plugins
@@ -37,6 +37,7 @@
 #include <string.h>
 #include <errno.h>
 #include <stdlib.h>
+#include <ctype.h>
 #include <signal.h>
 
 #include "xine_internal.h"
@@ -55,13 +56,8 @@
 #define LOG
 */
 
-extern int errno;
-
 static char *plugin_name;
 
-/* new code, comment this in case of trouble */
-#define USE_CACHED_CATALOG
-
 #if DONT_CATCH_SIGSEGV
 
 #define install_segv_handler()
@@ -96,14 +92,6 @@ static void remove_segv_handler(void){
  *
  */
 
-static char  *_strclone(const char *str){
-  char *new;
-
-  new = xine_xmalloc(strlen(str)+1);
-  strcpy(new, str);
-  return new;
-}
-
 static int _get_decoder_priority (xine_t *this, int default_priority,
 				  char *id) {
 
@@ -116,10 +104,10 @@ static int _get_decoder_priority (xine_t *this, int default_priority,
 				     default_priority,
 				     "decoder's priority compared to others",
 				     NULL, 20,
-				     NULL, NULL /* FIXME: implement callback */);
+				     NULL, NULL /*FIXME: implement callback*/);
 }
 
-#ifdef USE_CACHED_CATALOG  
+
 static plugin_info_t *_get_cached_plugin ( xine_list_t *list,
 			    char *filename, struct stat *statbuffer, 
 			    plugin_info_t *previous_info){
@@ -144,7 +132,7 @@ static plugin_info_t *_get_cached_plugin ( xine_list_t *list,
   }
   return NULL;
 }
-#endif
+
 
 static void _insert_plugin (xine_t *this,
 			    xine_list_t *list,
@@ -156,6 +144,7 @@ static void _insert_plugin (xine_t *this,
   vo_info_t         *vo_new, *vo_old;
   ao_info_t         *ao_new, *ao_old;
   decoder_info_t    *decoder_new, *decoder_old;
+  post_info_t       *post_new, *post_old;
   uint32_t          *types;
   int                priority = 0;
   int                i;
@@ -168,17 +157,17 @@ static void _insert_plugin (xine_t *this,
   }
 
   entry = xine_xmalloc(sizeof(plugin_node_t));
-  entry->filename     = _strclone(filename);
+  entry->filename     = strdup(filename);
   entry->filesize     = statbuffer->st_size;
   entry->filemtime    = statbuffer->st_mtime;
   entry->info         = xine_xmalloc(sizeof(plugin_info_t));
   *(entry->info)      = *info;
-  entry->info->id     = _strclone(info->id);
+  entry->info->id     = strdup(info->id);
   entry->info->init   = NULL;
   entry->plugin_class = NULL;
   entry->ref          = 0;
 
-  switch (info->type){
+  switch (info->type & PLUGIN_TYPE_MASK){
 
   case PLUGIN_VIDEO_OUT:
     vo_old = info->special_info;
@@ -217,6 +206,12 @@ static void _insert_plugin (xine_t *this,
 
     entry->info->special_info = decoder_new;
     break;
+  
+  case PLUGIN_POST:
+    post_old = info->special_info;
+    post_new = xine_xmalloc(sizeof(post_info_t));
+    post_new->type = post_old->type;
+    entry->info->special_info = post_new;
   }
 
   xine_list_append_priority_content (list, entry, priority);
@@ -280,15 +275,19 @@ static void collect_plugins(xine_t *this, char *path){
 	case S_IFREG:
 	  /* regular file, ie. plugin library, found => load it */
 
+	  /* this will fail whereever shared libs are called *.dll or such
+	   * better solutions:
+           * a) don't install .la files on user's system
+           * b) also cache negative hits, ie. files that failed to dlopen()
+	   */
 	  if(!strstr(str, ".so") )
 	    break;
 	  
 	  plugin_name = str;
 	  lib = NULL;
-	  info = NULL;
-#ifdef USE_CACHED_CATALOG  
 	  info = _get_cached_plugin ( this->plugin_catalog->cache,
 			              str, &statbuffer, NULL);
+#ifdef LOG
 	  if( info )
 	    printf("load_plugins: using cached %s\n", str);
 	  else
@@ -303,8 +302,8 @@ static void collect_plugins(xine_t *this, char *path){
 	      printf ("load_plugins: cannot open plugin lib %s:\n%s\n",
 		      str, dl_error_msg);
 	    }
-	  }
-	  else {
+
+	  } else {
 
 	    if (info || (info = dlsym(lib, "xine_plugin_info"))) {
        
@@ -313,7 +312,7 @@ static void collect_plugins(xine_t *this, char *path){
 		xine_log (this, XINE_LOG_PLUGIN,
 			  _("load_plugins: plugin %s found\n"), str);
 
-		switch (info->type){
+		switch (info->type & PLUGIN_TYPE_MASK){
 		case PLUGIN_INPUT:
 		  _insert_plugin (this, this->plugin_catalog->input, str,
 				  &statbuffer, info,
@@ -361,16 +360,14 @@ static void collect_plugins(xine_t *this, char *path){
 		}
 
 		/* get next info either from lib or cache */
-		if( lib )
+		if( lib ) {
 		  info++;
-#ifdef USE_CACHED_CATALOG  
-		else
+		}
+		else {
 		  info = _get_cached_plugin ( this->plugin_catalog->cache,
 			                      str, &statbuffer, info);
-#endif
-
+		}
 	      }
-
 	    }
 	    else {
 	      char *dl_error_msg = dlerror();
@@ -385,7 +382,7 @@ static void collect_plugins(xine_t *this, char *path){
 	  break;
 	case S_IFDIR:
 
-	  if (*pEntry->d_name != '.'){ /* catches ".", ".." or ".hidden" dirs */
+	  if (*pEntry->d_name != '.'){ /* unless ".", ".." or ".hidden" dirs */
 	    collect_plugins(this, str);
 	  }
 	} /* switch */
@@ -394,6 +391,11 @@ static void collect_plugins(xine_t *this, char *path){
     } /* while */
     closedir (dir);
   } /* if (dir) */
+  else {
+    xine_log (this, XINE_LOG_PLUGIN,
+	      _("load_plugins: skipping unreadable plugin directory %s.\n"),
+	      path);
+  }
 } /* collect_plugins */
 
 /*
@@ -441,47 +443,56 @@ static void *_load_plugin_class(xine_t *this,
 
 /*
  *  load input+demuxer plugins
+ *  load plugins that asked to be initialized
  */
-static void load_plugins(xine_t *this) {
+static void _load_required_plugins(xine_t *this, xine_list_t *list) {
 
   plugin_node_t *node;
+  int load;
 
-  /*
-   * input plugins
-   */
-
-  node = xine_list_first_content (this->plugin_catalog->input);
+  node = xine_list_first_content (list);
   while (node) {
-
+    
+    if( (node->info->type & PLUGIN_TYPE_MASK) == PLUGIN_INPUT ||
+        (node->info->type & PLUGIN_TYPE_MASK) == PLUGIN_DEMUX ||
+        (node->info->type & PLUGIN_MUST_PRELOAD) )
+      load = 1;
+    else
+      load = 0;
+    
+    if( load && !node->plugin_class ) {
 #ifdef LOG
-    printf("load_plugins: load input plugin %s from %s\n",
-	   node->info->id, node->filename);
+      printf("load_plugins: preload plugin %s from %s\n",
+	     node->info->id, node->filename);
 #endif
 
-    node->plugin_class = _load_plugin_class (this, node->filename, node->info, NULL);
-
-    node = xine_list_next_content (this->plugin_catalog->input);
-  }
-
-  /*
-   * demux plugins
-   */
-
-  node = xine_list_first_content (this->plugin_catalog->demux);
-  while (node) {
-
-#ifdef LOG
-    printf("load_plugins: load demux plugin %s from %s\n",
-	   node->info->id, node->filename);
-#endif
-
-    node->plugin_class = _load_plugin_class (this, node->filename, node->info, NULL);
-
-    node = xine_list_next_content (this->plugin_catalog->demux);
+      node->plugin_class = _load_plugin_class (this, node->filename, node->info, NULL);
+        
+      /* in case of failure remove from list */
+      if( !node->plugin_class ) {
+        xine_list_delete_current(list);
+        node = xine_list_prev_content(list); /* delete advances, so get previous */
+      }
+    }
+    
+    node = xine_list_next_content (list);
   }
 }
 
-#ifdef USE_CACHED_CATALOG  
+static void load_required_plugins(xine_t *this) {
+
+  _load_required_plugins (this, this->plugin_catalog->input);
+  _load_required_plugins (this, this->plugin_catalog->demux);
+  _load_required_plugins (this, this->plugin_catalog->spu);
+  _load_required_plugins (this, this->plugin_catalog->audio);
+  _load_required_plugins (this, this->plugin_catalog->video);
+  _load_required_plugins (this, this->plugin_catalog->aout);
+  _load_required_plugins (this, this->plugin_catalog->vout);
+  _load_required_plugins (this, this->plugin_catalog->post);
+}
+
+
+
 /*
  *  save plugin list information to file (cached catalog)
  */
@@ -491,6 +502,7 @@ static void save_plugin_list(FILE *fp, xine_list_t *plugins) {
   decoder_info_t *decoder_info;
   vo_info_t *vo_info;
   ao_info_t *ao_info;
+  post_info_t *post_info;
   
   int i;
 
@@ -506,7 +518,7 @@ static void save_plugin_list(FILE *fp, xine_list_t *plugins) {
     fprintf(fp, "id=%s\n", node->info->id );
     fprintf(fp, "version=%lu\n", (unsigned long) node->info->version );
   
-    switch (node->info->type){
+    switch (node->info->type & PLUGIN_TYPE_MASK){
     
       case PLUGIN_VIDEO_OUT:
         vo_info = node->info->special_info;
@@ -530,6 +542,11 @@ static void save_plugin_list(FILE *fp, xine_list_t *plugins) {
         fprintf(fp, "\n");
         fprintf(fp, "decoder_priority=%d\n", decoder_info->priority );
         break;
+      
+      case PLUGIN_POST:
+        post_info = node->info->special_info;
+	fprintf(fp, "post_type=%lu\n", (unsigned long)post_info->type);
+	break;
     }        
     
     fprintf(fp, "\n");
@@ -538,7 +555,7 @@ static void save_plugin_list(FILE *fp, xine_list_t *plugins) {
 }
 
 /*
- *  save plugin list information to file (cached catalog)
+ *  load plugin list information from file (cached catalog)
  */
 static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
 
@@ -546,6 +563,7 @@ static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
   decoder_info_t *decoder_info = NULL;
   vo_info_t *vo_info = NULL;
   ao_info_t *ao_info = NULL;
+  post_info_t *post_info = NULL;
   int i;
   unsigned long long llu;
   unsigned long lu;
@@ -568,12 +586,13 @@ static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
         xine_list_append_content (plugins, node);
       }
       node                = xine_xmalloc(sizeof(plugin_node_t));
-      node->filename      = _strclone(line+1);
+      node->filename      = strdup(line+1);
       node->info          = xine_xmalloc(2*sizeof(plugin_info_t));
       node->info[1].type  = PLUGIN_NONE;
       decoder_info        = NULL;
       vo_info             = NULL;
       ao_info             = NULL;
+      post_info           = NULL;
     }
 
     if ((value = strchr (line, '='))) {
@@ -600,7 +619,7 @@ static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
           sscanf(value," %d",&i);
           node->info->type = i;
           
-          switch (node->info->type){
+          switch (node->info->type & PLUGIN_TYPE_MASK){
           
             case PLUGIN_VIDEO_OUT:
               vo_info = node->info->special_info =
@@ -618,13 +637,18 @@ static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
               decoder_info = node->info->special_info =
                              xine_xmalloc(sizeof(decoder_info_t));
               break;
+	    
+	    case PLUGIN_POST:
+	      post_info = node->info->special_info =
+			  xine_xmalloc(sizeof(post_info_t));
+	      break;
           }        
           
         } else if( !strcmp("api",line) ) {
           sscanf(value," %d",&i);
           node->info->API = i;
         } else if( !strcmp("id",line) ) {
-          node->info->id = _strclone(value);
+          node->info->id = strdup(value);
         } else if( !strcmp("version",line) ) {
           sscanf(value," %lu",&lu);
           node->info->version = lu;
@@ -651,6 +675,9 @@ static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
         } else if( !strcmp("decoder_priority",line) && decoder_info ) {
           sscanf(value," %d",&i);
           decoder_info->priority = i;
+        } else if( !strcmp("post_type",line) && post_info ) {
+	  sscanf(value," %lu",&lu);
+	  post_info->type = lu;
         }
       }
     }
@@ -667,14 +694,22 @@ static void load_plugin_list(FILE *fp, xine_list_t *plugins) {
  */
 static void save_catalog (xine_t *this) {
 
-  FILE *fp;
-  char *cachefile;                                               
+  FILE       *fp;
+  char       *cachefile, *dirfile; 
   const char *relname = CACHE_CATALOG_FILE;
+  const char *dirname = CACHE_CATALOG_DIR;
     
   cachefile = (char *) xine_xmalloc(strlen(xine_get_homedir()) + 
                                     strlen(relname) + 3);
   sprintf(cachefile, "%s/%s", xine_get_homedir(), relname);
   
+  /* make sure homedir (~/.xine) exists */
+  dirfile = (char *) xine_xmalloc(strlen(xine_get_homedir()) + 
+				  strlen(dirname) + 3);
+  sprintf(dirfile, "%s/%s", xine_get_homedir(), dirname);
+  mkdir (dirfile, 0755);
+  free (dirfile);
+
   if( (fp = fopen(cachefile,"w")) != NULL ) {
   
     fprintf(fp, "# this file is automatically created by xine, do not edit.\n\n");
@@ -687,6 +722,7 @@ static void save_catalog (xine_t *this) {
     save_plugin_list (fp, this->plugin_catalog->video);
     save_plugin_list (fp, this->plugin_catalog->aout);
     save_plugin_list (fp, this->plugin_catalog->vout);
+    save_plugin_list (fp, this->plugin_catalog->post);
     fclose(fp);
   }
   free(cachefile);
@@ -711,7 +747,7 @@ static void load_cached_catalog (xine_t *this) {
   }
   free(cachefile);
 }
-#endif
+
 
 static void map_decoders (xine_t *this) {
 
@@ -876,11 +912,17 @@ static void map_decoders (xine_t *this) {
   }
 }
 
+
 /*
  *  initialize catalog, load all plugins into new catalog
  */
 void scan_plugins (xine_t *this) {
-
+  
+  const char *homedir;
+  char *plugindir;
+  char *pluginpath;
+  int i,j;
+  
 #ifdef LOG
   printf("load_plugins: scan_plugins()\n");
 #endif
@@ -890,20 +932,38 @@ void scan_plugins (xine_t *this) {
     abort();
   }
 
+  homedir = xine_get_homedir();
   this->plugin_catalog = _new_catalog();
-
-#ifdef USE_CACHED_CATALOG  
   load_cached_catalog (this);
-#endif
-    
-  /* TODO: add more plugin dir(s), maybe ~/.xine/plugins or /usr/local/... */
-  collect_plugins(this, XINE_PLUGINDIR);
 
-#ifdef USE_CACHED_CATALOG  
+  if ( !(pluginpath = getenv("XINE_PLUGIN_PATH")) ){
+    pluginpath = "~/.xine/plugins:" XINE_PLUGINDIR;
+  }
+  plugindir = xine_xmalloc(strlen(pluginpath)+strlen(homedir)+2);
+  j=0;
+  for (i=0; i <= strlen(pluginpath); ++i){
+    switch (pluginpath[i]){
+    case ':':
+    case '\0':
+      plugindir[j] = '\0';
+      collect_plugins(this, plugindir);
+      j = 0;
+      break;
+    case '~':
+      if (j == 0){
+	strcpy(plugindir, homedir);
+	j = strlen(plugindir);
+	break;
+      }
+    default:
+      plugindir[j++] = pluginpath[i];
+    }
+  }
+  free(plugindir);
+
   save_catalog (this);
-#endif
     
-  load_plugins (this);
+  load_required_plugins (this);
 
   map_decoders (this);
 }
@@ -971,8 +1031,9 @@ static demux_plugin_t *probe_demux (xine_stream_t *stream, int method1, int meth
 #ifdef LOG
       printf ("load_plugins: probing demux '%s'\n", node->info->id);
 #endif
-
       if ((plugin = ((demux_class_t *)node->plugin_class)->open_plugin(node->plugin_class, stream, input))) {
+	if (stream->xine->verbosity)
+	  printf ("load_plugins: using demuxer '%s'\n", node->info->id);
 	pthread_mutex_unlock (&catalog->lock);
 	return plugin;
       }
@@ -1040,30 +1101,31 @@ demux_plugin_t *find_demux_plugin_by_name(xine_stream_t *stream, const char *nam
 const char *const *xine_get_autoplay_input_plugin_ids(xine_t *this) {
 
   plugin_catalog_t   *catalog;
-  int                 i;
   plugin_node_t      *node;
 
   catalog = this->plugin_catalog;
 
   pthread_mutex_lock (&catalog->lock);
 
-  i = 0;
+  catalog->ids[0] = NULL;
   node = xine_list_first_content (catalog->input);
   while (node) {
     input_class_t *ic;
 
     ic = (input_class_t *) node->plugin_class;
     if (ic->get_autoplay_list) {
+      int i = 0, j;
+
+      while (catalog->ids[i] && strcmp(catalog->ids[i], node->info->id) < 0)
+        i++;
+      for (j = PLUGIN_MAX - 1; j > i; j--)
+        catalog->ids[j] = catalog->ids[j - 1];
 
       catalog->ids[i] = node->info->id;
-
-      i++;
     }
     node = xine_list_next_content (catalog->input);
   }
 
-  catalog->ids[i] = NULL;
-
   pthread_mutex_unlock (&catalog->lock);
 
   return catalog->ids;
@@ -1073,30 +1135,31 @@ const char *const *xine_get_browsable_input_plugin_ids(xine_t *this) {
 
 
   plugin_catalog_t   *catalog;
-  int                 i;
   plugin_node_t      *node;
 
   catalog = this->plugin_catalog;
 
   pthread_mutex_lock (&catalog->lock);
 
-  i = 0;
+  catalog->ids[0] = NULL;
   node = xine_list_first_content (catalog->input);
   while (node) {
     input_class_t *ic;
 
     ic = (input_class_t *) node->plugin_class;
     if (ic->get_dir) {
+      int i = 0, j;
+
+      while (catalog->ids[i] && strcmp(catalog->ids[i], node->info->id) < 0)
+        i++;
+      for (j = PLUGIN_MAX - 1; j > i; j--)
+        catalog->ids[j] = catalog->ids[j - 1];
 
       catalog->ids[i] = node->info->id;
-
-      i++;
     }
     node = xine_list_next_content (catalog->input);
   }
 
-  catalog->ids[i] = NULL;
-
   pthread_mutex_unlock (&catalog->lock);
 
   return catalog->ids;
@@ -1159,13 +1222,16 @@ xine_video_port_t *xine_open_video_driver (xine_t *this,
 					   int visual_type, void *visual) {
 
   plugin_node_t      *node;
-  vo_driver_t   *driver;
+  vo_driver_t        *driver;
   xine_video_port_t  *port;
   vo_info_t          *vo_info;
   plugin_catalog_t   *catalog = this->plugin_catalog;
 
   driver = NULL;
 
+  if (id && !strcasecmp(id, "auto"))
+    id = NULL;
+  
   pthread_mutex_lock (&catalog->lock);
 
   node = xine_list_first_content (catalog->vout);
@@ -1185,15 +1251,6 @@ xine_video_port_t *xine_open_video_driver (xine_t *this,
 
 	if (driver) {
 
-	  xine_cfg_entry_t entry;
-
-	  /* remember plugin id */
-
-	  if (xine_config_lookup_entry (this, "video.driver", &entry)) {
-	    entry.str_value = node->info->id;
-	    xine_config_update_entry (this, &entry);
-	  }
-
 	  break;
 	}
       }
@@ -1209,7 +1266,44 @@ xine_video_port_t *xine_open_video_driver (xine_t *this,
     return NULL;
   }
 
-  port = vo_new_port(this, driver);
+  port = vo_new_port(this, driver, 0);
+  
+  return port;
+}
+
+xine_video_port_t *xine_new_framegrab_video_port (xine_t *this) {
+
+  plugin_node_t      *node;
+  vo_driver_t        *driver;
+  xine_video_port_t  *port;
+  vo_info_t          *vo_info;
+  plugin_catalog_t   *catalog = this->plugin_catalog;
+  char               *id;
+
+  driver = NULL;
+  id     = "none";
+
+  pthread_mutex_lock (&catalog->lock);
+
+  node = xine_list_first_content (catalog->vout);
+  while (node) {
+
+    vo_info = node->info->special_info;
+    if (!strcasecmp (node->info->id, id)) {
+      driver = _load_video_driver (this, node, NULL);
+      break;
+    }
+    node = xine_list_next_content (catalog->vout);
+  }
+
+  pthread_mutex_unlock (&catalog->lock);
+
+  if (!driver) {
+    printf ("load_plugins: failed to load video output plugin <%s>\n", id);
+    return NULL;
+  }
+
+  port = vo_new_port(this, driver, 1);
   
   return port;
 }
@@ -1307,11 +1401,14 @@ xine_audio_port_t *xine_open_audio_driver (xine_t *this, const char *id,
 					   void *data) {
 
   plugin_node_t      *node;
-  ao_driver_t   *driver;
+  ao_driver_t        *driver;
   xine_audio_port_t  *port;
   ao_info_t          *ao_info;
   plugin_catalog_t   *catalog = this->plugin_catalog;
 
+  if (id && !strcasecmp(id, "auto") )
+    id = NULL;
+  
   pthread_mutex_lock (&catalog->lock);
 
   driver = NULL;
@@ -1330,15 +1427,6 @@ xine_audio_port_t *xine_open_audio_driver (xine_t *this, const char *id,
       driver = _load_audio_driver (this, node, data);
       if (driver) {
 
-	xine_cfg_entry_t entry;
-
-	/* remember plugin id */
-
-	if (xine_config_lookup_entry (this, "audio.driver", &entry)) {
-	  entry.str_value = node->info->id;
-	  xine_config_update_entry (this, &entry);
-	}
-
 	break;
       }
     }
@@ -1356,19 +1444,28 @@ xine_audio_port_t *xine_open_audio_driver (xine_t *this, const char *id,
     return NULL;
   }
 
-  port = ao_new_port(this, driver);
+  port = ao_new_port(this, driver, 0);
 
   return port;
 }
 
-void xine_close_audio_driver (xine_t *this, xine_audio_port_t  *driver) {
+xine_audio_port_t *xine_new_framegrab_audio_port (xine_t *this) {
 
-  /* FIXME : implement */
+  xine_audio_port_t  *port;
+
+  port = ao_new_port (this, NULL, 1);
+
+  return port;
+}
+
+void xine_close_audio_driver (xine_t *this, xine_audio_port_t  *ao_port) {
+
+  ao_port->exit(ao_port);
 
 }
-void xine_close_video_driver (xine_t *this, xine_video_port_t  *driver) {
+void xine_close_video_driver (xine_t *this, xine_video_port_t  *vo_port) {
 
-  /* FIXME : implement */
+  vo_port->exit(vo_port);
 
 }
 
@@ -1466,7 +1563,7 @@ video_decoder_t *get_video_decoder (xine_stream_t *stream, uint8_t stream_type)
       for (j = i + 1; j < PLUGINS_PER_TYPE; j++)
         catalog->video_decoder_map[stream_type][j - 1] =
           catalog->video_decoder_map[stream_type][j];
-      catalog->video_decoder_map[stream_type][PLUGINS_PER_TYPE] = NULL;
+      catalog->video_decoder_map[stream_type][PLUGINS_PER_TYPE-1] = NULL;
       i--;
       continue;
     }
@@ -1484,7 +1581,7 @@ video_decoder_t *get_video_decoder (xine_stream_t *stream, uint8_t stream_type)
       for (j = i + 1; j < PLUGINS_PER_TYPE; j++)
         catalog->video_decoder_map[stream_type][j - 1] =
           catalog->video_decoder_map[stream_type][j];
-      catalog->video_decoder_map[stream_type][PLUGINS_PER_TYPE] = NULL;
+      catalog->video_decoder_map[stream_type][PLUGINS_PER_TYPE-1] = NULL;
       i--;
     }
   }
@@ -1542,7 +1639,7 @@ audio_decoder_t *get_audio_decoder (xine_stream_t *stream, uint8_t stream_type)
       for (j = i + 1; j < PLUGINS_PER_TYPE; j++)
         catalog->audio_decoder_map[stream_type][j - 1] =
           catalog->audio_decoder_map[stream_type][j];
-      catalog->audio_decoder_map[stream_type][PLUGINS_PER_TYPE] = NULL;
+      catalog->audio_decoder_map[stream_type][PLUGINS_PER_TYPE-1] = NULL;
       i--;
       continue;
     }
@@ -1560,7 +1657,7 @@ audio_decoder_t *get_audio_decoder (xine_stream_t *stream, uint8_t stream_type)
       for (j = i + 1; j < PLUGINS_PER_TYPE; j++)
         catalog->audio_decoder_map[stream_type][j - 1] =
           catalog->audio_decoder_map[stream_type][j];
-      catalog->audio_decoder_map[stream_type][PLUGINS_PER_TYPE] = NULL;
+      catalog->audio_decoder_map[stream_type][PLUGINS_PER_TYPE-1] = NULL;
       i--;
     }
   }
@@ -1617,7 +1714,7 @@ spu_decoder_t *get_spu_decoder (xine_stream_t *stream, uint8_t stream_type) {
       for (j = i + 1; j < PLUGINS_PER_TYPE; j++)
         catalog->spu_decoder_map[stream_type][j - 1] =
           catalog->spu_decoder_map[stream_type][j];
-      catalog->spu_decoder_map[stream_type][PLUGINS_PER_TYPE] = NULL;
+      catalog->spu_decoder_map[stream_type][PLUGINS_PER_TYPE-1] = NULL;
       i--;
       continue;
     }
@@ -1635,7 +1732,7 @@ spu_decoder_t *get_spu_decoder (xine_stream_t *stream, uint8_t stream_type) {
       for (j = i + 1; j < PLUGINS_PER_TYPE; j++)
         catalog->spu_decoder_map[stream_type][j - 1] =
           catalog->spu_decoder_map[stream_type][j];
-      catalog->spu_decoder_map[stream_type][PLUGINS_PER_TYPE] = NULL;
+      catalog->spu_decoder_map[stream_type][PLUGINS_PER_TYPE-1] = NULL;
       i--;
     }
   }
@@ -1678,12 +1775,35 @@ const char *const *xine_list_post_plugins(xine_t *xine) {
   return catalog->ids;
 }
 
+const char *const *xine_list_post_plugins_typed(xine_t *xine, int type) {
+  plugin_catalog_t *catalog = xine->plugin_catalog;
+  plugin_node_t    *node;
+  int               i;
+  
+  pthread_mutex_lock (&catalog->lock);
+
+  i = 0;
+  node = xine_list_first_content (catalog->post);
+  while (node) {
+    if (((post_info_t *)node->info->special_info)->type == type)
+      catalog->ids[i++] = node->info->id;
+    node = xine_list_next_content (catalog->post);
+  }
+  catalog->ids[i] = NULL;
+
+  pthread_mutex_unlock (&catalog->lock);
+  return catalog->ids;
+}
+
 xine_post_t *xine_post_init(xine_t *xine, const char *name, int inputs,
 			    xine_audio_port_t **audio_target,
 			    xine_video_port_t **video_target) {
   plugin_catalog_t *catalog = xine->plugin_catalog;
   plugin_node_t    *node;
   
+  if( !name )
+    return NULL;
+  
   pthread_mutex_lock(&catalog->lock);
   
   node = xine_list_first_content(catalog->post);
@@ -1744,6 +1864,9 @@ xine_post_t *xine_post_init(xine_t *xine, const char *name, int inputs,
 	}
 	post->output_ids[i] = NULL;
 	
+	/* copy the post plugin type to the public part */
+	post->xine_post.type = ((post_info_t *)node->info->special_info)->type;
+	
 	return &post->xine_post;
       } else {
         printf("load_plugins: post plugin %s failed to instantiate itself\n", name);
@@ -1889,67 +2012,116 @@ char *xine_get_mime_types (xine_t *self) {
 }
 
 
+/* get the demuxer identifier that handles a given mime type
+ *
+ * the pointer returned can be free()ed when no longer used
+ * returns NULL if no demuxer is available to handle this. */
+char *xine_get_demux_for_mime_type (xine_t *self, const char *mime_type) {
+
+  plugin_catalog_t *catalog = self->plugin_catalog;
+  plugin_node_t    *node;
+  char             *id = NULL;
+  char             *mime_arg, *mime_demux;
+  char             *s;
+
+  /* create a copy and convert to lower case */  
+  mime_arg = strdup(mime_type);
+  for(s=mime_arg; *s; s++)
+    *s = tolower(*s);
+  
+  pthread_mutex_lock (&catalog->lock);
+
+  node = xine_list_first_content (catalog->demux);
+  while (node && !id) {
+    demux_class_t *cls = (demux_class_t *)node->plugin_class;
+
+    s = cls->get_mimetypes (cls);
+    if (s) {
+      mime_demux = strdup(s);
+      
+      for(s=mime_demux; *s; s++)
+        *s = tolower(*s);
+      
+      if( strstr(mime_demux, mime_arg) )
+        id = strdup(node->info->id);
+      
+      free(mime_demux);
+    }
+    node = xine_list_next_content (catalog->demux);
+  }
+
+  pthread_mutex_unlock (&catalog->lock);
+
+  free(mime_arg);
+  
+  return id;
+}
+
+
 static void dispose_plugin_list (xine_list_t *list) {
 
   plugin_node_t *node;
   decoder_info_t *decoder_info;
 
-  node = xine_list_first_content (list);
-  while (node) {
-    if (node->plugin_class) {
-      void *cls = node->plugin_class;
-
-      /* dispose of plugin class */
-      switch (node->info->type) {
-      case PLUGIN_INPUT:
-        ((input_class_t *)cls)->dispose ((input_class_t *)cls);
-        break;
-      case PLUGIN_DEMUX:
-        ((demux_class_t *)cls)->dispose ((demux_class_t *)cls);
-        break;
-      case PLUGIN_SPU_DECODER:
-        ((spu_decoder_class_t *)cls)->dispose ((spu_decoder_class_t *)cls);
-        break;
-      case PLUGIN_AUDIO_DECODER:
-        ((audio_decoder_class_t *)cls)->dispose ((audio_decoder_class_t *)cls);
-        break;
-      case PLUGIN_VIDEO_DECODER:
-        ((video_decoder_class_t *)cls)->dispose ((video_decoder_class_t *)cls);
-        break;
-      case PLUGIN_AUDIO_OUT:
-        ((audio_driver_class_t *)cls)->dispose ((audio_driver_class_t *)cls);
-        break;
-      case PLUGIN_VIDEO_OUT:
-        ((video_driver_class_t *)cls)->dispose ((video_driver_class_t *)cls);
-        break;
-      case PLUGIN_POST:
-        ((post_class_t *)cls)->dispose ((post_class_t *)cls);
-        break;
+  if(list) {
+    
+    node = xine_list_first_content (list);
+    while (node) {
+      if (node->plugin_class) {
+	void *cls = node->plugin_class;
+	
+	/* dispose of plugin class */
+	switch (node->info->type & PLUGIN_TYPE_MASK) {
+	case PLUGIN_INPUT:
+	  ((input_class_t *)cls)->dispose ((input_class_t *)cls);
+	  break;
+	case PLUGIN_DEMUX:
+	  ((demux_class_t *)cls)->dispose ((demux_class_t *)cls);
+	  break;
+	case PLUGIN_SPU_DECODER:
+	  ((spu_decoder_class_t *)cls)->dispose ((spu_decoder_class_t *)cls);
+	  break;
+	case PLUGIN_AUDIO_DECODER:
+	  ((audio_decoder_class_t *)cls)->dispose ((audio_decoder_class_t *)cls);
+	  break;
+	case PLUGIN_VIDEO_DECODER:
+	  ((video_decoder_class_t *)cls)->dispose ((video_decoder_class_t *)cls);
+	  break;
+	case PLUGIN_AUDIO_OUT:
+	  ((audio_driver_class_t *)cls)->dispose ((audio_driver_class_t *)cls);
+	  break;
+	case PLUGIN_VIDEO_OUT:
+	  ((video_driver_class_t *)cls)->dispose ((video_driver_class_t *)cls);
+	  break;
+	case PLUGIN_POST:
+	  ((post_class_t *)cls)->dispose ((post_class_t *)cls);
+	  break;
+	}
       }
+      
+      /* free special info */
+      switch (node->info->type & PLUGIN_TYPE_MASK) {
+      case PLUGIN_SPU_DECODER:
+      case PLUGIN_AUDIO_DECODER:
+      case PLUGIN_VIDEO_DECODER:
+	decoder_info = (decoder_info_t *)node->info->special_info;
+	free (decoder_info->supported_types);
+	
+      default:
+	free (node->info->special_info);
+	break;
+      }
+      
+      /* free info structure and string copies */
+      free (node->info->id);
+      free (node->info);
+      free (node->filename);
+      free (node);
+      
+      node = xine_list_next_content (list);
     }
-
-    /* free special info */
-    switch (node->info->type) {
-    case PLUGIN_SPU_DECODER:
-    case PLUGIN_AUDIO_DECODER:
-    case PLUGIN_VIDEO_DECODER:
-      decoder_info = (decoder_info_t *)node->info->special_info;
-      free (decoder_info->supported_types);
-
-    default:
-      free (node->info->special_info);
-      break;
-    }
-
-    /* free info structure and string copies */
-    free (node->info->id);
-    free (node->info);
-    free (node->filename);
-    free (node);
-
-    node = xine_list_next_content (list);
+    xine_list_free(list);
   }
-  xine_list_free(list);
 }
 
 
@@ -1959,15 +2131,17 @@ static void dispose_plugin_list (xine_list_t *list) {
 
 void dispose_plugins (xine_t *this) {
 
-  dispose_plugin_list (this->plugin_catalog->input);
-  dispose_plugin_list (this->plugin_catalog->demux);
-  dispose_plugin_list (this->plugin_catalog->spu);
-  dispose_plugin_list (this->plugin_catalog->audio);
-  dispose_plugin_list (this->plugin_catalog->video);
-  dispose_plugin_list (this->plugin_catalog->aout);
-  dispose_plugin_list (this->plugin_catalog->vout);
-  dispose_plugin_list (this->plugin_catalog->post);
-
-  dispose_plugin_list (this->plugin_catalog->cache);
-  free (this->plugin_catalog);
+  if(this->plugin_catalog) {
+    dispose_plugin_list (this->plugin_catalog->input);
+    dispose_plugin_list (this->plugin_catalog->demux);
+    dispose_plugin_list (this->plugin_catalog->spu);
+    dispose_plugin_list (this->plugin_catalog->audio);
+    dispose_plugin_list (this->plugin_catalog->video);
+    dispose_plugin_list (this->plugin_catalog->aout);
+    dispose_plugin_list (this->plugin_catalog->vout);
+    dispose_plugin_list (this->plugin_catalog->post);
+    
+    dispose_plugin_list (this->plugin_catalog->cache);
+    free (this->plugin_catalog);
+  }
 }
