@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2000-2002 the xine project
+ * Copyright (C) 2000-2003 the xine project
  * 
  * This file is part of xine, a free video player.
  * 
@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: video_out.c,v 1.117 2002/12/06 18:38:40 miguelfreitas Exp $
+ * $Id: video_out.c,v 1.138 2003/01/26 18:12:39 mroi Exp $
  *
  * frame allocation / queuing / scheduling / output functions
  */
@@ -36,6 +36,8 @@
 #include <pthread.h>
 #include <assert.h>
 
+#define XINE_ENABLE_EXPERIMENTAL_FEATURES
+
 #include "xine_internal.h"
 #include "video_out.h"
 #include "metronom.h"
@@ -63,6 +65,7 @@ typedef struct {
   vo_frame_t               *last_frame;
   vo_frame_t               *img_backup;
   int                       redraw_needed;
+  int                       flush_frames;
   
   int                       video_loop_running;
   int                       video_opened;
@@ -78,6 +81,14 @@ typedef struct {
 
   video_overlay_instance_t *overlay_source;
   int                       overlay_enabled;
+
+  /* do we true real-time output or is this a grab only instance ? */
+  int                       grab_only;
+
+  extra_info_t             *extra_info_base; /* used to free mem chunk */
+
+  int                       current_width, current_height;
+  int64_t                   current_duration;
 } vos_t;
 
 /*
@@ -269,6 +280,7 @@ static vo_frame_t *vo_get_frame (xine_video_port_t *this_gen,
   img->ratio          = ratio;
   img->format         = format;
   img->copy_called    = 0;
+  extra_info_reset ( img->extra_info );
   
   /* let driver ensure this image has the right format */
 
@@ -287,29 +299,47 @@ static vo_frame_t *vo_get_frame (xine_video_port_t *this_gen,
 static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
 
   vos_t         *this = (vos_t *) img->port;
+  extra_info_t   empty_info;
   int64_t        diff;
   int64_t        cur_vpts;
   int64_t        pic_vpts ;
   int            frames_to_skip;
 
+  img->stream = stream;
+  extra_info_reset(&empty_info);
+  if (memcmp(&empty_info, img->extra_info, sizeof(extra_info_t)) == 0)
+    /* the extra info is empty -> set it here */
+    extra_info_merge( img->extra_info, stream->video_decoder_extra_info );
+  this->current_width = img->width;
+  this->current_height = img->height;
+  
   stream->metronom->got_video_frame (stream->metronom, img);
+  this->current_duration = img->duration;
 
-  pic_vpts = img->vpts;
+  if (!this->grab_only) {
 
-  cur_vpts = this->clock->get_current_time(this->clock);
-  this->last_delivery_pts = cur_vpts;
+    pic_vpts = img->vpts;
+    img->extra_info->vpts = img->vpts;
+
+    cur_vpts = this->clock->get_current_time(this->clock);
+    this->last_delivery_pts = cur_vpts;
 
 #ifdef LOG
-  printf ("video_out: got image at master vpts %lld. vpts for picture is %lld (pts was %lld)\n",
-	  cur_vpts, pic_vpts, img->pts);
+    printf ("video_out: got image at master vpts %lld. vpts for picture is %lld (pts was %lld)\n",
+	    cur_vpts, pic_vpts, img->pts);
 #endif
 
-  this->num_frames_delivered++;
+    this->num_frames_delivered++;
 
-  diff = pic_vpts - cur_vpts;
-  frames_to_skip = ((-1 * diff) / img->duration + 3) * 2;
+    diff = pic_vpts - cur_vpts;
+    /* avoid division by zero */
+    if( img->duration <= 0 )
+      img->duration = 3000;
+    frames_to_skip = ((-1 * diff) / img->duration + 3) * 2;
 
-  if (frames_to_skip<0)
+    if (frames_to_skip<0)
+      frames_to_skip = 0;
+  } else
     frames_to_skip = 0;
 
 #ifdef LOG
@@ -328,16 +358,6 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
     if( !frames_to_skip && img->copy && !img->copy_called )
       vo_frame_driver_copy(img);
     
-    /*
-     * Wake up xine_play if it's waiting for a frame
-     */
-    pthread_mutex_lock (&stream->first_frame_lock);
-    if (stream->first_frame_flag) {
-      stream->first_frame_flag = 0;
-      pthread_cond_broadcast(&stream->first_frame_reached);
-    }
-    pthread_mutex_unlock (&stream->first_frame_lock);
-
     /*
      * put frame into FIFO-Buffer
      */
@@ -353,6 +373,9 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
 #ifdef LOG
     printf ("video_out: bad_frame\n");
 #endif
+    pthread_mutex_lock( &stream->current_extra_info_lock );
+    extra_info_merge( stream->current_extra_info, img->extra_info );
+    pthread_mutex_unlock( &stream->current_extra_info_lock );
 
     this->num_frames_skipped++;
   }
@@ -362,7 +385,7 @@ static int vo_frame_draw (vo_frame_t *img, xine_stream_t *stream) {
    */
 
   if ((this->num_frames_delivered % 200) == 0
-  	 && (this->num_frames_skipped || this->num_frames_discarded)) {
+  	&& (this->num_frames_skipped || this->num_frames_discarded)) {
     xine_log(this->xine, XINE_LOG_MSG,
 	     _("%d frames delivered, %d frames skipped, %d frames discarded\n"), 
 	     this->num_frames_delivered, 
@@ -440,8 +463,13 @@ static vo_frame_t * duplicate_frame( vos_t *this, vo_frame_t *img ) {
   dupl->bad_frame = 0;
   dupl->pts       = 0;
   dupl->vpts      = 0;
+  dupl->copy_called    = 0;
+
   dupl->duration  = img->duration;
 
+  dupl->stream    = img->stream;
+  memcpy( dupl->extra_info, img->extra_info, sizeof(extra_info_t) );
+  
   /* delay frame copying for now, we might not even need it (eg. frame will be discarded) */
   /* vo_frame_driver_copy(dupl); */
   
@@ -465,54 +493,69 @@ static void expire_frames (vos_t *this, int64_t cur_vpts) {
 
   diff = 1000000; /* always enter the while-loop */
 
-  while (img && (diff > img->duration)) {
+  while (img && (diff > img->duration || this->flush_frames)) {
     pts = img->vpts;
     diff = cur_vpts - pts;
       
-    if (diff > img->duration) {
-
-      /* do not print this message in stop/exit (scr is adjusted to force
-       * discarding audio and video frames)
-       */
-      if( diff < 20 * 90000 )
+    if (diff > img->duration || this->flush_frames) {
+  
+      if( !this->flush_frames ) {
         xine_log(this->xine, XINE_LOG_MSG,
 	         _("video_out: throwing away image with pts %lld because "
 		   "it's too old (diff : %lld).\n"), pts, diff);
 
-      this->num_frames_discarded++;
-
-      img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue);
-
-      /*
-       * last frame? back it up for 
-       * still frame creation
-       */
-
-      if (!this->display_img_buf_queue->first) {
-	  
-	if (this->img_backup) {
-#ifdef LOG
-	  printf("video_out: overwriting frame backup\n");
-#endif
-	  vo_frame_dec_lock( this->img_backup );
-	}
-#ifdef LOG
-	printf("video_out: possible still frame (old)\n");
-#endif
-
-	this->img_backup = img;
-	
-	/* wait 4 frames before drawing this one. 
-	   this allow slower systems to recover. */
-	this->redraw_needed = 4; 
-      } else {
-	vo_frame_dec_lock( img );
+        this->num_frames_discarded++;
       }
+      
+      img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue);
+  
+      pthread_mutex_lock( &img->stream->current_extra_info_lock );
+      extra_info_merge( img->stream->current_extra_info, img->extra_info );
+      pthread_mutex_unlock( &img->stream->current_extra_info_lock );
+
+      /* when flushing frames, keep the first one as backup */
+      if( this->flush_frames ) {
+        
+        if (!this->img_backup) {
+	  this->img_backup = img;
 	
+	  this->redraw_needed = 1;
+        } else {
+	  vo_frame_dec_lock( img );
+        }     
+           
+      } else {
+        /*
+         * last frame? back it up for 
+         * still frame creation
+         */
+        
+        if (!this->display_img_buf_queue->first) {
+	  
+	  if (this->img_backup) {
+#ifdef LOG
+	    printf("video_out: overwriting frame backup\n");
+#endif
+	    vo_frame_dec_lock( this->img_backup );
+	  }
+#ifdef LOG
+	  printf("video_out: possible still frame (old)\n");
+#endif
+
+	  this->img_backup = img;
+	
+	  /* wait 4 frames before drawing this one. 
+	     this allow slower systems to recover. */
+	  this->redraw_needed = 4; 
+        } else {
+	  vo_frame_dec_lock( img );
+        }
+      }
       img = this->display_img_buf_queue->first;
     }
   }
-
+  this->flush_frames = 0;
+  
   pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
 }
 
@@ -548,8 +591,11 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
       pthread_mutex_lock( &this->free_img_buf_queue->mutex );
       img = duplicate_frame (this, this->img_backup );
       pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
-      if( img )
+      if( img ) {
         img->vpts = cur_vpts;
+        /* extra info of the backup is thrown away, because it is not up to date */
+        extra_info_reset(img->extra_info);
+      }
         
       return img;
 
@@ -595,22 +641,13 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
      */
     pthread_mutex_lock( &this->free_img_buf_queue->mutex );
     if (img && !img->next) {
-      xine_stream_t *stream;
-      int backup_needed = 0;
+      
+      if (img->stream->stream_info[XINE_STREAM_INFO_VIDEO_HAS_STILL] ||
+          img->stream->video_fifo->size(img->stream->video_fifo) < 10) {
 
-      pthread_mutex_lock(&this->streams_lock);
-      for (stream = xine_list_first_content(this->streams); stream;
-           stream = xine_list_next_content(this->streams)) {
-        if (stream->stream_info[XINE_STREAM_INFO_VIDEO_HAS_STILL] ||
-            stream->video_fifo->size(stream->video_fifo) < 10)
-        backup_needed = 1;
-      }
-      pthread_mutex_unlock(&this->streams_lock);
-
-      if( backup_needed ) {
-/*#ifdef LOG*/
+#ifdef LOG
         printf ("video_out: possible still frame\n");
-/*#endif*/
+#endif
         this->img_backup = duplicate_frame (this, img);
       }
     }
@@ -629,6 +666,7 @@ static vo_frame_t *get_next_frame (vos_t *this, int64_t cur_vpts) {
 
 static void overlay_and_display_frame (vos_t *this, 
 				       vo_frame_t *img, int64_t vpts) {
+  xine_stream_t *stream;
 
 #ifdef LOG
   printf ("video_out: displaying image with vpts = %lld\n", 
@@ -640,6 +678,10 @@ static void overlay_and_display_frame (vos_t *this,
    */
   if( img->copy && !img->copy_called )
     vo_frame_driver_copy(img);
+  
+  pthread_mutex_lock( &img->stream->current_extra_info_lock );
+  extra_info_merge( img->stream->current_extra_info, img->extra_info );
+  pthread_mutex_unlock( &img->stream->current_extra_info_lock );
 
   if (this->overlay_source) {
     this->overlay_source->multiple_overlay_blend (this->overlay_source, 
@@ -657,6 +699,21 @@ static void overlay_and_display_frame (vos_t *this,
 
   this->driver->display_frame (this->driver, img);
   
+  /*
+   * Wake up xine_play if it's waiting for a frame
+   */
+  pthread_mutex_lock(&this->streams_lock);
+  for (stream = xine_list_first_content(this->streams); stream;
+       stream = xine_list_next_content(this->streams)) {
+    pthread_mutex_lock (&stream->first_frame_lock);
+    if (stream->first_frame_flag) {
+      stream->first_frame_flag = 0;
+      pthread_cond_broadcast(&stream->first_frame_reached);
+    }
+    pthread_mutex_unlock (&stream->first_frame_lock);
+  }
+  pthread_mutex_unlock(&this->streams_lock);
+
   this->redraw_needed = 0; 
 }
 
@@ -707,13 +764,17 @@ static void paused_loop( vos_t *this, int64_t vpts )
     if( this->redraw_needed && this->img_backup ) {
       img = duplicate_frame (this, this->img_backup );
       if( img ) {
+        /* extra info of the backup is thrown away, because it is not up to date */
+        extra_info_reset(img->extra_info);
         pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
         overlay_and_display_frame (this, img, vpts);
         pthread_mutex_lock( &this->free_img_buf_queue->mutex );
       }  
     }
     
+    pthread_mutex_unlock( &this->free_img_buf_queue->mutex );
     xine_usec_sleep (20000);
+    pthread_mutex_lock( &this->free_img_buf_queue->mutex );
   } 
   
   this->free_img_buf_queue->locked_for_read = 0;
@@ -730,12 +791,14 @@ static void *video_out_loop (void *this_gen) {
   vos_t             *this = (vos_t *) this_gen;
   int64_t            frame_duration, next_frame_vpts;
   int64_t            usec_to_sleep;
-
+ 
+  /* nice(-2); */
+    
   /*
    * here it is - the heart of xine (or rather: one of the hearts
    * of xine) : the video output loop
    */
-
+   
   frame_duration = 1500; /* default */
   next_frame_vpts = this->clock->get_current_time (this->clock);
 
@@ -784,18 +847,22 @@ static void *video_out_loop (void *this_gen) {
       pthread_mutex_lock(&this->streams_lock);
       for (stream = xine_list_first_content(this->streams); stream;
            stream = xine_list_next_content(this->streams)) {
-        if (stream->video_decoder_plugin) {
-
+        if (stream->video_decoder_plugin && stream->video_fifo) {
+          buf_element_t *buf;
+          
 #ifdef LOG
-	  printf ("video_out: flushing current video decoder plugin (%d %d)\n", 
-		  this->display_img_buf_queue->num_buffers,
-		  this->free_img_buf_queue->num_buffers);
+	  printf ("video_out: flushing current video decoder plugin\n");
 #endif
-	
-	  stream->video_decoder_plugin->flush(stream->video_decoder_plugin);
+          
+          buf = stream->video_fifo->buffer_pool_try_alloc (stream->video_fifo);
+          if( buf ) {
+            buf->type = BUF_CONTROL_FLUSH_DECODER;
+            stream->video_fifo->put(stream->video_fifo, buf);
+          }
         }
       }
       pthread_mutex_unlock(&this->streams_lock);
+
       this->last_delivery_pts = vpts;
     }
 
@@ -865,6 +932,91 @@ static void *video_out_loop (void *this_gen) {
   pthread_exit(NULL);
 }
 
+/*
+ * public function for video processing frontends to manually
+ * consume video frames
+ */
+
+int xine_get_next_video_frame (xine_video_port_t *this_gen,
+			       xine_video_frame_t *frame) {
+
+  vos_t         *this = (vos_t *) this_gen;
+  vo_frame_t    *img;
+  xine_stream_t *stream=NULL;
+
+  while (!stream) {
+    stream = xine_list_first_content(this->streams);
+    if (!stream)
+      xine_usec_sleep (1000);
+  } 
+
+  pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+  
+  img = this->display_img_buf_queue->first;
+
+  /* FIXME: ugly, use conditions and locks instead */
+
+  while (!img 
+	 && (stream->demux_plugin->get_status (stream->demux_plugin)==DEMUX_OK)) {
+
+    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+    xine_usec_sleep (1000);
+    pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+
+    img = this->display_img_buf_queue->first;
+  }
+
+  if (!img) {
+    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+    return 0;
+  }
+
+  /*
+   * remove frame from display queue and show it
+   */
+    
+  img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue);
+  pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+
+  frame->vpts       = img->vpts;
+  frame->duration   = img->duration;
+  frame->width      = img->width;
+  frame->height     = img->height;
+
+  switch (img->ratio) {
+  case XINE_VO_ASPECT_ANAMORPHIC:  /* anamorphic     */
+  case XINE_VO_ASPECT_PAN_SCAN:    /* we display pan&scan as widescreen */
+    frame->aspect_ratio = 16.0 /9.0;
+    break;
+  case XINE_VO_ASPECT_DVB:         /* 2.11:1 */
+    frame->aspect_ratio = 2.11/1.0;
+    break;
+  case XINE_VO_ASPECT_SQUARE:      /* square pels */
+  case XINE_VO_ASPECT_DONT_TOUCH:  /* probably non-mpeg stream => don't touch aspect ratio */
+    frame->aspect_ratio = (double) img->width / (double) img->height;
+    break;
+  case 0:                          /* forbidden -> 4:3 */
+  default:
+  case XINE_VO_ASPECT_4_3:         /* 4:3             */
+    frame->aspect_ratio = 4.0 / 3.0;
+    break;
+  }
+  frame->colorspace = img->format;
+  frame->data       = img->base[0];
+  frame->xine_frame = img;
+
+  return 1;
+}
+
+void xine_free_video_frame (xine_video_port_t *port, 
+			    xine_video_frame_t *frame) {
+
+  vo_frame_t *img = (vo_frame_t *) frame->xine_frame;
+
+  vo_frame_dec_lock (img);
+}
+
+
 static uint32_t vo_get_capabilities (xine_video_port_t *this_gen) {
   vos_t      *this = (vos_t *) this_gen;
   return this->driver->get_capabilities (this->driver);
@@ -874,7 +1026,11 @@ static void vo_open (xine_video_port_t *this_gen, xine_stream_t *stream) {
 
   vos_t      *this = (vos_t *) this_gen;
 
+#ifdef LOG
+  printf("video_out: vo_open\n");
+#endif
   this->video_opened = 1;
+  this->flush_frames = 0;
   this->last_delivery_pts = 0;
   if (!this->overlay_enabled && stream->spu_channel_user > -2)
     /* enable overlays if our new stream might want to show some */
@@ -906,6 +1062,29 @@ static void vo_close (xine_video_port_t *this_gen, xine_stream_t *stream) {
   pthread_mutex_unlock(&this->streams_lock);
 }
 
+static int vo_status (xine_video_port_t *this_gen, xine_stream_t *stream,
+                      int *width, int *height, int64_t *img_duration) {
+
+  vos_t      *this = (vos_t *) this_gen;
+  xine_stream_t *cur;
+  int ret = 0;
+
+  pthread_mutex_lock(&this->streams_lock);
+  for (cur = xine_list_first_content(this->streams); cur;
+       cur = xine_list_next_content(this->streams))
+    if (cur == stream || !stream) {
+      *width = this->current_width;
+      *height = this->current_height;
+      *img_duration = this->current_duration;
+      ret = 1;
+      break;
+    }
+  pthread_mutex_unlock(&this->streams_lock);
+  
+  return ret;
+}
+
+
 static void vo_free_img_buffers (xine_video_port_t *this_gen) {
   vos_t      *this = (vos_t *) this_gen;
   vo_frame_t *img;
@@ -919,6 +1098,8 @@ static void vo_free_img_buffers (xine_video_port_t *this_gen) {
     img = vo_remove_from_img_buf_queue (this->display_img_buf_queue) ;
     img->dispose (img);
   }
+
+  free (this->extra_info_base);
 }
 
 static void vo_exit (xine_video_port_t *this_gen) {
@@ -999,32 +1180,21 @@ static void vo_enable_overlay (xine_video_port_t *this_gen, int overlay_enabled)
  */
 static void vo_flush (xine_video_port_t *this_gen) {
   vos_t      *this = (vos_t *) this_gen;
-  vo_frame_t *img, *first;
-  int        i, num_buffers;
 
-  pthread_mutex_lock (&this->display_img_buf_queue->mutex);
-  num_buffers = this->display_img_buf_queue->num_buffers;
-  /* we keep the first frame so that we have something to display when
-   * seeking wildly around, but we schedule it in the past, so that it
-   * will expire immediately and does not block anything */
-  first = this->display_img_buf_queue->first;
-  if (first) {
-    vo_frame_inc_lock(first);
-    first->vpts -= 30 * 90000;
+  if( this->video_loop_running ) {
+    pthread_mutex_lock(&this->display_img_buf_queue->mutex);
+    this->flush_frames = 1;
+    pthread_mutex_unlock(&this->display_img_buf_queue->mutex);
+   
+    /* do not try this in paused mode */
+    while(this->flush_frames)
+      xine_usec_sleep (20000); /* pthread_cond_t could be used here */
   }
-  printf ("video_out: flush fifo (%d buffers)\n", num_buffers);
-  for (i = 0; i < num_buffers; i++) {
-    img = vo_remove_from_img_buf_queue_int (this->display_img_buf_queue);
-    vo_frame_dec_lock( img );
-  }
-  if (first)
-    vo_append_to_img_buf_queue_int(this->display_img_buf_queue, first);
-  this->redraw_needed = 1;
-  pthread_mutex_unlock (&this->display_img_buf_queue->mutex);
 }
 
 
-xine_video_port_t *vo_new_port (xine_t *xine, vo_driver_t *driver) {
+xine_video_port_t *vo_new_port (xine_t *xine, vo_driver_t *driver,
+				int grabonly) {
 
   vos_t            *this;
   int               i;
@@ -1051,6 +1221,7 @@ xine_video_port_t *vo_new_port (xine_t *xine, vo_driver_t *driver) {
   this->vo.enable_ovl            = vo_enable_overlay;
   this->vo.get_overlay_instance  = vo_get_overlay_instance;
   this->vo.flush                 = vo_flush;
+  this->vo.status                = vo_status;
   this->vo.driver                = driver;
 
   this->num_frames_delivered  = 0;
@@ -1074,10 +1245,14 @@ xine_video_port_t *vo_new_port (xine_t *xine, vo_driver_t *driver) {
   else if (num_frame_buffers<5) 
     num_frame_buffers = 5;
 
+  this->extra_info_base = calloc (num_frame_buffers,
+					  sizeof(extra_info_t));
+
   for (i=0; i<num_frame_buffers; i++) {
     vo_frame_t *img;
 
     img = driver->alloc_frame (driver) ;
+    if (!img) break;
 
     img->id        = i;
     
@@ -1086,35 +1261,46 @@ xine_video_port_t *vo_new_port (xine_t *xine, vo_driver_t *driver) {
     img->displayed = vo_frame_dec_lock;
     img->draw      = vo_frame_draw;
 
+    img->extra_info = &this->extra_info_base[i];
+
     vo_append_to_img_buf_queue (this->free_img_buf_queue,
 				img);
   }
 
+  if (grabonly) {
 
-  /*
-   * start video output thread
-   *
-   * this thread will alwys be running, displaying the
-   * logo when "idle" thus making it possible to have
-   * osd when not playing a stream
-   */
+    this->video_loop_running   = 0;
+    this->video_opened         = 0;
+    this->grab_only            = 1;
 
-  this->video_loop_running   = 1;
-  this->video_opened         = 0;
-  
-  pthread_attr_init(&pth_attrs);
-  pthread_attr_setscope(&pth_attrs, PTHREAD_SCOPE_SYSTEM);
+  } else {
 
-  if ((err = pthread_create (&this->video_thread,
-			     &pth_attrs, video_out_loop, this)) != 0) {
+    /*
+     * start video output thread
+     *
+     * this thread will alwys be running, displaying the
+     * logo when "idle" thus making it possible to have
+     * osd when not playing a stream
+     */
 
-    printf (_("video_out: can't create thread (%s)\n"), 
-	    strerror(err));
-    /* FIXME: how does this happen ? */
-    printf (_("video_out: sorry, this should not happen. please restart xine.\n"));
-    abort();
-  } else
-    printf ("video_out: thread created\n");
+    this->video_loop_running   = 1;
+    this->video_opened         = 0;
+    this->grab_only            = 0;
+    
+    pthread_attr_init(&pth_attrs);
+    pthread_attr_setscope(&pth_attrs, PTHREAD_SCOPE_SYSTEM);
+    
+    if ((err = pthread_create (&this->video_thread,
+			       &pth_attrs, video_out_loop, this)) != 0) {
+
+      printf (_("video_out: can't create thread (%s)\n"), 
+	      strerror(err));
+      /* FIXME: how does this happen ? */
+      printf (_("video_out: sorry, this should not happen. please restart xine.\n"));
+      abort();
+    } else
+      printf ("video_out: thread created\n");
+  }
 
   return &this->vo;
 }
