@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine.c,v 1.199 2002/12/08 21:43:52 miguelfreitas Exp $
+ * $Id: xine.c,v 1.225 2003/01/27 21:43:18 mroi Exp $
  *
  * top-level xine functions
  *
@@ -29,6 +29,7 @@
 
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/time.h>
 #include <fcntl.h>
 #include <string.h>
 #include <stdlib.h>
@@ -86,18 +87,30 @@ void xine_handle_stream_end (xine_stream_t *stream, int non_user) {
   }
 }
 
-void xine_report_codec (xine_stream_t *stream, int codec_type, 
-			uint32_t fourcc, uint32_t buf_type, int handled) {
-
-  if (codec_type == XINE_CODEC_VIDEO) {
-    stream->stream_info[XINE_STREAM_INFO_VIDEO_FOURCC]  = fourcc;
-    stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = handled;
-  } else {
-    stream->stream_info[XINE_STREAM_INFO_AUDIO_FOURCC]  = fourcc;
-    stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = handled;
-  }
+void extra_info_reset( extra_info_t *extra_info ) {
+  memset( extra_info, 0, sizeof(extra_info_t) );  
 }
 
+void extra_info_merge( extra_info_t *dst, extra_info_t *src ) {
+  
+  if( src->input_pos )
+    dst->input_pos = src->input_pos;
+  
+  if( src->input_length )
+    dst->input_length = src->input_length;
+    
+  if( src->input_time )
+    dst->input_time = src->input_time;
+    
+  if( src->frame_number )
+    dst->frame_number = src->frame_number;
+    
+  if( src->seek_count )
+    dst->seek_count = src->seek_count;
+
+  if( src->vpts )
+    dst->vpts = src->vpts;
+}
 
 static void xine_set_speed_internal (xine_stream_t *stream, int speed) {
 
@@ -105,15 +118,15 @@ static void xine_set_speed_internal (xine_stream_t *stream, int speed) {
 
   /* see coment on audio_out loop about audio_paused */
   if( stream->audio_out ) {
-    stream->audio_out->audio_paused = (speed != XINE_SPEED_NORMAL) + 
-      (speed == XINE_SPEED_PAUSE);
+    stream->audio_out->set_property( stream->audio_out, AO_PROP_PAUSED,
+      (speed != XINE_SPEED_NORMAL) + (speed == XINE_SPEED_PAUSE) );
 
     /*
      * slow motion / fast forward does not play sound, drop buffered
      * samples from the sound driver
      */
     if (speed != XINE_SPEED_NORMAL && speed != XINE_SPEED_PAUSE)
-      stream->audio_out->flush(stream->audio_out);
+      stream->audio_out->control (stream->audio_out, AO_CTRL_FLUSH_BUFFERS);
 
     stream->audio_out->control(stream->audio_out,
 			       speed == XINE_SPEED_PAUSE ? AO_CTRL_PLAY_PAUSE : AO_CTRL_PLAY_RESUME);
@@ -205,6 +218,9 @@ void xine_stop (xine_stream_t *stream) {
   if (stream->video_out)
     stream->video_out->flush(stream->video_out);
   
+  if (stream->slave && (stream->slave_affection & XINE_MASTER_SLAVE_STOP))
+    xine_stop(stream->slave);
+  
   pthread_mutex_unlock (&stream->frontend_lock);
 }
 
@@ -213,6 +229,15 @@ static void xine_close_internal (xine_stream_t *stream) {
 
   int i ;
 
+  if( stream->slave ) {
+    xine_close( stream->slave );
+    if( stream->slave_is_subtitle ) {
+      xine_dispose(stream->slave);
+      stream->slave = NULL;
+      stream->slave_is_subtitle = 0;
+    }
+  }
+
   xine_stop_internal( stream );
   
 #ifdef LOG
@@ -254,6 +279,49 @@ void xine_close (xine_stream_t *stream) {
   pthread_mutex_unlock (&stream->frontend_lock);
 }
 
+static int xine_stream_rewire_audio(xine_post_out_t *output, void *data)
+{
+  xine_stream_t *stream = (xine_stream_t *)output->data;
+  xine_audio_port_t *new_port = (xine_audio_port_t *)data;
+  uint32_t bits, rate;
+  int mode;
+  
+  if (!data)
+    return 0;
+    
+  if (stream->audio_out && 
+      stream->audio_out->status(stream->audio_out, stream, &bits, &rate, &mode)) {
+    /* register our stream at the new output port */
+    stream->audio_out->close(stream->audio_out, stream);
+    new_port->open(new_port, stream, bits, rate, mode);
+  }
+  /* reconnect ourselves */
+  stream->audio_out = new_port;
+  return 1;
+}
+
+static int xine_stream_rewire_video(xine_post_out_t *output, void *data)
+{
+  xine_stream_t *stream = (xine_stream_t *)output->data;
+  xine_video_port_t *new_port = (xine_video_port_t *)data;
+  int width, height;
+  int64_t img_duration;
+
+  if (!data)
+    return 0;
+    
+  if (stream->video_out && 
+      stream->video_out->status(stream->video_out, stream, 
+                                &width, &height, &img_duration)) {
+    /* register our stream at the new output port */
+    stream->video_out->close(stream->video_out, stream);
+    new_port->open(new_port, stream);
+  }
+  /* reconnect ourselves */
+  stream->video_out = new_port;
+  return 1;
+}
+
 
 xine_stream_t *xine_stream_new (xine_t *this, 
 				xine_audio_port_t *ao, xine_video_port_t *vo) {
@@ -268,8 +336,14 @@ xine_stream_t *xine_stream_new (xine_t *this,
    */
 
   pthread_mutex_lock (&this->streams_lock);
-
+  
   stream = (xine_stream_t *) xine_xmalloc (sizeof (xine_stream_t)) ;
+  stream->current_extra_info       = malloc( sizeof( extra_info_t ) );
+  stream->audio_decoder_extra_info = malloc( sizeof( extra_info_t ) );
+  stream->video_decoder_extra_info = malloc( sizeof( extra_info_t ) );
+  extra_info_reset( stream->current_extra_info );
+  extra_info_reset( stream->video_decoder_extra_info );
+  extra_info_reset( stream->audio_decoder_extra_info );
 
   stream->xine                   = this;
   stream->status                 = XINE_STATUS_STOP;
@@ -277,15 +351,12 @@ xine_stream_t *xine_stream_new (xine_t *this,
     stream->stream_info[i]       = 0;
     stream->meta_info[i]         = NULL;
   }
-  stream->input_pos              = 0;
-  stream->input_length           = 0;
-  stream->input_time             = 0;
   stream->spu_out                = NULL;
   stream->spu_decoder_plugin     = NULL;
   stream->spu_decoder_streamtype = -1;
   stream->audio_out              = ao;
   stream->audio_channel_user     = -1;
-  stream->audio_channel_auto     = 0;
+  stream->audio_channel_auto     = -1;
   stream->audio_decoder_plugin   = NULL;
   stream->audio_decoder_streamtype = -1;
   stream->spu_channel_auto       = -1;
@@ -303,7 +374,14 @@ xine_stream_t *xine_stream_new (xine_t *this,
   stream->finished_count_audio   = 0; 
   stream->finished_count_video   = 0; 
   stream->err                    = 0;
-
+  
+  /*
+   * initial master/slave
+   */
+  stream->master                 = stream;
+  stream->slave                  = NULL;
+  stream->slave_is_subtitle      = 0;
+  
   /*
    * init mutexes and conditions
    */
@@ -316,6 +394,7 @@ xine_stream_t *xine_stream_new (xine_t *this,
   pthread_cond_init  (&stream->counter_changed, NULL);
   pthread_mutex_init (&stream->first_frame_lock, NULL);
   pthread_cond_init  (&stream->first_frame_reached, NULL);
+  pthread_mutex_init (&stream->current_extra_info_lock, NULL);
 
   /*
    * event queues
@@ -351,13 +430,38 @@ xine_stream_t *xine_stream_new (xine_t *this,
 
   pthread_mutex_unlock (&this->streams_lock);
 
+  stream->video_source.name   = "video source";
+  stream->video_source.type   = XINE_POST_DATA_VIDEO;
+  stream->video_source.data   = stream;
+  stream->video_source.rewire = xine_stream_rewire_video;
+  
+  stream->audio_source.name   = "audio source";
+  stream->audio_source.type   = XINE_POST_DATA_AUDIO;
+  stream->audio_source.data   = stream;
+  stream->audio_source.rewire = xine_stream_rewire_audio;
+  
   return stream;
 }
 
+static void mrl_unescape(char *mrl) {
+  int i, len = strlen(mrl);
+
+  for (i = 0; i < len; i++) {
+    if ((mrl[i]=='%') && (i<(len-2))) {
+      int c;
+      
+      if (sscanf(&mrl[i + 1], "%02x", &c) == 1) {
+	mrl[i]= (char)c;
+	memmove(mrl + i + 1, mrl + i + 3, len - i - 3);
+	len -= 2;
+      }
+    }
+  }
+  mrl[len] = 0;
+}
+
 static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
 
-  int header_count_audio;
-  int header_count_video;
   char *stream_setup;
 
 #ifdef LOG
@@ -369,7 +473,6 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
    */
 
   xine_close_internal (stream);
-  stream->input_length = 0;
 
 #ifdef LOG
   printf ("xine: engine should be stopped now\n");
@@ -423,6 +526,7 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
 	    memcpy(demux_name, tmp, strlen(tmp));
 	    demux_name[strlen(tmp)] = '\0';
 	  }
+	  mrl_unescape(demux_name);
 	  if (!(stream->demux_plugin = find_demux_plugin_by_name(stream, demux_name, stream->input_plugin))) {
 	    xine_log(stream->xine, XINE_LOG_MSG,
 	      _("xine: specified demuxer %s failed to start\n"), demux_name);
@@ -499,6 +603,7 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
 	    memcpy(volume, tmp, strlen(tmp));
 	    volume[strlen(tmp)] = '\0';
 	  }
+	  mrl_unescape(volume);
 	  xine_set_param(stream, XINE_PARAM_AUDIO_VOLUME, atoi(volume));
 	  free(volume);
 	} else {
@@ -523,6 +628,7 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
 	    memcpy(compression, tmp, strlen(tmp));
 	    compression[strlen(tmp)] = '\0';
 	  }
+	  mrl_unescape(compression);
 	  xine_set_param(stream, XINE_PARAM_AUDIO_COMPR_LEVEL, atoi(compression));
 	  free(compression);
 	} else {
@@ -533,18 +639,57 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
 	}
 	continue;
       }
+      if (strncasecmp(stream_setup, "subtitle", 8) == 0) {
+        if (*(stream_setup += 8) == ':') {
+	  char *tmp = ++stream_setup;
+	  char *subtitle_mrl;
+	  stream_setup = strchr(stream_setup, ';');
+	  if (stream_setup) {
+	    subtitle_mrl = (char *)malloc(stream_setup - tmp + 1);
+	    memcpy(subtitle_mrl, tmp, stream_setup - tmp);
+	    subtitle_mrl[stream_setup - tmp] = '\0';
+	  } else {
+	    subtitle_mrl = (char *)malloc(strlen(tmp));
+	    memcpy(subtitle_mrl, tmp, strlen(tmp));
+	    subtitle_mrl[strlen(tmp)] = '\0';
+	  }
+	  mrl_unescape(subtitle_mrl);
+	  stream->slave = xine_stream_new (stream->xine, NULL, stream->video_out );
+	  stream->slave_affection = XINE_MASTER_SLAVE_PLAY | XINE_MASTER_SLAVE_STOP;
+	  if( xine_open( stream->slave, subtitle_mrl ) ) {
+	    printf("xine: subtitle mrl opened\n");
+	    stream->slave->master = stream;
+	    stream->slave_is_subtitle = 1; 
+	  } else {
+	    printf("xine: error opening subtitle mrl\n");
+	    xine_dispose( stream->slave );
+	    stream->slave = NULL;
+	  }
+	  free(subtitle_mrl);
+	} else {
+	  printf("xine: error while parsing mrl\n");
+	  stream->err = XINE_ERROR_MALFORMED_MRL;
+	  stream->status = XINE_STATUS_STOP;
+	  return 0;
+	}
+	continue;
+      }
       {
         /* when we got here, the stream setup parameter must be a config entry */
 	char *tmp = stream_setup;
+	char *config_entry;
 	if ((stream_setup = strchr(stream_setup, ';'))) {
-	  char *config_entry = (char *)malloc(stream_setup - tmp + 1);
+	  config_entry = (char *)malloc(stream_setup - tmp + 1);
 	  memcpy(config_entry, tmp, stream_setup - tmp);
 	  config_entry[stream_setup - tmp] = '\0';
-	  xine_config_change_opt(stream->xine->config, config_entry);
-	  free(config_entry);
 	} else {
-	  xine_config_change_opt(stream->xine->config, tmp);
+	  config_entry = (char *)malloc(strlen(tmp));
+	  memcpy(config_entry, tmp, strlen(tmp));
+	  config_entry[strlen(tmp)] = '\0';
 	}
+	mrl_unescape(config_entry);
+	xine_config_change_opt(stream->xine->config, config_entry);
+	free(config_entry);
       }
     }
     
@@ -597,6 +742,19 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
       = strdup (stream->demux_plugin->demux_class->get_identifier(stream->demux_plugin->demux_class));
   }
 
+  printf("xine: found input plugin  : %s\n",stream->input_plugin->input_class->get_description(stream->input_plugin->input_class));
+  printf("xine: found demuxer plugin: %s\n",stream->demux_plugin->demux_class->get_description(stream->demux_plugin->demux_class));
+
+  extra_info_reset( stream->current_extra_info );
+  extra_info_reset( stream->video_decoder_extra_info );
+  extra_info_reset( stream->audio_decoder_extra_info );
+
+  /* assume handled for now. we will only know for sure after trying
+   * to init decoders (which should happen when headers are sent)
+   */
+  stream->stream_info[XINE_STREAM_INFO_VIDEO_HANDLED] = 1;
+  stream->stream_info[XINE_STREAM_INFO_AUDIO_HANDLED] = 1;
+  
   /*
    * send and decode headers
    */
@@ -617,9 +775,11 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
     stream->err = XINE_ERROR_NO_DEMUX_PLUGIN;
 
     /* remove buffered samples from the sound device driver */
-    if (stream->audio_out)
+    /* why? */
+    /*if (stream->audio_out)
       stream->audio_out->control (stream->audio_out, AO_CTRL_FLUSH_BUFFERS);
-
+    */
+    
     stream->status = XINE_STATUS_STOP;
 
     printf ("xine: return from xine_open_internal\n");
@@ -627,24 +787,8 @@ static int xine_open_internal (xine_stream_t *stream, const char *mrl) {
     return 0;
   }
   
-  pthread_mutex_lock (&stream->counter_lock);
-  if (stream->audio_fifo)
-    header_count_audio = stream->header_count_audio + 1;
-  else
-    header_count_audio = 0;
-
-  header_count_video = stream->header_count_video + 1;
-  
   xine_demux_control_headers_done (stream);
 
-  while ((stream->header_count_audio<header_count_audio) || 
-	 (stream->header_count_video<header_count_video)) {
-    printf ("xine: waiting for headers.\n");
-    pthread_cond_wait (&stream->counter_changed, &stream->counter_lock);
-  }
-  printf ("xine: headers processed.\n");
-  pthread_mutex_unlock (&stream->counter_lock);
-
 #ifdef LOG
   printf ("xine: xine_open_internal done\n");
 #endif
@@ -662,7 +806,7 @@ int xine_open (xine_stream_t *stream, const char *mrl) {
 #endif
 
   ret = xine_open_internal (stream, mrl);
-
+  
   pthread_mutex_unlock (&stream->frontend_lock);
 
   return ret;
@@ -687,7 +831,12 @@ static int xine_play_internal (xine_stream_t *stream, int start_pos, int start_t
   pthread_mutex_lock (&stream->first_frame_lock);
   /* FIXME: howto detect if video frames will be produced */
   if (stream->first_frame_flag && stream->video_decoder_plugin) {
-    pthread_cond_wait(&stream->first_frame_reached, &stream->first_frame_lock);
+    struct timeval  tv;
+    struct timespec ts;
+    gettimeofday(&tv, NULL);
+    ts.tv_sec  = tv.tv_sec + 2;
+    ts.tv_nsec = tv.tv_usec * 1000;
+    pthread_cond_timedwait(&stream->first_frame_reached, &stream->first_frame_lock, &ts);
   }
   pthread_mutex_unlock (&stream->first_frame_lock);
 
@@ -695,8 +844,11 @@ static int xine_play_internal (xine_stream_t *stream, int start_pos, int start_t
    * start/seek demux
    */
   if (start_pos) {
+    pthread_mutex_lock( &stream->current_extra_info_lock );
+    len = stream->current_extra_info->input_length;
+    pthread_mutex_unlock( &stream->current_extra_info_lock );
     /* FIXME: do we need to protect concurrent access to input plugin here? */
-    len = stream->input_plugin->get_length (stream->input_plugin);
+    if (len == 0) len = stream->input_plugin->get_length (stream->input_plugin);
     share = (double) start_pos / 65535;
     pos = (off_t) (share * len) ;
   } else
@@ -729,10 +881,13 @@ static int xine_play_internal (xine_stream_t *stream, int start_pos, int start_t
     xine_demux_start_thread( stream );
     stream->status = XINE_STATUS_PLAY;
   }
-
+  
   pthread_mutex_lock (&stream->first_frame_lock);
   stream->first_frame_flag = 1;
   pthread_mutex_unlock (&stream->first_frame_lock);
+  pthread_mutex_lock( &stream->current_extra_info_lock );
+  extra_info_reset( stream->current_extra_info );
+  pthread_mutex_unlock( &stream->current_extra_info_lock );
 
   printf ("xine: xine_play_internal ...done\n");
 
@@ -746,6 +901,8 @@ int xine_play (xine_stream_t *stream, int start_pos, int start_time) {
   pthread_mutex_lock (&stream->frontend_lock);
 
   ret = xine_play_internal (stream, start_pos, start_time/1000);
+  if( stream->slave && (stream->slave_affection & XINE_MASTER_SLAVE_PLAY) )
+    xine_play (stream->slave, start_pos, start_time/1000);
 
   pthread_mutex_unlock (&stream->frontend_lock);
   
@@ -762,8 +919,10 @@ int xine_eject (xine_stream_t *stream) {
   pthread_mutex_lock (&stream->frontend_lock);
 
   status = 0;
-  if ((stream->status == XINE_STATUS_STOP)
-      && stream->eject_class && stream->eject_class->eject_media) {
+  /* only eject, if we are stopped OR a different input plugin is playing */
+  if (stream->eject_class && stream->eject_class->eject_media &&
+      ((stream->status == XINE_STATUS_STOP) ||
+      stream->eject_class != stream->input_plugin->input_class)) {
 
     status = stream->eject_class->eject_media (stream->eject_class);
   }
@@ -779,6 +938,13 @@ void xine_dispose (xine_stream_t *stream) {
   stream->status = XINE_STATUS_QUIT;
 
   xine_close(stream);
+  
+  if( stream->master != stream ) {
+    stream->master->slave = NULL;  
+  }
+  if( stream->slave && stream->slave->master == stream ) {
+    stream->slave->master = NULL;
+  }
 
   printf ("xine_exit: shutdown audio\n");
 
@@ -789,17 +955,20 @@ void xine_dispose (xine_stream_t *stream) {
   video_decoder_shutdown (stream);
 
   stream->osd_renderer->close( stream->osd_renderer );
-  stream->video_out->exit (stream->video_out);
   stream->video_fifo->dispose (stream->video_fifo);
 
   pthread_mutex_destroy (&stream->frontend_lock);
   pthread_mutex_destroy (&stream->counter_lock);
   pthread_mutex_destroy (&stream->osd_lock);
   pthread_mutex_destroy (&stream->event_queues_lock);
+  pthread_mutex_destroy (&stream->current_extra_info_lock);
   pthread_cond_destroy (&stream->counter_changed);
 
   stream->metronom->exit (stream->metronom);
 
+  free (stream->current_extra_info);
+  free (stream->video_decoder_extra_info);
+  free (stream->audio_decoder_extra_info);
   free (stream);
 }
 
@@ -813,8 +982,12 @@ void xine_exit (xine_t *this) {
     this->log_buffers[i]->dispose (this->log_buffers[i]);
 
   dispose_plugins (this);
-  this->clock->exit (this->clock);
-  this->config->dispose(this->config);
+
+  if(this->clock)
+    this->clock->exit (this->clock);
+  
+  if(this->config)
+    this->config->dispose(this->config);
 
   free (this);
 }
@@ -935,25 +1108,49 @@ static int xine_get_current_position (xine_stream_t *stream) {
 
   off_t len;
   double share;
-  
+    
   pthread_mutex_lock (&stream->frontend_lock);
 
   if (!stream->input_plugin) {
     printf ("xine: xine_get_current_position: no input source\n");
     pthread_mutex_unlock (&stream->frontend_lock);
-    return 0;
+    return -1;
+  }
+
+  if ( (!stream->video_decoder_plugin && !stream->audio_decoder_plugin) ) {
+    if( stream->stream_info[XINE_STREAM_INFO_HAS_VIDEO] )
+      extra_info_merge( stream->current_extra_info, stream->video_decoder_extra_info );
+    else    
+      extra_info_merge( stream->current_extra_info, stream->audio_decoder_extra_info );
   }
   
-  /* pos = stream->mCurInput->seek (0, SEEK_CUR); */
-  len = stream->input_length;
+  if ( stream->current_extra_info->seek_count != stream->video_seek_count ) {
+    pthread_mutex_unlock (&stream->frontend_lock);
+    return -1; /* position not yet known */  
+  }
+  
+  pthread_mutex_lock( &stream->current_extra_info_lock );
+  len = stream->current_extra_info->input_length;
+  share = (double) stream->current_extra_info->input_pos;
+  pthread_mutex_unlock( &stream->current_extra_info_lock );
+
   if (len == 0) len = stream->input_plugin->get_length (stream->input_plugin);
-  share = (double) stream->input_pos / (double) len * 65535;
+  share /= (double) len;
+  share *= 65536;
 
   pthread_mutex_unlock (&stream->frontend_lock);
 
   return (int) share;
 }
 
+void xine_get_current_info (xine_stream_t *stream, extra_info_t *extra_info, int size) {
+  
+  pthread_mutex_lock( &stream->current_extra_info_lock );
+  memcpy( extra_info, stream->current_extra_info, size );  
+  pthread_mutex_unlock( &stream->current_extra_info_lock );
+}
+
+
 int xine_get_status (xine_stream_t *stream) {
   return stream->status;
 }
@@ -1005,12 +1202,20 @@ static int xine_get_stream_length (xine_stream_t *stream) {
 int xine_get_pos_length (xine_stream_t *stream, int *pos_stream, 
 			 int *pos_time, int *length_time) {
   
+  int pos = xine_get_current_position (stream); /* force updating extra_info */
+  
+  if (pos == -1)
+    return 0;
+  
   if (pos_stream)
-    *pos_stream  = xine_get_current_position (stream); 
-  if (pos_time)
-    *pos_time    = stream->input_time * 1000;
+    *pos_stream  = pos; 
+  if (pos_time) {
+    pthread_mutex_lock( &stream->current_extra_info_lock );
+    *pos_time    = stream->current_extra_info->input_time;
+    pthread_mutex_unlock( &stream->current_extra_info_lock );
+  }
   if (length_time)
-    *length_time = xine_get_stream_length (stream) * 1000;
+    *length_time = xine_get_stream_length (stream);
 
   return 1;
 }
@@ -1199,3 +1404,14 @@ int xine_trick_mode (xine_stream_t *stream, int mode, int value) {
   abort ();
 }
 
+int xine_stream_master_slave(xine_stream_t *master, xine_stream_t *slave,
+                         int affection) {
+  master->slave = slave;
+  master->slave_affection = affection;
+  /* respect transitivity: if our designated master already has a master
+   * of its own, we point to this master's master; if our master is a 
+   * standalone stream, its master pointer will point to itself */
+  slave->master = master->master;
+  return 1;
+}                           
+
