@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xine_interface.c,v 1.32 2002/12/01 15:11:02 mroi Exp $
+ * $Id: xine_interface.c,v 1.38 2003/01/13 02:15:07 miguelfreitas Exp $
  *
  * convenience/abstraction layer, functions to implement
  * libxine's public interface
@@ -322,6 +322,10 @@ void xine_set_param (xine_stream_t *stream, int param, int value) {
   case XINE_PARAM_AV_OFFSET:
     stream->metronom->set_option (stream->metronom, METRONOM_AV_OFFSET, value);
     break;
+  
+  case XINE_PARAM_SPU_OFFSET:
+    stream->metronom->set_option (stream->metronom, METRONOM_SPU_OFFSET, value);
+    break;
 
   case XINE_PARAM_AUDIO_CHANNEL_LOGICAL:
     pthread_mutex_lock (&stream->frontend_lock);
@@ -402,6 +406,9 @@ int  xine_get_param (xine_stream_t *stream, int param) {
 
   case XINE_PARAM_AV_OFFSET:
     return stream->metronom->get_option (stream->metronom, METRONOM_AV_OFFSET);
+  
+  case XINE_PARAM_SPU_OFFSET:
+    return stream->metronom->get_option (stream->metronom, METRONOM_SPU_OFFSET);
 
   case XINE_PARAM_AUDIO_CHANNEL_LOGICAL:
     return stream->audio_channel_user;
@@ -506,8 +513,15 @@ uint32_t xine_get_stream_info (xine_stream_t *stream, int info) {
   case XINE_STREAM_INFO_IGNORE_VIDEO:
   case XINE_STREAM_INFO_IGNORE_AUDIO:
   case XINE_STREAM_INFO_IGNORE_SPU:
+  case XINE_STREAM_INFO_VIDEO_HAS_STILL:
     return stream->stream_info[info];
 
+  case XINE_STREAM_INFO_MAX_AUDIO_CHANNEL:
+    return stream->audio_track_map_entries;
+
+  case XINE_STREAM_INFO_MAX_SPU_CHANNEL:
+    return stream->spu_track_map_entries;
+
   default:
     printf ("xine_interface: error, unknown stream info (%d) requested\n",
 	    info);
@@ -626,7 +640,7 @@ const xine_post_out_t *xine_post_output(xine_post_t *this_gen, char *name) {
 }
 
 int xine_post_wire(xine_post_out_t *source, xine_post_in_t *target) {
-  if (source) {
+  if (source && source->rewire) {
     if (target) {
       if (source->type == target->type)
         return source->rewire(source, target->data);
@@ -637,3 +651,38 @@ int xine_post_wire(xine_post_out_t *source, xine_post_in_t *target) {
   }
   return 0;
 }
+
+int xine_post_wire_video_port(xine_post_out_t *source, xine_video_port_t *vo) {
+  if (source && source->rewire) {
+    if (vo) {
+      if (source->type == XINE_POST_DATA_VIDEO)
+        return source->rewire(source, vo);
+      else
+        return 0;
+    } else
+      return source->rewire(source, NULL);
+  }
+  return 0;
+}
+
+int xine_post_wire_audio_port(xine_post_out_t *source, xine_audio_port_t *ao) {
+  if (source && source->rewire) {
+    if (ao) {
+      if (source->type == XINE_POST_DATA_AUDIO)
+        return source->rewire(source, ao);
+      else
+        return 0;
+    } else
+      return source->rewire(source, NULL);
+  }
+  return 0;
+}
+
+xine_post_out_t * xine_get_video_source(xine_stream_t *stream) {
+  return &stream->video_source;  
+}
+
+xine_post_out_t * xine_get_audio_source(xine_stream_t *stream) {
+  return &stream->audio_source;
+}
+
