@@ -17,7 +17,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
- * $Id: xineutils.h,v 1.28 2002/12/04 05:33:40 tmmm Exp $
+ * $Id: xineutils.h,v 1.32 2002/12/31 19:30:53 esnel Exp $
  *
  */
 #ifndef XINEUTILS_H
@@ -35,6 +35,11 @@ extern "C" {
 #include "attributes.h"
 #include "compat.h"
 #include "xmlparser.h"
+#include "xine_buffer.h"
+
+#ifdef __SUNPRO_C
+#define inline
+#endif
 
   /*
    * debugable mutexes
@@ -678,7 +683,7 @@ static inline char *_x_strsep(char **stringp, const char *delim) {
 #endif
 
 
-#ifdef	HAVE_SETENV
+#ifdef HAVE_SETENV
 #define	xine_setenv	setenv
 #else
 static inline void _x_setenv(const char *name, const char *val, int _xx)
